#include "lemur_cli.hpp"

void getCLIConf(void)
{
    FILE  *pFile = NULL;
    CHAR  line[500], tag[20];
    int   vlaue;

    #ifdef DEBUG
        pFile = fopen ("Lemur_env.ini","r");
    #else
        pFile = fopen ("/etc/lemur/Lemur_env.ini", "r");
    #endif

    if (pFile != NULL)
    {
        while (fgets(line, 500, pFile) != NULL)
        {
            if (line[0] == '#')
            {
                continue;
            }

            if ( strlen(line) == 1 && line[0] == '\n' )
            {
                continue;
            }

            //Parse the tag:debug_level, pass_message
            sscanf(line, "%[^=]=%d", tag, &vlaue);
            if (strcmp(tag, "debug_level") == 0)
            {
                switch(vlaue)
                {
                    case 0:
                        g_eDebugLevel = DEBUG_LEVEL_0;
                        break;
                    case 1:
                        g_eDebugLevel = DEBUG_LEVEL_1;
                        break;
                    case 2:
                        g_eDebugLevel = DEBUG_LEVEL_2;
                        break;
                    default:
                        break;
                }
            }
            if (strcmp(tag, "pass_message") == 0)
            {
                switch(vlaue)
                {
                    case 0:
                        g_eResultMessage = RESULT_MESSAGE_OFF;
                        break;
                    case 1:
                        g_eResultMessage = RESULT_MESSAGE_ON;
                        break;
                    default:
                        break;
                }
            }
        }
        fclose(pFile);
    }
}

MODULE_TYPE convertStringToEnum(string module)
{
    if (module == "")          return NO_MODULE;
    if (module == "help")      return MODULE_HELP;
    if (module == "i2c")       return I2C_MODULE;
    if (module == "cpu")       return CPU_MODULE;
    if (module == "ethernet")  return ETHERNET_MODULE;
    if (module == "bbu")       return BBU_MODULE;
    if (module == "mem")       return MEMORY_MODULE;
    if (module == "cpu")       return PROCESSOR_MODULE;
    if (module == "pci")       return PCI_MODULE;
    if (module == "usb")       return USB_MODULE;
    if (module == "gpio")      return GPIO_MODULE;
    if (module == "storage")   return STORAGE_MODULE;
    //if (module == "sata")      return SATA_MODULE;
    if (module == "sas")       return SAS_MODULE;
    if (module == "nvsram")    return NVSRAM_MODULE;
    if (module == "temp")      return TEMPERATURE_MODULE;
    if (module == "uart")      return UART_MODULE;
    if (module == "rtc")       return RTC_MODULE;
    //if (module == "psu")       return PSU_MODULE;
    if (module == "tpm")       return TPM_MODULE;
    if (module == "led")       return LED_MODULE;
    if (module == "ntb")       return NTB_MODULE;
    if (module == "expander")  return EXPANDER_MODULE;
    if (module == "fpga")      return FPGA_MODULE;
    if (module == "nvme")      return NVMe_MODULE;
    //Add new module name here for enum

    return UNSUPPORT;
}


//Show supported module list procedure
void showSupportModule()
{
    int width = 15;

    cout << "Usage:     lemur ModuleName" << endl;
    cout << "Module Name Options:" << endl;

    //Add show supported module

    cout << "  " << setw(width) << setiosflags(ios::left) << "CPU" << "Processor" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "I2C" << "I2C" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "Ethernet" << "Ethernet" << endl;
    //cout << "  " << setw(width) << setiosflags(ios::left) << "BBU" << "Backup battery unit" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "PCI" << "PCI" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "USB" << "Universal Serial Bus" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "MEM" << "Memory" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "GPIO" << "General Purpose I/O" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "Storage" << "SAS/NVMe/USB Storage" << endl;
    //cout << "  " << setw(width) << setiosflags(ios::left) << "SATA" << "SATA" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "Temp" << "Temperature" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "UART" << "UART" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "SAS" << "Serial Attached SCSI" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "NVSRAM" << "NVSRAM" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "RTC" << "Real Time Clock" << endl;
    //cout << "  " << setw(width) << setiosflags(ios::left) << "PSU" << "Power Supply" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "TPM" << "Trusted Platform Module" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "LED" << "LED" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "NTB" << "NTB" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "Expander" << "Expander" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "FPGA" << "FPGA" << endl;
    cout << "  " << setw(width) << setiosflags(ios::left) << "NVMe" << "NVM Express" << endl;
    //Add new module help here

}

int main(int argc, char *argv[])
{
    StringParser    cli;
    int             error = 0;
    string          module;
    ModuleBaseClass *pModule = NULL;
    COMMAND_TYPE    eCommandType;

    //Initial command parameter
    cli.initialParser(argv, argc);
    //Get the module name from comman line
    cli.getModule(&module);

    getCLIConf();

    //Display result message when this is a test command
    cli.getCommandType(&eCommandType);
    if (eCommandType == CMD_TEST)
        g_eResultMessage = RESULT_MESSAGE_ON;

    //Create module object
    switch (convertStringToEnum(module))
    {
        case MODULE_HELP:
            showSupportModule();
            return 0;
        case I2C_MODULE:
            pModule = new I2C_module("I2C");
            break;
        case PCI_MODULE:
            pModule = new PCIModule("PCI");
            break;
        case USB_MODULE:
            pModule = new USBModule("USB");
            break;
        case MEMORY_MODULE:
            pModule = new MemoryModule("MEM");
            break;
        case GPIO_MODULE:
            pModule = new GPIOModule("GPIO");
            break;
        //case BBU_MODULE:
            //pModule = new BBUModule("BBU");
            //break;
        case STORAGE_MODULE:
            pModule = new StorageModule("STORAGE");
            break;
	/*case SATA_MODULE:
            pModule = new SATAModule("SATA");
            break;*/
        case SAS_MODULE:
            pModule = new SASModule("SAS");
            break;
        case NVSRAM_MODULE:
            pModule = new NVSRAMModule("NVSRAM");
            break;
        case TEMPERATURE_MODULE:
            pModule = new TemperatureModule("TEMPERATURE");
            break;
        case UART_MODULE:
            pModule = new UARTModule("UART");
            break;
        case ETHERNET_MODULE:
            pModule = new EthernetModule("ETHERNET");
            break;
        case RTC_MODULE:
            pModule = new RTCModule("RTC");
            break;
        /*
        case PSU_MODULE:
            pModule = new PSUModule("PSU");
            break;
        */
        case CPU_MODULE:
            pModule = new CPUModule("CPU");
            break;
        case TPM_MODULE:
            pModule = new TPMModule("TPM");
            break;
        case LED_MODULE:
            pModule = new LEDModule("LED");
            break;
        case NTB_MODULE:
            pModule = new NTBModule("NTB");
            break;
        case EXPANDER_MODULE:
            pModule = new ExpanderModule("EXPANDER");
            break;
        case FPGA_MODULE:
            pModule = new FPGAModule("FPGA");
            break;
        case NVMe_MODULE:
            pModule = new NVMeModule("NVMe");
            break;
        default:
            cout << "Error: Unknown module!" << endl;
            if (g_eResultMessage == RESULT_MESSAGE_ON)
                cout << "RESULT: FAILED" << endl;
            return -1;
    }

    if (pModule == NULL)
    {
        cout << "Error: Fail to create a module object!" << endl;
        if (g_eResultMessage == RESULT_MESSAGE_ON)
                        cout << "RESULT: FAILED" << endl;
        return -1;
    }

    error = pModule->findExecuteCommand(cli);
    if (g_eResultMessage == RESULT_MESSAGE_ON)
    {
        if (error != 0)
        {
            cout << "RESULT: FAILED" << endl;
        }
        else
        {
            cout << "RESULT: PASSED" << endl;
        }
    }

    //Delete module object
    if (pModule)
    {
        delete pModule;
        pModule = NULL;
    }

    return error;
}
