#include "CommandBaseClass.hpp"


CommandBaseClass::CommandBaseClass(COMMAND_TYPE command_type_id, string help_content)
{
    //Initial all value
    Initial_command_name(command_type_id);
    m_command_type = command_type_id;
    m_help_content = help_content;
    m_operation_count = 0;
    
    for (int i=0; i < OPERATION_NUMBER; i++)
    {
        m_operation[i] = NULL;
    } 
}



CommandBaseClass::~CommandBaseClass()
{
    int index;

    for (index = 0; index < m_operation_count; index++)
    {
        if (m_operation[index] != NULL)
        {
            delete m_operation[index];
        }
    }
}



int
CommandBaseClass::Initial_command_name(COMMAND_TYPE command_type_id)
{
    int err = 0;
    
    switch (command_type_id)
    {
        case CMD_LIST:
            m_command_name = "list";
            break;
        case CMD_SHOW:
            m_command_name = "show";
            break;
        case CMD_UTILITY:
            m_command_name = "utility";
            break;
        case CMD_TEST:
            m_command_name = "test";
            break;
        case CMD_ERROR:
            m_command_name = "error";
            break;
        case CMD_MAC:
            m_command_name = "mac";
            break;
        case CMD_EEPROM:
            m_command_name = "eeprom";
            break;
        case CMD_SFP:
            m_command_name = "sfp";
            break;
        case CMD_SET:
            m_command_name = "set";
            break;
        case CMD_ACCURACY:
            m_command_name = "accuracy";
            break;
        case CMD_FAN:
            m_command_name = "fan";
            break;
        case CMD_LED:
            m_command_name = "led";
            break;
        case CMD_TEMP:
            m_command_name = "temp";
            break;
        case CMD_FW:
            m_command_name = "fw";
            break;
        case CMD_WWN:
            m_command_name = "wwn";
            break;
        case CMD_FLASH:
            m_command_name = "flash";
            break;
        case CMD_RISC:
            m_command_name = "risc";
            break;
        case CMD_GUID:
            m_command_name = "guid";
            break;
        case CMD_CHARGE:
            m_command_name = "charge";
            break;
        case CMD_ECC:
            m_command_name = "ecc";
            break;
        case CMD_SPAD:
            m_command_name = "spad";
            break;
        case CMD_PROGRAM:
            m_command_name = "program";
            break;
        //Add new command below
        default:
            m_command_name = "no_command";
            err = -1;
            break;
    }

    return err;
}




int
CommandBaseClass::addOperation(OperationBaseClass *point_operation)
{
    if (point_operation != NULL)
    {
        m_operation[m_operation_count] = point_operation;
        m_operation_count++;
    }

    return 0;
}



int
CommandBaseClass::listOperation(StringParser command_line)
{
    int i, width = 20;
    string module, name, help_content;

    command_line.getModule(&module);
    getName(&name); //Get the command name
    cout << "Usage:	lemur " << module << " " << name <<" Operation" << endl;
    cout << "Operation Options:" << endl;
    for (i=0; i<m_operation_count; i++)
    {
        m_operation[i]->getName(&name);
        m_operation[i]->getHelp(&help_content);
   	    cout << "  " << setw(width) << setiosflags(ios::left) << name << help_content << endl;
    }

    return 0;
}



int
CommandBaseClass::getName(string *command_name)
{
    *command_name = m_command_name;
    return 0;
}



int
CommandBaseClass::getHelp(string *help_contet)
{
    *help_contet = m_help_content;
    return 0;
}



int
CommandBaseClass::getTypeID(COMMAND_TYPE *command_type_id)
{
    *command_type_id = m_command_type;
    return 0;
}



int
CommandBaseClass::executeCommand(StringParser command_line)
{
    return 0;
}

int
CommandBaseClass::findAndExecuteOperation(StringParser in_parser)
{
    string strOperation, strTemp;
    int i;

    in_parser.getOperation(&strOperation);

    if (strOperation == HELP)
    {
        listOperation(in_parser);
        return 0;
    }

    for (i = 0; i< m_operation_count; i++)
    {
        if (m_operation[i] == NULL)
            break;

        strTemp.clear();
        m_operation[i]->getName(&strTemp);
        if (strOperation == strTemp)
        {
            return m_operation[i]->executeOperation(in_parser);
        }
    }

    cout << UNKNOWN_OPERATION_MSG << endl;

    return -1;
}
