#include "CPUCommand.hpp"

CPUCommand::CPUCommand(COMMAND_TYPE CommandType, string HelpContent) : CommandBaseClass(CommandType, HelpContent)
{
    //Base on command type to add different operations
    switch (CommandType)
    {
        case CMD_SHOW:
            addOperation(new CPUOperation(OPERATION_ALL, "Display CPU all information"));
            addOperation(new CPUOperation(OPERATION_SHOW_CPU, "Show CPU information"));
            addOperation(new CPUOperation(OPERATION_SHOW_VENDOR, "Show vendor information"));
            addOperation(new CPUOperation(OPERATION_SHOW_MODEL, "Show model information"));
            addOperation(new CPUOperation(OPERATION_SHOW_FAMILY, "Show family information"));
            addOperation(new CPUOperation(OPERATION_SHOW_CORE, "Show core information"));
            addOperation(new CPUOperation(OPERATION_SHOW_FREQUENCY, "Show frequency information"));
            addOperation(new CPUOperation(OPERATION_SHOW_STEPPING, "Show stepping information"));
            addOperation(new CPUOperation(OPERATION_SHOW_CACHE, "Show cache information"));
            addOperation(new CPUOperation(OPERATION_SHOW_TEMPERATUR, "Show temperature information"));
            break;
        case CMD_TEST:
            addOperation(new CPUOperation(OPERATION_STABILITY, "calculating PI value and verified the value by checksum"));
            break;
        case CMD_UTILITY:
            addOperation(new CPUOperation(OPERATION_READ, "Read register operation of MSR(x86)"));
            addOperation(new CPUOperation(OPERATION_WRITE, "Write registers operation of MSR(x86)"));
            addOperation(new CPUOperation(OPERATION_UTILITY_GETMCE, "Get the machine check exception message"));
            break;
        //Add NEW command and operation here
        default:
            break;
    }
}



CPUCommand::~CPUCommand()
{

}



int
CPUCommand::List(StringParser command_line)
{
    ProcessorDAPI *pCPUDAPI = NULL;
    int           CPU_count = 0, i;
    ErrorMsg      *error_code = NULL;


    pCPUDAPI = new ProcessorDAPI("CPUDAPI");

    //Get CPU device number by API
    error_code = pCPUDAPI->Get_CPU_physical_number(&CPU_count);
    if (error_code != NO_ERROR_MSG)
    {
        printError(error_code);
        return -1;
    }

    for (i = 0; i < CPU_count; i++)
    {
        cout << "Processor" << i << endl;
    }

    return 0;
}



int
CPUCommand::Show(StringParser command_line)
{
   // cout << "I2C Show command execution" << endl;
    string parser_operation_name;
	  int i, err = 0;
	  string operation_name;

    command_line.getOperation(&parser_operation_name);

    //Check help or not operation than list supported operation
    if (parser_operation_name == HELP || parser_operation_name == "")
    {
        listOperation(command_line);
        return -1;
    }

    //Search for operation
    for (i = 0; i< m_operation_count; i++)
    {
        m_operation[i]->getName(&operation_name);
        if (operation_name == parser_operation_name)
        {
            err = m_operation[i]->executeOperation(command_line);
            break;
        }
    }

    if (i == m_operation_count)
    {
        cout << parser_operation_name << " operation not found" << endl;
        return -1;
    }
    else
    {
        return err;
    }

    return 0;
}



int
CPUCommand::Test(StringParser command_line)
{
    string parser_operation_name;
	  int i, err = 0;
	  string operation_name;
	  

    command_line.getOperation(&parser_operation_name);
    
    //Check help or not operation than list supported operation
    if (parser_operation_name == HELP || parser_operation_name == "")
    {
        listOperation(command_line);
        return -1;
    }

    //Search for operation
    for (i = 0; i< m_operation_count; i++)
    {
    	  m_operation[i]->getName(&operation_name);
        if (operation_name == parser_operation_name)
        {
            err = m_operation[i]->executeOperation(command_line);
            break;
        }
    }

    if (i == m_operation_count)
    {
        cout << parser_operation_name << " operation not found" << endl;
        return -1;
    }
    else
    {
        return err;
    }
}



int
CPUCommand::Utility(StringParser command_line)
{
    string parser_operation_name;
	  int i, err = 0;
	  string operation_name;

    command_line.getOperation(&parser_operation_name);

    //Check help or not operation than list supported operation
    if (parser_operation_name == HELP || parser_operation_name == "")
    {
        listOperation(command_line);
        return -1;
    }

    //Search for operation
    for (i = 0; i< m_operation_count; i++)
    {
        m_operation[i]->getName(&operation_name);
        if (operation_name == parser_operation_name)
        {
            err = m_operation[i]->executeOperation(command_line);
            break;
        }
    }

    if (i == m_operation_count)
    {
        cout << parser_operation_name << " operation not found" << endl;
        return -1;
    }
    else
    {
        return err;
    }
}



int
CPUCommand::executeCommand(StringParser command_line)
{
	  int err = 0;
	  COMMAND_TYPE command_type = NO_COMMAND;

    command_line.getCommandType(&command_type);

    switch (command_type)
    {
        case CMD_LIST:
            err = List(command_line);
            break;
        case CMD_SHOW:
            err = Show(command_line);
            break;
        case CMD_TEST:
            err = Test(command_line);
            break;
        case CMD_UTILITY:
            err = Utility(command_line);
            break;
        default:
            cout << "Not supported command" << endl;
            err = -1;
            break;
    }

    return err;
}

