#include "ExpanderCommand.hpp"
#include "CommonCLIFunctions.hpp"
#include "ExpanderDAPI.hpp"

ExpanderCommand::ExpanderCommand(COMMAND_TYPE commandType, string helpContent) : CommandBaseClass(commandType, helpContent)
{
    //Base on command type to add different operations
    switch (commandType)
    {
        case CMD_LIST:
            addOperation(new ExpanderOperation(OPERATION_CONTROLLER,        "Display Expander Information."));
            addOperation(new ExpanderOperation(OPERATION_PORT,              "Display Expander Port Information."));
            addOperation(new ExpanderOperation(OPERATION_PHY,               "Display Expander PHY Information."));
            break;
        case CMD_SHOW:
            addOperation(new ExpanderOperation(OPERATION_ALL,               "Display All Information."));
            addOperation(new ExpanderOperation(OPERATION_ATTACHED_TYPE,     "Display PHY Attached Devices Type."));
            addOperation(new ExpanderOperation(OPERATION_LINK,              "Display PHY Link Status."));
            addOperation(new ExpanderOperation(OPERATION_SPEED,             "Display Link Speed."));
            addOperation(new ExpanderOperation(OPERATION_ADDRESS,           "Display SAS Address."));
            break;
        case CMD_UTILITY:
            addOperation(new ExpanderOperation(OPERATION_RESET_CONTROLLER,  "Reset Expander."));
            break;
        case CMD_ERROR:
            addOperation(new ExpanderOperation(OPERATION_ERROR_ALL,         "Get All Expander PHY Error Statistics."));
            addOperation(new ExpanderOperation(OPERATION_DISPARITY_ERR,     "Display Disparity Error."));
            addOperation(new ExpanderOperation(OPERATION_INVALID_DWORD_ERR, "Display Invalid Dword Error."));
            addOperation(new ExpanderOperation(OPERATION_LOSS_DWORD_SYNC,   "Display Loss Dword Synch Error."));
            addOperation(new ExpanderOperation(OPERATION_PHY_RESET_PRBM,    "Display PHY Reset Problem Error."));
            addOperation(new ExpanderOperation(OPERATION_CLEAR,             "Clear All Expander PHY Error Statistics."));
            break;
        default:
            break;
    }
}

ExpanderCommand::~ExpanderCommand()
{

}

int
ExpanderCommand::executeCommand(StringParser commandLine)
{
    COMMAND_TYPE    commandType = NO_COMMAND;

    commandLine.getCommandType(&commandType);

    switch (commandType)
    {
        case CMD_LIST:
        case CMD_SHOW:
        case CMD_TEST:
        case CMD_ERROR:
        case CMD_UTILITY:
            return findAndExecuteOperation(commandLine);
        default:
            cout << UNKNOWN_COMMAND_MSG << endl;
            break;
    }
    
    return -1;
}
