#include "I2CCommand.hpp"

I2CCommand::I2CCommand(COMMAND_TYPE CommandType, string HelpContent) : CommandBaseClass(CommandType, HelpContent)
{
    //Base on command type to add different operations
    switch (CommandType)
    {
        case CMD_SHOW:
            addOperation(new I2COperation(OPERATION_PATH, "Shows I2C device path information"));
            break;
        case CMD_UTILITY:
            addOperation(new I2COperation(OPERATION_READ, "Read I2C device data"));
            addOperation(new I2COperation(OPERATION_WRITE, "Write data to I2C device"));
            addOperation(new I2COperation(OPERATION_DUMP, "Dump data in assignation data size"));
            break;
        case CMD_TEST:
            addOperation(new I2COperation(OPERATION_MUXTest, "MUX read/write test"));
            addOperation(new I2COperation(OPERATION_PreventionStuck, "bus stuck prevent test"));
            addOperation(new I2COperation(OPERATION_WRC, "Write data and read back, and do data comparison"));
            break;
        //Add NEW command and operation here
        default:
            break;
    }
}



I2CCommand::~I2CCommand()
{

}



int
I2CCommand::List(StringParser command_line)
{
    I2CDAPI       *pI2CDAPI = NULL;
    list<string>  i2c_list;
    string        i2c_device;
    ErrorMsg      *error_code = NULL;

    char          line[200], *pch;
    string        device_name, bus, slave_address;
    
    
    pI2CDAPI = new I2CDAPI("I2CDAPI");
    
    //Get I2C device list by API
    error_code = pI2CDAPI->DisplayI2cDevicesList(&i2c_list);
    if (error_code != NO_ERROR_MSG)
    {
        getLastErrorMessage(error_code, &i2c_device);
        cout << "Error: " << i2c_device << endl;
    }
    else
    {
        cout << "Bus  Slave     Names" << endl;
        while (!i2c_list.empty())
        {
            i2c_device = i2c_list.front();
            i2c_list.pop_front();
            strcpy(line, i2c_device.c_str());
            //printf("%s\n",line);

            pch = strtok(line, " ");
            device_name = pch;

            pch = strtok(NULL, " ");
            bus = pch;

            pch = strtok(NULL, " ");
            slave_address = pch;

            //Show I2C list
            cout << setw(5) << setiosflags(ios::left) << bus << setw(10) << slave_address <<  device_name << endl;
        }
    }

    return 0;
}



int
I2CCommand::Show(StringParser command_line)
{
   // cout << "I2C Show command execution" << endl;
    string parser_operation_name;
	  int i, err = 0;
	  string operation_name;

    command_line.getOperation(&parser_operation_name);

    //Check help or not operation than list supported operation
    if (parser_operation_name == HELP || parser_operation_name == "")
    {
        listOperation(command_line);
        return -1;
    }

    //Search for operation
    for (i = 0; i< m_operation_count; i++)
    {
        m_operation[i]->getName(&operation_name);
        if (operation_name == parser_operation_name)
        {
            err = m_operation[i]->executeOperation(command_line);
            break;
        }
    }

    if (i == m_operation_count)
    {
        cout << parser_operation_name << " operation not found" << endl;
        return -1;
    }
    else
    {
        return err;
    }

    return 0;
}



int
I2CCommand::Utility(StringParser command_line)
{
    string parser_operation_name;
	  int i, err = 0;
	  string operation_name;
	  

    command_line.getOperation(&parser_operation_name);
    
    //Check help or not operation than list supported operation
    if (parser_operation_name == HELP || parser_operation_name == "")
    {
        listOperation(command_line);
        return -1;
    }

    //Search for operation
    for (i = 0; i< m_operation_count; i++)
    {
    	  m_operation[i]->getName(&operation_name);
        if (operation_name == parser_operation_name)
        {
            err = m_operation[i]->executeOperation(command_line);
            break;
        }
    }

    if (i == m_operation_count)
    {
        cout << parser_operation_name << " operation not found" << endl;
        return -1;
    }
    else
    {
        return err;
    }
}



int
I2CCommand::Test(StringParser command_line)
{
    string parser_operation_name;
	  int i, err = 0;
	  string operation_name;

    command_line.getOperation(&parser_operation_name);

    //Check help or not operation than list supported operation
    if (parser_operation_name == HELP || parser_operation_name == "")
    {
        listOperation(command_line);
        return -1;
    }

    //Search for operation
    for (i = 0; i< m_operation_count; i++)
    {
        m_operation[i]->getName(&operation_name);
        if (operation_name == parser_operation_name)
        {
            err = m_operation[i]->executeOperation(command_line);
            break;
        }
    }

    if (i == m_operation_count)
    {
        cout << parser_operation_name << " operation not found" << endl;
        return -1;
    }
    else
    {
        return err;
    }
}



int
I2CCommand::executeCommand(StringParser command_line)
{
	  int err = 0;
	  COMMAND_TYPE command_type = NO_COMMAND;

    command_line.getCommandType(&command_type);

    switch (command_type)
    {
        case CMD_LIST:
            err = List(command_line);
            break;
        case CMD_SHOW:
            err = Show(command_line);
            break;
        case CMD_UTILITY:
            err = Utility(command_line);
            break;
        case CMD_TEST:
            err = Test(command_line);
            break;
        default:
            cout << "Not supported command" << endl;
            err = -1;
            break;
    }

    return err;
}

