
#include "MemoryCommand.hpp"
#include "CommonCLIFunctions.hpp"
#include "MemoryDAPI.hpp"

MemoryCommand::MemoryCommand(COMMAND_TYPE   in_eCommandType,
                             string         in_strHelpContent) : CommandBaseClass(in_eCommandType, in_strHelpContent)
{
    switch (in_eCommandType)
    {
        case CMD_SHOW:
            addOperation(new MemoryOperation(OPERATION_INFO, "Show DIMM information according to SPD"));
            addOperation(new MemoryOperation(OPERATION_TEST_RANGE, "Show testable memory range"));
            break;
        case CMD_TEST:
            addOperation(new MemoryOperation(OPERATION_ADDR_LINE, "Validate address pins of DIMMs"));
            addOperation(new MemoryOperation(OPERATION_DATA_LINE, "Validate data pins of DIMMs"));
            addOperation(new MemoryOperation(OPERATION_MARCHC, "Validate memory cells of DRAM chips by MarchC algorithm"));
            addOperation(new MemoryOperation(OPERATION_PATTERN, "Validate memory cells of DRAM by the test pattern"));
            addOperation(new MemoryOperation(OPERATION_RANDOM, "Validate memory cells of DRAM chips by random access"));
            addOperation(new MemoryOperation(OPERATION_CHECKSUM, "Validate checksum value for the specified memory range"));
            break;
        case CMD_UTILITY:
            addOperation(new MemoryOperation(OPERATION_READ, "Read data from the specified memory range"));
            addOperation(new MemoryOperation(OPERATION_WRITE, "Write data to the specified memory range"));
            break;
        case CMD_ECC:
            addOperation(new MemoryOperation(OPERATION_CHECK, "Check whether ECC errors occur or not"));
            addOperation(new MemoryOperation(OPERATION_CLEAR, "Clear ECC error status"));
            //addOperation(new MemoryOperation(OPERATION_INJECT, "Validate memory ECC error detection and correction"));
            break;
        default:
            break;
    }
}

MemoryCommand::~MemoryCommand()
{

}

int
MemoryCommand::executeCommand(StringParser in_parser)
{
	COMMAND_TYPE    eCommand = NO_COMMAND;

	in_parser.getCommandType(&eCommand);

    switch (eCommand)
    {
        case CMD_LIST:
            return executeCommandList(in_parser);
        case CMD_SHOW:
        case CMD_TEST:
        case CMD_UTILITY:
        case CMD_ECC:
            return findAndExecuteOperation(in_parser);
        default:
            cout << UNKNOWN_COMMAND_MSG << endl;
            break;
    }

    return -1;
}

int
MemoryCommand::executeCommandList(StringParser in_parser)
{
    MemoryDAPI      memoryDAPI("MemoryDAPI");
    ErrorMsg*       pErrorMsg = NO_ERROR_MSG;
    list<string>    deviceList;
    
    if ((pErrorMsg = memoryDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }

    pErrorMsg = memoryDAPI.getDeviceList(&deviceList);
    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    while (!deviceList.empty())
    {
        cout << deviceList.front() << endl;
        deviceList.pop_front();
    }

    return 0;
}
