#include "NTBOperation.hpp"

NTBOperation::NTBOperation(OPERATION_TYPE   in_eOprationType,
                           string           in_strHelpContent) : OperationBaseClass(in_eOprationType, in_strHelpContent)
{
    m_s32Result = 0;
    m_pNTBDAPI = new NTBDAPI("NTBDAPI");
    m_pPCIDAPI = new PCIDAPI("PCIDAPI");
}

NTBOperation::~NTBOperation()
{
    if(m_pNTBDAPI != NULL)
    {
        delete m_pNTBDAPI;
        m_pNTBDAPI = NULL;
    }
    
    if(m_pPCIDAPI != NULL)
    {
        delete m_pPCIDAPI;
        m_pPCIDAPI = NULL;
    }
}

    
int 
NTBOperation::checkName(string  in_name)
{
    bool    bDeviceFound = FALSE;

    vector<NTB_INFO>            ntbList;
    vector<NTB_INFO>::iterator  it;
    
    transform(in_name.begin(), in_name.end(), in_name.begin(), ::toupper);
    
    m_pErrorMsg = m_pNTBDAPI->searchDevice(&ntbList);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(NTB_DEBUG_LEVEL, m_pErrorMsg);
        return -1;
    }

    for(it = ntbList.begin(); it != ntbList.end(); ++it)
    {
        if(strcmp(it->ntbName.c_str(), in_name.c_str()) == 0)
        {
            bDeviceFound = TRUE;
            break;
        }
    }

    if(!bDeviceFound)
    {
        cout << "Invalid device name" << endl;
        return -1;
    }
    return 0;
}


int 
NTBOperation::getDeviceBDF(string   in_ntbName,
                           PFA*     out_pBDF)
{
    bool    deviceFound = FALSE;

    vector<NTB_INFO>            ntbList;
    vector<NTB_INFO>::iterator  it;
    
    transform(in_ntbName.begin(), in_ntbName.end(), in_ntbName.begin(), ::toupper);
    
    m_pErrorMsg = m_pNTBDAPI->searchDevice(&ntbList);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(NTB_DEBUG_LEVEL, m_pErrorMsg);
        return -1;
    }

    for(it = ntbList.begin(); it != ntbList.end(); ++it)
    {
        if(strcmp(it->ntbName.c_str(), in_ntbName.c_str()) == 0)
        {
            out_pBDF->u16Domain = it->pciInfo->u16Domain;
            out_pBDF->u16Bus = it->pciInfo->u16Bus;
            out_pBDF->u16Device = it->pciInfo->u16Device;
            out_pBDF->u8Function = it->pciInfo->u8Function;
            deviceFound = TRUE;
        }
    }

    if(!deviceFound)
    {
        cout << "Invalid device name" << endl;
        return -1;
    }
    return 0;
}

int 
NTBOperation::initialWindow(StringParser in_classCommandLine)
{
    m_pErrorMsg = m_pNTBDAPI->initialWindow();
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(NTB_DEBUG_LEVEL, m_pErrorMsg);
        return -1;
    }
    return 0;
}


void 
NTBOperation::getIoParms(StringParser   in_classCommandLine,
                         UINT64         in_defWritePat,
                         UINT64         in_defReadPat, 
                         IO_PARMS*      out_ioParms)
{
    int s32ParameterAmount;
    int s32Loop;

    in_classCommandLine.getParameterNumber(&s32ParameterAmount);
    if(s32ParameterAmount == 1) 
    {
        out_ioParms->u64PatternSend = in_defWritePat;
        out_ioParms->u64PatternRev = in_defReadPat;
        out_ioParms->u32Loop = 1;
        out_ioParms->name = m_strParameter[0];
    }
    else 
    {
        in_classCommandLine.convertStringToHex(m_strParameter[0], &(out_ioParms->u64PatternSend));
        in_classCommandLine.convertStringToHex(m_strParameter[1], &(out_ioParms->u64PatternRev));
        in_classCommandLine.convertStringToDec(m_strParameter[2], &s32Loop);
        out_ioParms->u32Loop = (UINT32)s32Loop;
        out_ioParms->name = m_strParameter[3]; 
    }
    transform(out_ioParms->name.begin(), out_ioParms->name.end(), out_ioParms->name.begin(), ::toupper);

    //check 
    printf("NTBOperation::getIoParms, u64PatternSend: 0x%llx\n", out_ioParms->u64PatternSend);
    printf("NTBOperation::getIoParms, u64PatternRev: 0x%llx\n", out_ioParms->u64PatternRev);
    printf("NTBOperation::getIoParms, s32Loop: %d\n", out_ioParms->u32Loop);
    printf("NTBOperation::getIoParms, name: %s\n", out_ioParms->name.c_str()); 
}


int 
NTBOperation::testPerformance(StringParser in_classCommandLine)
{
    int     s32ParameterAmount;
    int     s32BlockSize;
    UINT64  u64PatternSend;
    int     s32Loop;

    in_classCommandLine.getParameterNumber(&s32ParameterAmount);
    if(s32ParameterAmount == 4)
    {
        in_classCommandLine.convertStringToDec(m_strParameter[2], &s32BlockSize);
        in_classCommandLine.convertStringToDec(m_strParameter[3], &s32Loop);
    }
    else 
    {
        s32BlockSize = 0; //use default block size which is caught from system 
        s32Loop = 256; 
    }
    u64PatternSend = 0x55aa55aa55aa55aa;

    /*
    //check 
    printf("NTBOperation::client, u64PatternSend: 0x%llx\n", u64PatternSend);
    printf("NTBOperation::client, u64BlockSize: %d\n", s32BlockSize);
    printf("NTBOperation::client, s32Loop: %d\n", s32Loop);
    printf("NTBOperation::client, mode: %s\n", m_strParameter[0].c_str());
    */

    if(m_strParameter[0] == "server")
    {
        m_pErrorMsg = m_pNTBDAPI->testPerformance(u64PatternSend, (UINT64)s32BlockSize, (UINT64)s32Loop);
    }
    else //client
    {
        m_pErrorMsg = m_pNTBDAPI->receiver(u64PatternSend, 1);
    }
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(NTB_DEBUG_LEVEL, m_pErrorMsg);
        return -1;
    }
    return 0;
    
}

int 
NTBOperation::client(StringParser in_classCommandLine)
{
    IO_PARMS ioParms;

    getIoParms(in_classCommandLine, 0x33cc33cc33cc33cc, 0x55aa55aa55aa55aa, &ioParms);

    m_pErrorMsg = m_pNTBDAPI->client(ioParms.name, ioParms.u64PatternRev, ioParms.u64PatternSend, ioParms.u32Loop);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(NTB_DEBUG_LEVEL, m_pErrorMsg);
        return -1;
    }
    return 0;
}


int 
NTBOperation::server(StringParser in_classCommandLine)
{
    IO_PARMS ioParms;

    getIoParms(in_classCommandLine, 0x55aa55aa55aa55aa, 0x33cc33cc33cc33cc, &ioParms);

    m_pErrorMsg = m_pNTBDAPI->server(ioParms.name, ioParms.u64PatternSend, ioParms.u64PatternRev, ioParms.u32Loop);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(NTB_DEBUG_LEVEL, m_pErrorMsg);
        return -1;
    }
    return 0;
}


int 
NTBOperation::showInfo(string in_ntbName)
{
    PFA     bdf;
    int     s32Result = 0;

    s32Result = getDeviceBDF(in_ntbName, &bdf);
    if(s32Result == 0)
    {
        if(getOperationType() == OPERATION_ALL)
        {
            showSpecificInfo(bdf, OPERATION_SHOW_LINK);
            showSpecificInfo(bdf, OPERATION_SHOW_SPEED);
            showSpecificInfo(bdf, OPERATION_SHOW_WIDTH);
        }
        else 
        {
            s32Result = showSpecificInfo(bdf, getOperationType());            
        }
    }

    return s32Result;
}


int 
NTBOperation::clearErrors(string in_ntbName)
{
    int s32Result = 0;
    PFA bdf;

    s32Result = getDeviceBDF(in_ntbName, &bdf);
    if(s32Result == 0)
    {
        m_pErrorMsg = m_pPCIDAPI->clearErrors(bdf);
        if(m_pErrorMsg != NO_ERROR_MSG)
        {
            printError(NTB_DEBUG_LEVEL, m_pErrorMsg);
            return -1;
        }

        //check errors
        m_pPCIDAPI->showErrorStatistics(bdf, PCI_COMPATIBLE_ERROR, TRUE); //TRUE: only show summary
        m_pPCIDAPI->showErrorStatistics(bdf, PCIE_ERROR, TRUE);
        m_pPCIDAPI->showErrorStatistics(bdf, AER_ERROR, TRUE);        
    }

    return s32Result;
}

int
NTBOperation::showErrorStatistics(string    in_ntbName,
                                  UINT8     in_interface)
{
    int s32Result = 0;
    PFA bdf;

    s32Result = getDeviceBDF(in_ntbName, &bdf);
    if(s32Result != 0)
    {
        return s32Result;
    }

    if(getOperationType() == OPERATION_ERROR_ALL)
    {
        m_pPCIDAPI->showPlxNtbErrorStatistics(bdf, PCI_COMPATIBLE_ERROR, FALSE, in_interface);
        m_pPCIDAPI->showPlxNtbErrorStatistics(bdf, PCIE_ERROR, FALSE, in_interface);
        m_pPCIDAPI->showPlxNtbErrorStatistics(bdf, AER_ERROR, FALSE, in_interface);
    }
    else 
    {
        switch(getOperationType())
        {
            case OPERATION_PCI_COMPATIBLE:
                m_pPCIDAPI->showPlxNtbErrorStatistics(bdf, PCI_COMPATIBLE_ERROR, FALSE, in_interface);
                break;
            case OPERATION_PCIE:
                m_pPCIDAPI->showPlxNtbErrorStatistics(bdf, PCIE_ERROR, FALSE, in_interface);
                break;
            case OPERATION_AER:
                m_pPCIDAPI->showPlxNtbErrorStatistics(bdf, AER_ERROR, FALSE, in_interface);
                break;
            default:
                break;
        }
    }

    return s32Result;
}


int
NTBOperation::showSpecificInfo(PFA              in_bdf,
                               OPERATION_TYPE   in_opType)
{
    string      strTemp;
    

    switch(in_opType)
    {
        case OPERATION_SHOW_LINK:
            strTemp = m_pPCIDAPI->getReadableStringByBDF(in_bdf, PCI_DATA_TYPE_LINK_STATUS);
            printf("Link:\t\t\t %s\n", strTemp.c_str());
            break;
    
        case OPERATION_SHOW_SPEED:
            printf("Speed(MAX/CUR):\t\t %.1f/%.1f GT/s\n",
                    m_pPCIDAPI->GetReadableSpeed(in_bdf, PCI_DATA_TYPE_MAX_SPEED),
                    m_pPCIDAPI->GetReadableSpeed(in_bdf, PCI_DATA_TYPE_CURRENT_SPEED));
            break;

        case OPERATION_SHOW_WIDTH:
            printf("Width(MAX/CUR):\t\t x%d/x%d\n",
                    m_pPCIDAPI->GetDataByBDF(in_bdf, PCI_DATA_TYPE_MAX_WIDTH),
                    m_pPCIDAPI->GetDataByBDF(in_bdf, PCI_DATA_TYPE_CURRENT_WIDTH));
            break;

        default:
            break;
    }


    return 0;
}


int
NTBOperation::executeOperation(StringParser in_classCommandLine)
{
    int s32Index;

    m_s32Result = -1;

    //get parameter amount
    in_classCommandLine.getParameterNumber(&m_s32ParameterAmount);
    //get parameter list
    for(s32Index = 0; s32Index < m_s32ParameterAmount; s32Index++)
    {
        in_classCommandLine.getParameter(&m_strParameter[s32Index], s32Index);
    }

    in_classCommandLine.getParameter(&m_strParameter[0], 0);
    if(m_strParameter[0] == HELP)
    {
        showHelp();
        return 0;
    }
    //check Syntax
    m_s32Result = checkSyntax(in_classCommandLine);
    if(m_s32Result != 0)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return m_s32Result;
    }

    switch(getOperationType())
    {
        case OPERATION_ALL:
        case OPERATION_SHOW_LINK:
        case OPERATION_SHOW_SPEED:
        case OPERATION_SHOW_WIDTH:
            return showInfo(m_strParameter[0]);

        //test 
        case OPERATION_SERVER:
            return server(in_classCommandLine);
        case OPERATION_CLIENT:
            return client(in_classCommandLine);
        case OPERATION_PERFORMANCE:
            return testPerformance(in_classCommandLine);

        case OPERATION_CLEAR:
            return clearErrors(m_strParameter[0]);

        case OPERATION_ERROR_ALL:
        case OPERATION_PCI_COMPATIBLE:
        case OPERATION_PCIE:
        case OPERATION_AER:
            cout << "NTB Virtual Side Error Statistics:" << endl;
            m_s32Result =  showErrorStatistics(m_strParameter[0], PLX_PCIE_SWITCH_NTB_VIRTUAL);
            if(m_s32Result == 0)
            {
                cout << endl;
                cout << "NTB Link Side Error Statistics:" << endl;
                m_s32Result =  showErrorStatistics(m_strParameter[0], PLX_PCIE_SWITCH_NTB_LINK);
            }
            break;

        case OPERATION_INIT:
            return initialWindow(in_classCommandLine);

        default:
            break;
    }

    return m_s32Result;
}


//Syntax check for server/client functions
int 
NTBOperation::checkSyntaxSC(StringParser in_classCommandLine)
{
    int     s32Result = 0;
    int     s32Value;
    UINT64  u64Value;

    if(m_s32ParameterAmount == 4)
    {
        //check write pattern
        s32Result = in_classCommandLine.convertStringToHex(m_strParameter[0], &u64Value);
        if(s32Result != 0)
        {
            return s32Result;
        }
        //check read pattern 
        s32Result = in_classCommandLine.convertStringToHex(m_strParameter[1], &u64Value);
        if(s32Result != 0)
        {
            return s32Result;
        }
        //check loop time
        s32Result = in_classCommandLine.convertStringToDec(m_strParameter[2], &s32Value);
        if(s32Result != 0)
        {
            return s32Result;
        }
        //check name 
        s32Result = checkName(m_strParameter[3]);
    }
    else if(m_s32ParameterAmount == 1)
    {
        s32Result = checkName(m_strParameter[0]);
    }
    else 
    {
        s32Result = -1;
    }

    return s32Result;
}


int 
NTBOperation::checkSyntax(StringParser in_classCommandLine)
{
    int s32Result = 0;
    
    switch(getOperationType())
    {
        case OPERATION_ALL:
        case OPERATION_SHOW_LINK:
        case OPERATION_SHOW_SPEED:
        case OPERATION_SHOW_WIDTH:
        case OPERATION_CLEAR:
        case OPERATION_ERROR_ALL:
        case OPERATION_PCI_COMPATIBLE:
        case OPERATION_PCIE:
        case OPERATION_AER:
        case OPERATION_INIT:
            if(m_s32ParameterAmount != 1)
            {
                return -1;
            }
            s32Result = checkName(m_strParameter[0]);
            break;
        
        case OPERATION_SERVER:
        case OPERATION_CLIENT:
            s32Result = checkSyntaxSC(in_classCommandLine);
            break;
        case OPERATION_PERFORMANCE:
            if(m_s32ParameterAmount < 2)
            {
                return -1;
            }
            if(m_strParameter[0] != "server" && m_strParameter[0] != "client")
            {
                return -1;
            }
            s32Result = checkName(m_strParameter[1]);
            break;
          
        //test !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        /*
        case OPERATION_INIT: 
        case OPERATION_SERVER:
        case OPERATION_CLIENT:
        case OPERATION_PERFORMANCE:
            return 0;
        */

        default:
            s32Result = -1;
            break;
    }

    return s32Result;
}


//show help for server/client functions 
void 
NTBOperation::showHelpSC(UINT64 in_u64WritePat,
                         UINT64 in_u64ReadPat)
{
    cout << "P1: write pattern(64bit hex)" << endl;
    cout << "P2: read pattern(64bit hex)" << endl;
    cout << "P3: loop time(dec)" << endl;
    cout << "P4: device name" << endl;
    cout << endl;
    cout << "P1 - P3 are optional, default values are:" << endl;
    printf( "P1: 0x%08llx\n", in_u64WritePat);
    printf( "P2: 0x%08llx\n", in_u64ReadPat);
    cout << "P3: 1" << endl;
    cout << endl;
    cout << "Examples:" << endl;
    cout << "lemur ntb test server ntb0" << endl;
    printf( "lemur ntb test server 0x%08llx 0x%08llx 10 ntb0\n", in_u64WritePat, in_u64ReadPat);
    cout << endl;
}

    
void
NTBOperation::showHelp()
{
    string strHelpContent;
    
    getHelp(&strHelpContent);
    cout << strHelpContent << endl;
    //cout << "Parameters required:" << endl;
    cout << endl;

    switch(getOperationType())
    {
        case OPERATION_ALL:
        case OPERATION_LINK:
        case OPERATION_SHOW_SPEED:
        case OPERATION_SHOW_WIDTH:
        case OPERATION_CLEAR:
        case OPERATION_ERROR_ALL:
        case OPERATION_PCI_COMPATIBLE:
        case OPERATION_PCIE:
        case OPERATION_AER:
            cout << " P1: device name" << endl;
            break;

        case OPERATION_SERVER:
            showHelpSC(0x55aa55aa55aa55aa, 0x33cc33cc33cc33cc);
            break;
        case OPERATION_CLIENT:
            showHelpSC(0x33cc33cc33cc33cc, 0x55aa55aa55aa55aa);
            break;
        case OPERATION_PERFORMANCE:
            cout << "P1: server or client" << endl;
            cout << "P2: device name" << endl;
            cout << "Examples:" << endl;
            cout << "   lemur ntb test performance server ntb0" << endl;
            cout << "   lemur ntb test performance client ntb0" << endl;
        default:
            cout << "Unsupport operation" << endl;
            m_s32Result = -1;
            break;
    }
}







