#include "NVMePrint.hpp"
#include <endian.h>
#include <inttypes.h>
#include <stdio.h>

long double convert16ByteToDouble(__u8 *data, unsigned int size)
{
	unsigned int index;
	unsigned int dataIndex = size - 1;
	long double lVal = 0;

	for (index = 0; index < size; index++)
	{
		lVal *= 256;
		lVal += data[dataIndex--];
	}

	return lVal;
}

void showNvmeIdentifyNS(struct nvme_id_ns *idNS)
{
	int i;
    __u8 fpis, fpii;
    
    printf("Namespace Size                                                        : %#llX\n", le64toh(idNS->nsze));
	printf("Namespace Capacity                                                    : %#llX\n", le64toh(idNS->ncap));
	printf("Namespace Utilization                                                 : %#llX\n", le64toh(idNS->nuse));
	//printf("Namespace Features      : %#X\n", idNS->nsfeat);
    printf("Namespace Features\n");
    printf("    - Deallocated or Unwritten Logical Block Error Supported          : %s\n",
           (idNS->nsfeat & 0x4) >> 2 ? "Yes" : "No");
    printf("    - Namespace Uses                                                  : %s\n", 
           (idNS->nsfeat & 0x2) >> 1 ? "NAWUN, NAWUPF, and NACWU" : "AWUN, AWUPF, and ACWU");
    printf("    - Thin Provisioning Supported                                     : %s\n", idNS->nsfeat & 0x1 ? "Yes" : "No");
	printf("Number of LBA Formats                                                 : %d\n", idNS->nlbaf);
	//printf("Formatted LBA Size : %#X\n", idNS->flbas);
    printf("Formatted LBA Size\n");
    printf("    - Metadata Transferred                                            : %s\n", 
           (idNS->flbas & 0x10) >> 4 ? "at End of Data LBA" : "in Separate Contiguous Buffer");
    printf("    - Current LBA Format Selected                                     : %#X\n", idNS->flbas & 0xF);
	//printf("Metadata Capabilities : %#X\n", idNS->mc);
    printf("Metadata Capabilities\n");
    printf("    - Metadata Pointer Supported                                      : %s\n", (idNS->mc & 0x2) >> 1 ? "Yes" : "No");
    printf("    - Metadata as Part of Extended Data LBA Supported                 : %s\n", idNS->mc & 0x1 ? "Yes" : "No");
	//printf("End-to-end Data Protection Capabilities : %#X\n", idNS->dpc);
    printf("End-to-end Data Protection Capabilities\n");
    printf("    - Protect Info Transferred as Last 8 Bytes of Metadata Supported  : %s\n",  (idNS->dpc & 0x10) >> 4 ? "Yes" : "No");
    printf("    - Protect Info Transferred as First 8 Bytes of Metadata Supported : %s\n", (idNS->dpc & 0x8) >> 3 ? "Yes" : "No");
    printf("    - Protect Info Type 3 Supported                                   : %s\n", (idNS->dpc & 0x4) >> 2 ? "Yes" : "No");
    printf("    - Protect Info Type 2 Supported                                   : %s\n", (idNS->dpc & 0x2) >> 1 ? "Yes" : "No");
    printf("    - Protect Info Type 1 Supported                                   : %s\n", idNS->dpc & 0x1 ? "Yes" : "No");
	//printf("End-to-end Data Protection Type Settings : %#X\n", idNS->dps);
    printf("End-to-end Data Protection Type Settings\n");
    __u8 pit = idNS->dps & 0x7;
    printf("    - Protect Info Transferred                                        : %s 8 Bytes of Metadata\n", (idNS->dps & 0x8) >> 3 ? "First" : "Last");
    printf("    - Protect Info Type                                               : %s\n",
           pit == 3 ? "Type 3 Enabled" :
           pit == 2 ? "Type 2 Enabled" :
           pit == 1 ? "Type 1 Enabled" :
           pit == 0 ? "Disabled" : "Reserved Enabled");
	//printf("Namespace Multi-path I/O and Namespace Sharing Capabilities : %#X\n", idNS->nmic);
    printf("Namespace Multi-path I/O and Namespace Sharing Capabilities\n");
	printf("    - Namespace Multipath Capable                                     : %s\n", idNS->nmic & 0x1 ? "Yes" : "No");
	//printf("Reservation Capabilities : %#X\n", idNS->rescap);
    printf("Reservation Capabilities\n");
    printf("    - Exclusive Access - All Registrants Supported                    : %s\n", (idNS->rescap & 0x40) >> 6 ? "Yes" : "No");
    printf("    - Write Exclusive  - All Registrants Supported                    : %s\n", (idNS->rescap & 0x20) >> 5 ? "Yes" : "No");
    printf("    - Exclusive Access - Registrants Only Supported                   : %s\n", (idNS->rescap & 0x10) >> 4 ? "Yes" : "No");
    printf("    - Write Exclusive  - Registrants Only Supported                   : %s\n", (idNS->rescap & 0x8) >> 3 ? "Yes" : "No");
    printf("    - Exclusive Access Supported                                      : %s\n", (idNS->rescap & 0x4) >> 2 ? "Yes" : "No");
    printf("    - Write Exclusive Supported                                       : %s\n", (idNS->rescap & 0x2) >> 1 ? "Yes" : "No");
    printf("    - Persist Through Power Loss Supported                            : %s\n", idNS->rescap & 0x1 ? "Yes" : "No");
	//printf("Format Progress Indicator : %#X\n", idNS->fpi);
    printf("Format Progress Indicator\n");
    fpis = (idNS->fpi & 0x80) >> 7;
    fpii = idNS->fpi & 0x7F;
    printf("    - Format Progress Indicator Supported                             : %s\n", fpis ? "Yes" : "No");
    if (fpis || (!fpis && fpii))
        printf("    - Format Progress Indicator Remaining                             : %d%%\n", 100 - fpii);
	
	printf("Namespace Atomic Write Unit Normal                                    : %d\n", idNS->nawun);
	printf("Namespace Atomic Write Unit Power Fail                                : %d\n", idNS->nawupf);
	printf("Namespace Atomic Compare & Write Unit                                 : %d\n", idNS->nacwu);
	printf("Namespace Atomic Boundary Size Normal                                 : %d\n", idNS->nabsn);
	printf("Namespace Atomic Boundary Offset                                      : %d\n", idNS->nabo);
	printf("Namespace Atomic Boundary Size Power Fail                             : %d\n", idNS->nabspf);
	printf("NVM Capacity                                                          : %.Lf\n", convert16ByteToDouble(idNS->nvmcap, sizeof(idNS->nvmcap)));
	printf("Namespace Globally Unique Identifier                                  : ");
	for (i = 0; i < 16; i++)
		printf("%02X", idNS->nguid[i]);
	printf("\n");
	printf("IEEE Extended Unique Identifier                                       : ");
	for (i = 0; i < 8; i++)
		printf("%02X", idNS->eui64[i]);
	printf("\n");

	for (i = 0; i <= idNS->nlbaf; i++)
	{
        printf("LBA Format %2d\n", i);
        printf("    - in Use                                                          : %s\n", i == (idNS->flbas & 0xf) ? "Yes" : "No");
        printf("    - Metadata Size                                                   : %-3d bytes\n", idNS->lbaf[i].ms);
        printf("    - LBA Data Size                                                   : %-2d bytes\n", 1 << idNS->lbaf[i].ds);
        printf("    - Relative  Performance                                           : %s\n",
                idNS->lbaf[i].rp == 3 ? "Degraded" :
                idNS->lbaf[i].rp == 2 ? "Good" :
                idNS->lbaf[i].rp == 1 ? "Better" : "Best");

	}
    
    //==================================================
    //Vendor Specific
	//printf("Vendor Specific                                                       :\n");
	//displayRaw(idNS->vs, sizeof(idNS->vs));

}

void showNvmeIdentifyCtrl(struct nvme_id_ctrl *idCtrl)
{
    int i;
    __u32 sglltb;
    __u32 bacmdb;
    __u32 bbs;
    __u32 sglsp;
    
    //Controller Capabilities and Features
	printf("PCI Vendor ID                                                : %#X\n", idCtrl->vid);
	printf("PCI Subsystem Vendor ID                                      : %#X\n", idCtrl->ssvid);
	printf("Serial Number                                                : %-20.20s\n", idCtrl->sn);
	printf("Model Number                                                 : %-20.20s\n", idCtrl->mn);
  	printf("Firmware Revision                                            : %-.8s\n", idCtrl->fr);
	printf("Recommended Arbitration Burst                                : %d\n", idCtrl->rab);
	printf("IEEE OUI Identifier                                          : %02X%02X%02X\n", 
            idCtrl->ieee[2], idCtrl->ieee[1], idCtrl->ieee[0]);
    //printf("Controller Multi-Path I/O and Namespace Sharing Capabilities : %#X\n", idCtrl->cmic);
	printf("Controller Multi-Path I/O and Namespace Sharing Capabilities\n");
    printf("    - Associated Function                                    : %s\n", (idCtrl->cmic & 0x4) >> 2 ? "SR-IOV" : "PCI");
    printf("    - Connected Host(s)                                      : %s Controller\n", (idCtrl->cmic & 0x2) >> 1 ? "Multi" : "Single");
    printf("    - Contain NVM Subsystem Port(s)                          : %s Port\n", idCtrl->cmic & 0x1 ? "Multi" : "Single");
	printf("Maximum Data Transfer Size                                   : %d\n", idCtrl->mdts);
	printf("Controller ID                                                : %X\n", idCtrl->cntlid);
	printf("Version                                                      : %X\n", idCtrl->ver);
	printf("RTD3 Resume Latency                                          : %X\n", idCtrl->rtd3r);
	printf("RTD3 Entry Latency                                           : %X\n", idCtrl->rtd3e);
	//printf("Optional Asynchronous Events Supported                       : %#X\n", idCtrl->oaes);
    printf("Optional Asynchronous Events Supported\n");
    printf("    - Firmware Activation Changed Event Supported            : %s\n", (idCtrl->oaes & 0x200) >> 9 ? "Yes" : "No");
	printf("    - Namespace Attribute Changed Event Supported            : %s\n", (idCtrl->oaes & 0x100) >> 8 ? "Yes" : "No");
	//printf("Controller Attributes                                        : %#X\n", idCtrl->ctratt);
    printf("Controller Attributes\n");
	printf("    - 128-bit Host Identifier Supported                      : %s\n", idCtrl->ctratt & 0x1 ? "Yes" : "No");
    
    //==================================================
    //Admin Command Set Attributes & Optional Controller Capabilities
	//printf("Optional Admin Command Support                               : %#X\n", idCtrl->oacs);
    printf("Optional Admin Command Support\n");
    printf("    - NS Management and Attachment Supported                 : %s\n", (idCtrl->oacs & 0x8) >> 3 ? "Yes" : "No");
    printf("    - FW Commit and Download Supported                       : %s\n", (idCtrl->oacs & 0x4) >> 2 ? "Yes" : "No");
    printf("    - Format NVM Supported                                   : %s\n", (idCtrl->oacs & 0x2) >> 1 ? "Yes" : "No");
    printf("    - Security Send and Receive Supported                    : %s\n", idCtrl->oacs & 0x1 ? "Yes" : "No");
	printf("Abort Command Limit                                          : %d\n", idCtrl->acl);
	printf("Asynchronous Event Request Limit                             : %d\n", idCtrl->aerl);
	//printf("Firmware Updates                                             : %#X\n", idCtrl->frmw);
    printf("Firmware Updates\n");
    printf("    - Firmware Activate Without Reset Supported              : %s\n", (idCtrl->frmw & 0x10) >> 4 ? "Yes" : "No");
    printf("    - Number of Firmware Slots                               : %d\n", (idCtrl->frmw & 0xE) >> 1);
    printf("    - First Firmware Slot                                    : Read%s\n", idCtrl->frmw & 0x1 ? "-Only" : "/Write");	
    //printf("Log Page Attributes                                          : %#X\n", idCtrl->lpa);
    printf("Log Page Attributes\n");
    printf("    - Command Effects Log Page Supported                     : %s\n", (idCtrl->lpa & 0x2) >> 1 ? "Yes" : "No");
    printf("    - SMART/Health Log Page per NS Supported                 : %s\n", idCtrl->lpa & 0x1 ? "Yes" : "No");
	printf("Error Log Page Entries                                       : %d\n", idCtrl->elpe);
	printf("Number of Power States Support                               : %d\n", idCtrl->npss);
	//printf("Admin Vendor Specific Command Configuration                  : %#X\n", idCtrl->avscc);
    printf("Admin Vendor Specific Command Configuration\n");
    printf("    - Admin Vendor Specific Commands uses Format             : %s\n", idCtrl->avscc & 0x1 ? "NVMe" : "Vendor Specific");
    //printf("Autonomous Power State Transition Attributes                 : %#X\n", idCtrl->apsta);
    printf("Autonomous Power State Transition Attributes\n");
	printf("    - Autonomous Power State Transitions Supported           : %s\n", idCtrl->apsta & 0x1 ? "Yes" : "No");
	printf("Warning Composite Temperature Threshold                      : %d C\n", idCtrl->wctemp - 273);
	printf("Critical Composite Temperature Threshold                     : %d C\n", idCtrl->cctemp - 273);
	printf("Maximum Time for Firmware Activation                         : %d\n", idCtrl->mtfa);
	printf("Host Memory Buffer Preferred Size                            : %d\n", idCtrl->hmpre);
	printf("Host Memory Buffer Minimum Size                              : %d\n", idCtrl->hmmin);
	printf("Total NVM Capacity                                           : %.Lf\n", convert16ByteToDouble(idCtrl->tnvmcap, sizeof(idCtrl->tnvmcap)));
	printf("Unallocated NVM Capacity                                     : %.Lf\n", convert16ByteToDouble(idCtrl->unvmcap, sizeof(idCtrl->unvmcap)));
	//printf("Replay Protected Memory Block Support                        : %#X\n", idCtrl->rpmbs);
    printf("Replay Protected Memory Block Support\n");
    printf("    - Access Size                                            : %d\n", (idCtrl->rpmbs & 0xFF000000) >> 24);
    printf("    - Total Size                                             : %d\n", (idCtrl->rpmbs & 0xFF0000) >> 16);
    printf("    - Authentication Method                                  : %#X\n", (idCtrl->rpmbs & 0x38) >> 3);
    printf("    - Number of RPMB Units                                   : %#X\n", idCtrl->rpmbs & 0x3);
    
    //==================================================
    //NVM Command Set Attributes
	//printf("Submission Queue Entry Size                                  : %#X\n", idCtrl->sqes);
    printf("Submission Queue Entry Size\n");
    printf("    - Max SQ Entry Size                                      : %d\n", 1 << ((idCtrl->sqes & 0xF0) >> 4));
    printf("    - Min SQ Entry Size                                      : %d\n", 1 << (idCtrl->sqes & 0xF));	
	//printf("Completion Queue Entry Size                                  : %#X\n", idCtrl->cqes);
    printf("Completion Queue Entry Size\n");
    printf("    - Max CQ Entry Size                                      : %d\n", 1 << ((idCtrl->cqes & 0xF0) >> 4));
    printf("    - Min CQ Entry Size                                      : %d\n", 1 << (idCtrl->cqes & 0xF));
	printf("Number of Namespaces                                         : %d\n", idCtrl->nn);
	//printf("Optional NVM Command Support                                 : %#X\n", idCtrl->oncs);
    printf("Optional NVM Command Support\n");
    printf("    - Reservations Supported                                 : %s\n", (idCtrl->oncs & 0x20) >> 5 ? "Yes" : "No");
    printf("    - Save and Select Supported                              : %s\n", (idCtrl->oncs & 0x10) >> 4 ? "Yes" : "No");
    printf("    - Write Zeroes Supported                                 : %s\n", (idCtrl->oncs & 0x8) >> 3 ? "Yes" : "No");
    printf("    - Data Set Management Supported                          : %s\n", (idCtrl->oncs & 0x4) >> 2 ? "Yes" : "No");
    printf("    - Write Uncorrectable Supported                          : %s\n", (idCtrl->oncs & 0x2) >> 1 ? "Yes" : "No");
    printf("    - Compare Supported                                      : %s\n", idCtrl->oncs & 0x1 ? "Yes" : "No");
	//printf("Fused Operation Support                                      : %#X\n", idCtrl->fuses);
    printf("Fused Operation Support\n");
	printf("    - Fused Compare and Write Supported                      : %s\n", idCtrl->fuses & 0x1 ? "Yes" : "No");
	//printf("Format NVM Attributes                                        : %#X\n", idCtrl->fna);
    printf("Format NVM Attributes\n");
    printf("    - Crypto Erase Supported as part of Secure Erase         : %s\n", (idCtrl->fna & 0x4) >> 2 ? "Yes" : "No");
    printf("    - Crypto Erase Applies to All Namespace                  : %s\n", (idCtrl->fna & 0x2) >> 1 ? "Yes" : "No");
    printf("    - Format Applies to All Namespace                        : %s\n", idCtrl->fna & 0x1 ? "Yes" : "No");
	//printf("Volatile Write Cache                                         : %#X\n", idCtrl->vwc);
    printf("Volatile Write Cache\n");
	printf("    - Volatile Write Cache Present                           : %s\n", idCtrl->vwc & 0x1 ? "Yes" : "No");
	printf("Atomic Write Unit Normal                                     : %d\n", idCtrl->awun);
	printf("Atomic Write Unit Power Fail                                 : %d\n", idCtrl->awupf);
    //printf("NVM Vendor Specific Command Configuration                    : %d\n", idCtrl->nvscc);
    printf("NVM Vendor Specific Command Configuration\n");
	printf("    - NVM Vendor Specific Commands uses NVMe Format          : %s\n", idCtrl->nvscc & 0x1 ? "Yes" : "No");
	printf("Atomic Compare & Write Unit                                  : %d\n", idCtrl->acwu);
    //printf("SGL Support                                                  : %#X\n", idCtrl->sgls);
    printf("SGL Support\n");
    sglltb = (idCtrl->sgls & 0x40000) >> 18;
    bacmdb = (idCtrl->sgls & 0x20000) >> 17;
    bbs = (idCtrl->sgls & 0x10000) >> 16;
    sglsp = idCtrl->sgls & 0x1;
    
    if (sglsp || (!sglsp && sglltb))
        printf("    - SGL Length Larger than Buffer Supported                : %s\n", sglltb ? "Yes" : "No");
    if (sglsp || (!sglsp && bacmdb))
        printf("    - Byte-Aligned Contig. MD Buffer Supported               : %s\n", bacmdb ? "Yes" : "No");
    if (sglsp || (!sglsp && bbs))
        printf("    - SGL Bit-Bucket Supported                               : %s\n", bbs ? "Yes" : "No");
    printf("    - Scatter-Gather Lists Supported                         : %s\n", sglsp ? "Yes" : "No");
	
    //==================================================
    //Power State Descriptors
	for (i = 0; i <= idCtrl->npss; i++)
	{
        //printf("Power State %2d Descriptor                                    : \n", i);
        printf("Power State %2d Descriptor\n", i);
        
		if (idCtrl->psd[i].flags & 0x2)
			printf("    - Operational State                                      : No\n");
        else
            printf("    - Operational State                                      : Yes\n");
            
		printf("    - Active Power                                           : ");
        switch (idCtrl->psd[i].active_work_scale)
        {
            case 0:     //Not reported for this power state
                
                printf("Not reported for this power state\n");
                break;

            case 1:     //0.0001 W
                
                printf("%01u.%04uW\n", idCtrl->psd[i].active_power / 10000, idCtrl->psd[i].active_power % 10000);
                break;

            case 2:     //0.01 W
                printf("%01u.%02uW\n", idCtrl->psd[i].active_power / 100, idCtrl->psd[i].active_power % 100);
                break;

            default:
                printf("Reserved\n");
        }
        
		printf("    - Idle Power                                             : ");
        switch (idCtrl->psd[i].idle_scale)
        {
            case 0:     //Not reported for this power state
                
                printf("Not reported for this power state\n");
                break;

            case 1:     //0.0001 W
                
                printf("%01u.%04uW\n", idCtrl->psd[i].idle_power / 10000, idCtrl->psd[i].idle_power % 10000);
                break;

            case 2:     //0.01 W
                printf("%01u.%02uW\n", idCtrl->psd[i].idle_power / 100, idCtrl->psd[i].idle_power % 100);
                break;

            default:
                printf("Reserved\n");
        }
                
		if (idCtrl->psd[i].flags & 0x1)
			printf("    - Maximum Power                                          : %01u.%04uW\n", idCtrl->psd[i].max_power / 10000, idCtrl->psd[i].max_power % 10000);
		else
			printf("    - Maximum Power                                          : %01u.%02uW\n", idCtrl->psd[i].max_power / 100, idCtrl->psd[i].max_power % 100);

	}
    
    //==================================================
    //Vendor Specific
    //printf("Vendor Specific                                              : \n");
	//displayRaw(idCtrl->vs, sizeof(idCtrl->vs));

}

void showErrorLog(struct nvme_error_log_page *err_log)
{
	printf("Error Count                           : %#llX\n", le64toh(err_log[0].error_count));
	printf("Submission Queue ID                   : %d\n", err_log[0].sqid);
	printf("Command ID                            : %#X\n", err_log[0].cmdid);
	printf("Status Field                          : %#X\n", err_log[0].status_field);
    printf("Parameter Error Location (Byte)       : %#X\n", err_log[0].parm_error_location & 0xFF);
    printf("Parameter Error Location (Bit)        : %#X\n", (err_log[0].parm_error_location & 0x700) >> 8);
	printf("LBA                                   : %#llX\n", le64toh(err_log[0].lba));
	printf("Namespace                             : %d\n", err_log[0].nsid);
	printf("Vendor Specific Information Available : %#X\n", err_log[0].vs);
	printf("Command Specific Information          : %#llX\n", le64toh(err_log[0].cs));

}

void showFWLog(struct nvme_firmware_log_page *fwLog)
{
	int i, j;
	char tmpStr[9];
	char *ptrStr;

	printf("Active Firmware Slot          : %#X\n", (fwLog->afi & 0x7));
	printf("Next Active Firmware Slot     : %#X\n", (fwLog->afi & 0x70) >> 4);
    
	for (i = 0; i < 7; i++)
	{
		if (fwLog->frs[i])
		{			
			ptrStr = (char *)&fwLog->frs[i];
			for (j = 0; j < 8; j++)
			{
				if ((ptrStr[j] >= 0x20) && (ptrStr[j] <= 0x71))
					tmpStr[j] = ptrStr[j];
				else
					tmpStr[j] = '.';
			}
			tmpStr[j] = '\0';
			printf("Firmware Revision for Slot %d  : %s\n", i + 1, tmpStr);

		}
	}
}

void showSmartLog(struct nvme_smart_log *smartLog)
{
	int index;

	printf("Critical Warning                    : %#X\n", smartLog->critical_warning);
	printf("Temperature                         : %u C\n", ((smartLog->temperature[1] << 8) | smartLog->temperature[0]) - 273);
	printf("Available Spare                     : %u%%\n", smartLog->avail_spare);
	printf("Available Spare Threshold           : %u%%\n", smartLog->spare_thresh);
	printf("Percentage Used                     : %u%%\n", smartLog->percent_used);
	printf("Data Units Read                     : %.Lf\n", convert16ByteToDouble(smartLog->data_units_read, sizeof(smartLog->data_units_read)));
    printf("Data Units Written                  : %.Lf\n", convert16ByteToDouble(smartLog->data_units_written, sizeof(smartLog->data_units_written)));
	printf("Host Read Commands                  : %.Lf\n", convert16ByteToDouble(smartLog->host_reads, sizeof(smartLog->host_reads)));
	printf("Host Write Commands                 : %.Lf\n", convert16ByteToDouble(smartLog->host_writes, sizeof(smartLog->host_writes)));
	printf("Controller Busy Time                : %.Lf\n", convert16ByteToDouble(smartLog->ctrl_busy_time, sizeof(smartLog->ctrl_busy_time)));
	printf("Power Cycles                        : %.Lf\n", convert16ByteToDouble(smartLog->power_cycles, sizeof(smartLog->power_cycles)));
	printf("Power On Hours                      : %.Lf\n", convert16ByteToDouble(smartLog->power_on_hours, sizeof(smartLog->power_on_hours)));
	printf("Unsafe Shutdowns                    : %.Lf\n", convert16ByteToDouble(smartLog->unsafe_shutdowns, sizeof(smartLog->unsafe_shutdowns)));
	printf("Media and Data Integrity Errors     : %.Lf\n", convert16ByteToDouble(smartLog->media_errors, sizeof(smartLog->media_errors)));
	printf("Number of Error Infor Log Entries   : %.Lf\n", convert16ByteToDouble(smartLog->num_err_log_entries, sizeof(smartLog->num_err_log_entries)));
	printf("Warning Temperature Time            : %u\n", smartLog->warning_temp_time);
	printf("Critical Temperature Time           : %u\n", smartLog->critical_comp_time);
    
	for (index = 0; index < 8; index++)
	{
		printf("Temperature Sensor %d                : ", index + 1);
        
        if (smartLog->temp_sensor[index] != 0)
            printf("%u C\n", smartLog->temp_sensor[index] - 273);
        else
            printf("0 C\n");
	}
}

/*
void nvmeFeatureShowFields(unsigned int fid, unsigned int result, unsigned char *buf)
{
	__u8 		field;
	uint64_t 	ull;
	uint32_t	value;
	int         i, j;
    
	switch (fid)
	{
		case NVME_FEAT_ARBITRATION:
            printf("Arbitration\n");
			printf("High Priority Weight   : %u\n", ((result & 0xFF000000) >> 24) + 1);
			printf("Medium Priority Weight : %u\n", ((result & 0xFF0000) >> 16) + 1);
			printf("Low Priority Weight    : %u\n", ((result & 0xFF00) >> 8) + 1);
			printf("Arbitration Burst      : %u\n",  (1 << (result & 0x7)));
			break;

		case NVME_FEAT_POWER_MGMT:
            printf("Power Management\n");
			field = (result & 0x000000E0) >> 5;
			printf("Workload Hint : %u - ",  field);
            switch (field)
            {
                case 0:	
                    printf("(No Workload)\n");
                    break;
                case 1:
                    printf("(Extended Idle Period with a Burst of Random Writes)\n");
                    break;
                case 2:
                    printf("(Heavy Sequential Writes)\n");
                    break;
                default:
                    printf("(Reserved)\n");
                    break;
            }
			printf("Power State   : %u\n",  result & 0x0000001f);
			break;

		case NVME_FEAT_LBA_RANGE:
            printf("LBA Range Type\n");
			field = result & 0x0000003f;
			printf("Number of LBA Ranges          : %u\n", field + 1);
            
            struct nvme_lba_range_type *lbrt;
            lbrt = (struct nvme_lba_range_type *)buf;
            for (i = 0; i <= field; i++)
            {
                printf("Type                      : %#X - %s\n", lbrt[i].type, nvme_feature_lba_type_to_string(lbrt[i].type));
                printf("Attributes                : %#X - %s, %s\n", lbrt[i].attributes, (lbrt[i].attributes & 0x0001) ? "LBA range may be overwritten":"LBA range should not be overwritten",
                    ((lbrt[i].attributes & 0x0002) >> 1) ? "LBA range should be hidden from the OS/EFI/BIOS":"LBA range should be visible from the OS/EFI/BIOS");
                printf("Starting LBA              : %#lX\n", (uint64_t)(lbrt[i].slba));
                printf("Number of Logical Blocks  : %#lX\n", (uint64_t)(lbrt[i].nlb));
                printf("Unique Identifier (GUID)  : ");
                for (j = 0; j < 16; j++)
                    printf("%02X", lbrt[i].guid[j]);
                printf("\n");
            }
			break;

		case NVME_FEAT_TEMP_THRESH:
            printf("Temperature Threshold\n");
			field = (result & 0x00300000) >> 20;
			printf("    - Threshold Type Select         (THSEL): %u - %s\n", field, nvme_feature_temp_type_to_string(field));
			field = (result & 0x000f0000) >> 16;
			printf("    - Threshold Temperature Select (TMPSEL): %u - %s\n", field, nvme_feature_temp_sel_to_string(field));
			printf("    - Temperature Threshold         (TMPTH): %u C\n", (result & 0x0000ffff) - 273);
			break;
		case NVME_FEAT_ERR_RECOVERY:
            printf("Error Recovery\n");
			printf("    - Deallocated or Unwritten Logical Block Error Enable (DULBE): %s\n", ((result & 0x00010000) >> 16) ? "Enabled":"Disabled");
			printf("    - Time Limited Error Recovery                          (TLER): %u ms\n", (result & 0x0000ffff) * 100);
			break;
		case NVME_FEAT_VOLATILE_WC:
            printf("Volatile Write Cache\n");
			printf("    - Volatile Write Cache Enable (WCE): %s\n", (result & 0x00000001) ? "Enabled":"Disabled");
			break;
		case NVME_FEAT_NUM_QUEUES:
            printf("Number of Queues\n");
			printf("    - Number of IO Completion Queues Allocated (NCQA): %u\n", ((result & 0xffff0000) >> 16) + 1);
			printf("    - Number of IO Submission Queues Allocated (NSQA): %u\n",  (result & 0x0000ffff) + 1);
			break;
		case NVME_FEAT_IRQ_COALESCE:
            printf("Interrupt Coalescing\n");
			printf("    - Aggregation Time     (TIME): %u ms\n", ((result & 0x0000ff00) >> 8) * 100);
			printf("    - Aggregation Threshold (THR): %u\n",  (result & 0x000000ff) + 1);
			break;
		case NVME_FEAT_IRQ_CONFIG:
            printf("Interrupt Vector Configuration\n");
			printf("    - Coalescing Disable (CD): %s\n", ((result & 0x00010000) >> 16) ? "True":"False");
			printf("    - Interrupt Vector   (IV): %u\n",  result & 0x0000ffff);
			break;
		case NVME_FEAT_WRITE_ATOMIC:
            printf("Write Atomicity Normal\n");
			printf("    - Disable Normal (DN): %s\n", (result & 0x00000001) ? "True":"False");
			break;
		case NVME_FEAT_ASYNC_EVENT:
            printf("Asynchronous Event Configuration\n");
			printf("    - Firmware Activation Notices     : %s\n", ((result & 0x00000200) >> 9) ? "Send async event":"Do not send async event");
			printf("    - Namespace Attribute Notices     : %s\n", ((result & 0x00000100) >> 8) ? "Send NameSpace Attribute Changed event":"Do not send NameSpace Attribute Changed event");
			printf("    - SMART / Health Critical Warnings: %s\n", (result & 0x000000ff) ? "Send async event":"Do not send async event");
			break;
		case NVME_FEAT_AUTO_PST:
            printf("Autonomous Power State Transition\n");
			printf("    - Autonomous Power State Transition Enable (APSTE): %s\n", (result & 0x00000001) ? "Enabled":"Disabled");
			show_auto_pst((struct nvme_auto_pst *)buf);
			break;
		case NVME_FEAT_HOST_MEM_BUF:
            printf("Host Memory Buffer\n");
			printf("    - Memory Return       (MR): %s\n", ((result & 0x00000002) >> 1) ? "True":"False");
			printf("    - Enable Host Memory (EHM): %s\n", (result & 0x00000001) ? "Enabled":"Disabled");
			show_host_mem_buffer((struct nvme_host_mem_buffer *)buf);
			break;
		case NVME_FEAT_SW_PROGRESS:
            printf("Software Progress Marker\n");
			printf("    - Pre-boot Software Load Count (PBSLC): %u\n", result & 0x000000ff);
			break;
		case NVME_FEAT_HOST_ID:
            printf("Host Identifier\n");
			ull =  buf[7]; ull <<= 8; ull |= buf[6]; ull <<= 8; ull |= buf[5]; ull <<= 8;
			ull |= buf[4]; ull <<= 8; ull |= buf[3]; ull <<= 8; ull |= buf[2]; ull <<= 8;
			ull |= buf[1]; ull <<= 8; ull |= buf[0];
			printf("    - Host Identifier (HOSTID):  %lX \n", ull);
            
            ull = (buf[7] << 56) | (buf[6] << 48) | (buf[5] << 40) | (buf[4] << 32) | (buf[3] << 24) | (buf[2] << 16) | (buf[1] << 8) | buf[0];
            
            printf("    - Host Identifier (HOSTID):  %lX \n", ull);
			break;
		case NVME_FEAT_RESV_MASK:
            printf("Reservation Notification Mask\n");
			printf("    - Mask Reservation Preempted Notification  (RESPRE): %s\n", ((result & 0x00000008) >> 3) ? "True":"False");
			printf("    - Mask Reservation Released Notification   (RESREL): %s\n", ((result & 0x00000004) >> 2) ? "True":"False");
			printf("    - Mask Registration Preempted Notification (REGPRE): %s\n", ((result & 0x00000002) >> 1) ? "True":"False");
			break;
		case NVME_FEAT_RESV_PERSIST:
            printf("Reservation Persistance\n");
			printf("    - Persist Through Power Loss (PTPL): %s\n", (result & 0x00000001) ? "True":"False");
			break;
		case NVME_FEAT_ENDRC_MANAG_DISABLE:
            printf("Get Endurance Management Disable Setting\n");
			printf("    - Endurance Management Disabled Setting: %s\n", (result & 0x00000001) ? "Disabled":"Enabled");
			break;
		case NVME_FEAT_MAX_LBA:
            printf("Get Max LBA\n");
			break;
		case NVME_FEAT_NATIVE_MAX_LBA:
            printf("Get Native Max LBA\n");
			break;
		case NVME_FEAT_POWER_GOVERNOR:
            printf("Get Power Governor Setting\n");
			field = result & 0x000000ff;
			printf("    - Power Governor Setting: ");
			if (field == 0x0)
				printf("25W\n");
			else if (field == 0x1)
				printf("20W\n");
			else if (field == 0x2)
				printf("15W\n");
			else
				printf("Unknown\n");
			break;
		case NVME_FEAT_SMB_ASIC_ADDR:
            printf("Get SMB ASIC Address\n");
            printf("\n");
			field = (result & 0x000003FE) >> 1;
			printf("SMB Controller Address: %u", field);
			if (field != 0xFF)
				printf("\n");
			else
				printf(" (FW disables SMB ASIC access)\n");
			break;
		case NVME_FEAT_BLINK_ACTLED:
            printf("Get Blink Activity for LED\n");
			field = result >> 24;
			value =  buf[3]; value <<= 8; value |= buf[2]; value <<= 8; 
			value |= buf[1]; value <<= 8; value |= buf[0];

			if (field == 0) //Feature Option
			{
				printf("LED state while host is inactive: ");
				printf("%s\n", value ? "ON":"OFF");
			}
			else if (field == 1)
			{
				printf("LED duration increment size: ");
				printf("%s\n", value ? "50ms":"25ms");

			}
			else if (field == 2)
			{
				printf("OFF duration during IO activity in 25ms/50ms increments: ");
				if (value == 0)
					printf("Solid ON\n");
				else
					printf("%u\n", value);
			}
			else if (field == 3)
			{
				printf("ON duration during IO activity in 25ms/50ms increments: ");
				if (value == 0)
					printf("Match OFF duration\n");
				else
					printf("%u\n", value);

			}
			else if (field == 4)
			{
				printf("OFF duration during format activity in 25ms/50ms increments: ");
				if (value == 0)
					printf("Solid ON\n");
				else
					printf("%u\n", value);

			}
			else if (field == 5)
			{
				printf("ON duration during format activity in 25ms/50ms increments: ");
				if (value == 0)
					printf("Match OFF duration\n");
				else
					printf("%u\n", value);
			}
			break;
		case NVME_FEAT_ENABLE_LATENCY_TRACK:
            printf("Get Enable Latency Tracking\n");
			break;
        default:
            break;
	}
}
*/
