#include <algorithm>
#include "PSUOperation.hpp"

PSUOperation::PSUOperation(OPERATION_TYPE   in_eOpType,
                           string           in_strHelp) : OperationBaseClass(in_eOpType, in_strHelp)
{
}

PSUOperation::PSUOperation(COMMAND_TYPE     in_eCmdType,
                           OPERATION_TYPE   in_eOpType,
                           string           in_strHelp) : OperationBaseClass(in_eOpType, in_strHelp)
{
    m_eCommandType = in_eCmdType;
}

PSUOperation::~PSUOperation()
{
  
}

int
PSUOperation::executeFANGet(StringParser in_parser)
{
    int             count;
    PSUDAPI         psuDAPI("PSUDAPI");
    ErrorMsg        *pErrorMsg = NO_ERROR_MSG;
    string          strP1, strSpeed;

    // Check syntax
    in_parser.getParameterNumber(&count);
    if (count != 1)
    {
        showOperationHelp();
        return -1;
    }

    if ((pErrorMsg = psuDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }

    in_parser.getParameter(&strP1, 0);

    pErrorMsg = psuDAPI.getDataByElement(PSU_FAN, strP1, &strSpeed);
    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    cout << strSpeed << endl;

    return 0;
}

int
PSUOperation::executeFANRange(StringParser in_parser)
{
    int             count;
    PSUDAPI         psuDAPI("PSUDAPI");
    ErrorMsg        *pErrorMsg = NO_ERROR_MSG;
    string          strP1;
    list<string>    valueList;

    // Check syntax
    in_parser.getParameterNumber(&count);
    if (count != 1)
    {
        showOperationHelp();
        return -1;
    }

    if ((pErrorMsg = psuDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }

    in_parser.getParameter(&strP1, 0);

    pErrorMsg = psuDAPI.getSupportValueByElement(PSU_FAN, strP1, &valueList);
    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    if (valueList.size() == 2)
    {
        cout << valueList.front();
        valueList.pop_front();
        cout << " - ";
        cout << valueList.front() << endl;
    }
    else
    {
        while (!valueList.empty())
        {
            cout << valueList.front() << endl;
            valueList.pop_front();
        }
    }

    return 0;
}

int
PSUOperation::executeFANSet(StringParser in_parser)
{
    int             count;
    PSUDAPI         psuDAPI("PSUDAPI");
    ErrorMsg        *pErrorMsg = NO_ERROR_MSG;
    string          strP1, strP2, strValue;

    // Check syntax
    in_parser.getParameterNumber(&count);
    if (count != 2)
    {
        showOperationHelp();
        return -1;
    }

    if ((pErrorMsg = psuDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }

    in_parser.getParameter(&strP1, 0);
    in_parser.getParameter(&strP2, 1);

    pErrorMsg = psuDAPI.setDataByElement(PSU_FAN, strP2, strP1);
    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    pErrorMsg = psuDAPI.getDataByElement(PSU_FAN, strP2, &strValue);
    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    cout << strValue << endl;

    return 0;
}

int
PSUOperation::executeFWProgram(StringParser in_parser)
{
    int             count;
    PSUDAPI         psuDAPI("PSUDAPI");
    ErrorMsg        *pErrorMsg = NO_ERROR_MSG;
    string          strP1, strP2, strP3;

    // Check syntax
    in_parser.getParameterNumber(&count);
    if (count != 3)
    {
        showOperationHelp();
        return -1;
    }

    if ((pErrorMsg = psuDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }

    in_parser.getParameter(&strP1, 0);
    in_parser.getParameter(&strP2, 1);
    in_parser.getParameter(&strP3, 2);

    pErrorMsg = psuDAPI.updateFWByFile(strP3, std::stoi(strP1, NULL, 0), strP2);
    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    return 0;
}

int
PSUOperation::executeLEDGet(StringParser in_parser)
{
    int             count;
    PSUDAPI         psuDAPI("PSUDAPI");
    ErrorMsg        *pErrorMsg = NO_ERROR_MSG;
    string          strP1, strStatus;

    // Check syntax
    in_parser.getParameterNumber(&count);
    if (count != 1)
    {
        showOperationHelp();
        return -1;
    }

    if ((pErrorMsg = psuDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }

    in_parser.getParameter(&strP1, 0);

    pErrorMsg = psuDAPI.getDataByElement(PSU_LED, strP1, &strStatus);
    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    std::transform(strStatus.begin(), strStatus.begin(), strStatus.begin(), ::toupper);

    cout << strStatus << endl;

    return 0;
}

int
PSUOperation::executeLEDSet(StringParser in_parser)
{
    int             count;
    PSUDAPI         psuDAPI("PSUDAPI");
    ErrorMsg        *pErrorMsg = NO_ERROR_MSG;
    string          strP1, strP2, strStatus;

    // Check syntax
    in_parser.getParameterNumber(&count);
    if (count != 2)
    {
        showOperationHelp();
        return -1;
    }

    if ((pErrorMsg = psuDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }

    in_parser.getParameter(&strP1, 0);
    in_parser.getParameter(&strP2, 1);

    pErrorMsg = psuDAPI.setDataByElement(PSU_LED, strP2, strP1);
    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    pErrorMsg = psuDAPI.getDataByElement(PSU_LED, strP2, &strStatus);
    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    std::transform(strStatus.begin(), strStatus.begin(), strStatus.begin(), ::toupper);

    cout << strStatus << endl;

    return 0;
}

int
PSUOperation::executeListOperations(StringParser in_parser)
{
    int             count;
    PSUDAPI         psuDAPI("PSUDAPI");
    ErrorMsg        *pErrorMsg = NO_ERROR_MSG;
    string          strOperation;
    list<string>    deviceList;

    // Check syntax
    in_parser.getParameterNumber(&count);
    if (count != 0)
    {
        showOperationHelp();
        return -1;
    }

    if ((pErrorMsg = psuDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }

    getName(&strOperation);

    pErrorMsg = psuDAPI.getDeviceList(strOperation, &deviceList);
    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    while (!deviceList.empty())
    {
        cout << deviceList.front() << endl;
        deviceList.pop_front();
    }

    return 0;
}

int
PSUOperation::executeOperation(StringParser in_parser)
{
    string  strParameter;
    int     error = 0;

    in_parser.getParameter(&strParameter, 0);

    if (strParameter == HELP)
    {
        showOperationHelp();
        return 0;
    }

    switch (getOperationType())
    {
        // Operations of List
        case OPERATION_PSU:
        case OPERATION_FAN:
        case OPERATION_LED:
        case OPERATION_TEMP:
        case OPERATION_THRESHOLD:
            return executeListOperations(in_parser);
        // Operations of Show
        case OPERATION_ALL:
        case OPERATION_PROTOCOL_TYPE:
        case OPERATION_VENDOR:
        case OPERATION_FW:
        case OPERATION_PN:
        case OPERATION_SN:
        case OPERATION_MAX_POWER:
            return executeShowOperations(in_parser);
        // Operations of Fan, Led, Temp
        case OPERATION_GET:
            switch (m_eCommandType)
            {
                case CMD_FAN:
                    return executeFANGet(in_parser);
                case CMD_LED:
                    return executeLEDGet(in_parser);
                case CMD_TEMP:
                    return executeTemperatureGet(in_parser);
                default:
                    break;
            }
            break;
        case OPERATION_SET:
            switch (m_eCommandType)
            {
                case CMD_FAN:
                    return executeFANSet(in_parser);
                case CMD_LED:
                    return executeLEDSet(in_parser);
                case CMD_TEMP:
                    return executeTemperatureSet(in_parser);
                default:
                    break;
            }
            break;
        case OPERATION_RANGE:
            return executeFANRange(in_parser);
        case OPERATION_PROGRAM:
            return executeFWProgram(in_parser);
        default:
            break;
    }

    return error;
}

int
PSUOperation::executeShowOperations(StringParser in_parser)
{
    int             count;
    PSUDAPI         psuDAPI("PSUDAPI");
    ErrorMsg        *pErrorMsg = NO_ERROR_MSG;
    string          strP1, strValue;
    list<string>    valueList;

    // Check syntax
    in_parser.getParameterNumber(&count);
    if (count != 1)
    {
        showOperationHelp();
        return -1;
    }

    if ((pErrorMsg = psuDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }

    in_parser.getParameter(&strP1, 0);

    switch (getOperationType())
    {
        case OPERATION_ALL:
            if ((pErrorMsg = psuDAPI.getDataByElement(PSU_VENDOR, strP1, &strValue)) != NO_ERROR_MSG)
            {
                break;
            }

            strValue = "Vendor:\t\t" + strValue;
            valueList.push_back(strValue);

            if ((pErrorMsg = psuDAPI.getDataByElement(PSU_TYPE, strP1, &strValue)) != NO_ERROR_MSG)
            {
                break;
            }

            strValue = "Type:\t\t" + strValue;
            valueList.push_back(strValue);

            if ((pErrorMsg = psuDAPI.getDataByElement(PSU_FW, strP1, &strValue)) != NO_ERROR_MSG)
            {
                break;
            }

            strValue = "FW:\t\t" + strValue;
            valueList.push_back(strValue);

            if ((pErrorMsg = psuDAPI.getDataByElement(PSU_PART_NUMBER, strP1, &strValue)) != NO_ERROR_MSG)
            {
                break;
            }

            strValue = "PN:\t\t" + strValue;
            valueList.push_back(strValue);

            if ((pErrorMsg = psuDAPI.getDataByElement(PSU_SERIAL_NUMBER, strP1, &strValue)) != NO_ERROR_MSG)
            {
                break;
            }

            strValue = "SN:\t\t" + strValue;
            valueList.push_back(strValue);

            if ((pErrorMsg = psuDAPI.getDataByElement(PSU_MAX_POWER, strP1, &strValue)) != NO_ERROR_MSG)
            {
                break;
            }

            strValue = "Max Power:\t" + strValue + " W";
            valueList.push_back(strValue);
            break;
        case OPERATION_PROTOCOL_TYPE:
            pErrorMsg = psuDAPI.getDataByElement(PSU_TYPE, strP1, &strValue);
            valueList.push_back(strValue);
            break;
        case OPERATION_VENDOR:
            pErrorMsg = psuDAPI.getDataByElement(PSU_VENDOR, strP1, &strValue);
            valueList.push_back(strValue);
            break;
        case OPERATION_FW:
            pErrorMsg = psuDAPI.getDataByElement(PSU_FW, strP1, &strValue);
            valueList.push_back(strValue);
            break;
        case OPERATION_PN:
            pErrorMsg = psuDAPI.getDataByElement(PSU_PART_NUMBER, strP1, &strValue);
            valueList.push_back(strValue);
            break;
        case OPERATION_SN:
            pErrorMsg = psuDAPI.getDataByElement(PSU_SERIAL_NUMBER, strP1, &strValue);
            valueList.push_back(strValue);
            break;
        case OPERATION_MAX_POWER:
            pErrorMsg = psuDAPI.getDataByElement(PSU_MAX_POWER, strP1, &strValue);
            strValue += " W";
            valueList.push_back(strValue);
            break;
        default:
            break;
    }

    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    while (!valueList.empty())
    {
        cout << valueList.front() << endl;
        valueList.pop_front();
    }

    return 0;
}

int
PSUOperation::executeTemperatureGet(StringParser in_parser)
{
    int             count;
    PSUDAPI         psuDAPI("PSUDAPI");
    ErrorMsg        *pErrorMsg = NO_ERROR_MSG;
    string          strP1, strValue;

    // Check syntax
    in_parser.getParameterNumber(&count);
    if (count != 1)
    {
        showOperationHelp();
        return -1;
    }

    if ((pErrorMsg = psuDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }

    in_parser.getParameter(&strP1, 0);

    if (strP1.find("threshold") == std::string::npos)
    {
        pErrorMsg = psuDAPI.getDataByElement(PSU_TEMPERATURE, strP1, &strValue);
    }
    else
    {
        pErrorMsg = psuDAPI.getDataByElement(PSU_TEMPERATURE_THRESHOLD, strP1, &strValue);
    }

    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    strValue.erase((strValue.find('.') + 3));

    cout << strValue << endl;

    return 0;
}

int
PSUOperation::executeTemperatureSet(StringParser in_parser)
{
    int             count;
    PSUDAPI         psuDAPI("PSUDAPI");
    ErrorMsg        *pErrorMsg = NO_ERROR_MSG;
    string          strP1, strP2, strValue;

    // Check syntax
    in_parser.getParameterNumber(&count);
    if (count != 2)
    {
        showOperationHelp();
        return -1;
    }

    if ((pErrorMsg = psuDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }

    in_parser.getParameter(&strP1, 0);
    in_parser.getParameter(&strP2, 1);

    pErrorMsg = psuDAPI.setDataByElement(PSU_TEMPERATURE_THRESHOLD, strP2, strP1);
    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    pErrorMsg = psuDAPI.getDataByElement(PSU_TEMPERATURE_THRESHOLD, strP2, &strValue);
    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    strValue.erase((strValue.find('.') + 3));

    cout << strValue << endl;

    return 0;
}

void
PSUOperation::showOperationHelp()
{
    string strOpName, strHelp;

    getName(&strOpName);
    getHelp(&strHelp);

    cout << strHelp << endl;
    cout << "Syntax " << strOpName << " ";

    switch (getOperationType())
    {
        // Operations of List
        case OPERATION_PSU:
        case OPERATION_FAN:
        case OPERATION_LED:
        case OPERATION_TEMP:
        case OPERATION_THRESHOLD:
            cout << endl;
            break;
        // Operations of Show
        case OPERATION_ALL:
        case OPERATION_PROTOCOL_TYPE:
        case OPERATION_VENDOR:
        case OPERATION_FW:
        case OPERATION_PN:
        case OPERATION_SN:
        case OPERATION_MAX_POWER:
            cout << "P1" << endl;
            cout << "P1: PSU name" << endl;
            break;
        // Operations of Fan, Led, Temp
        case OPERATION_GET:
            switch (m_eCommandType)
            {
                case CMD_FAN:
                    cout << "P1" << endl;
                    cout << "P1: FAN device name" << endl;
                    break;
                case CMD_LED:
                    cout << "P1" << endl;
                    cout << "P1: LED name" << endl;
                    break;
                case CMD_TEMP:
                    cout << "P1" << endl;
                    cout << "P1: Temperature sensor name/Threshold name" << endl;
                    break;
                default:
                    break;
            }
            break;
        case OPERATION_SET:
            switch (m_eCommandType)
            {
                case CMD_FAN:
                    cout << "P1 P2" << endl;
                    cout << "P1: FAN speed" << endl;
                    cout << "P2: FAN device name" << endl;
                    break;
                case CMD_LED:
                    cout << "P1 P2" << endl;
                    cout << "P1: On/Off" << endl;
                    cout << "P2: LED name" << endl;
                    break;
                case CMD_TEMP:
                    cout << "P1 P2" << endl;
                    cout << "P1: Value(Dec, degree Celsius)" << endl;
                    cout << "P2: Threshold name" << endl;
                    break;
                default:
                    break;
            }
            break;
        case OPERATION_RANGE:
            cout << "P1" << endl;
            cout << "P1: FAN device name" << endl;
            break;
        case OPERATION_PROGRAM:
            cout << "P1 P2 P3" << endl;
            cout << "P1: Region number(Dec)" << endl;
            cout << "P2: File path" << endl;
            cout << "P3: PSU name" << endl;
            break;
        default:
            break;
    }
}
