#include "RTCCommand.hpp"

RTCCommand::RTCCommand(COMMAND_TYPE commandType, string helpContent) : CommandBaseClass(commandType, helpContent)
{
    //Base on command type to add different operations
    switch (commandType)
    {
        case CMD_SET:
            addOperation(new RTCOperation(OPERATION_DATE,       "Set Date Only"));
            addOperation(new RTCOperation(OPERATION_TIME,       "Set Time Only"));
            addOperation(new RTCOperation(OPERATION_DATETIME,   "Set Date and Time"));
            break;
        case CMD_TEST:
            addOperation(new RTCOperation(OPERATION_ACCURACY,   "RTC Accuracy Test"));
            //!! addOperation(new RTCOperation(OPERATION_WDT,        "Watch Dog Timer Test"));
            break;
        default:
            break;
    }
}

RTCCommand::~RTCCommand()
{

}

int
RTCCommand::executeCommand(StringParser commandLine)
{
    COMMAND_TYPE    commandType = NO_COMMAND;
    
    commandLine.getCommandType(&commandType);

    switch (commandType)
    {
        case CMD_SET:
        case CMD_TEST:
            return findAndExecuteOperation(commandLine);
            
        case CMD_SHOW:
            return show(commandLine);
        default:
            cout << UNKNOWN_COMMAND_MSG << endl;
            break;
    }

    return -1;
}


int
RTCCommand::show(StringParser commandLine)
{
    RTCDAPI     RTCDAPI("RTCDAPI");
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    
    if ((pErrorMsg = RTCDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }
    
    //Get  RTC device data by API
    pErrorMsg = RTCDAPI.showDateTime();
	if (pErrorMsg)
	{
		printError(pErrorMsg);
		return -1;
	}

    return 0;
}

