#include <string>
#include <algorithm>
#include "UARTCommand.hpp"
#include "UARTOperation.hpp"
       

UARTCommand::UARTCommand(COMMAND_TYPE eCommandType, string strHelpContent) : CommandBaseClass(eCommandType, strHelpContent)
{
    m_pUartDAPI     = NULL;

    switch(eCommandType)
    {
        case CMD_SHOW:
            addOperation(new UARTOperation(OPERATION_ALL, "Show all information of an UART"));
            addOperation(new UARTOperation(OPERATION_NODE, "Show device node of an UART"));
            addOperation(new UARTOperation(OPERATION_UART_PORT_TYPE, "Show port type of an UART"));
            addOperation(new UARTOperation(OPERATION_PORT, "Show io port address of an UART"));
            addOperation(new UARTOperation(OPERATION_SHOW_IRQ, "Show irq of an UART"));
            break;
        case CMD_TEST:
            addOperation(new UARTOperation(OPERATION_SCRATCHPAD_TEST, "do scratchpad test"));
            addOperation(new UARTOperation(OPERATION_INTERNAL_LOOPBACK_TEST, "do internal looback test"));
            break;
        default:
            break;
    }
}   

UARTCommand::~UARTCommand()
{
    if(m_pUartDAPI != NULL)
    {
        delete m_pUartDAPI;
        m_pUartDAPI = NULL;
    }

}

int 
UARTCommand::List()
{
    vector<SERIAL_INFO>             uartList;
    vector<SERIAL_INFO>::iterator   it;

    m_pUartDAPI = new UARTDAPI("UartDAPI");

    m_pErrorMsg = m_pUartDAPI->getUartList(&uartList);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return -1;
    }

    for(it = uartList.begin(); it != uartList.end(); ++it)
    {
        transform(it->strUartName.begin(), it->strUartName.end(), it->strUartName.begin(), ::toupper);
        cout << it->strUartName << endl;    
    }


    delete m_pUartDAPI;
    m_pUartDAPI = NULL;

    return 0;
}

int 
UARTCommand::Run(StringParser    in_classCommandLine)
{
    int     s32Index;
    int     s32Result = 0;
    string  strTemp;
    string  strOperationName;

    in_classCommandLine.getOperation(&strOperationName);

    //printf("UARTCommand::Run, strOperationName: %s\n", strOperationName.c_str());//check 

    for(s32Index = 0; s32Index < m_operation_count; s32Index++)
    {
        m_operation[s32Index]->getName(&strTemp);
        //printf("UARTCommand::Run, m_operation[%d]: %s\n", s32Index, strTemp.c_str());//check 
        if(strTemp == strOperationName)
        {
            s32Result = m_operation[s32Index]->executeOperation(in_classCommandLine);
            break;
        }
    }

    if(s32Index == m_operation_count)
    {
        cout << strOperationName << " operation is not found" << endl;
        listOperation(in_classCommandLine);
        s32Result = -1;
    }

    return s32Result;
}

int
UARTCommand::executeCommand(StringParser in_classCommandLine)
{
    //cout << "UARTCommand::executeCommand" << endl; //check 

    string          strOperationName;
    COMMAND_TYPE    eCommandType = NO_COMMAND;


    in_classCommandLine.getCommandType(&eCommandType);
    switch (eCommandType)
    {
        case CMD_LIST:
            return List();
        case CMD_SHOW:
        case CMD_TEST:
            return findAndExecuteOperation(in_classCommandLine);

        default:
            cout << UNKNOWN_COMMAND_MSG << endl;
            return -1;
    }
   
    /*
    in_classCommandLine.getOperation(&strOperationName);
    if(strOperationName == "help" || strOperationName == "")
    {
        listOperation(in_classCommandLine);
        return -1;
    }
           
    m_s32Result = Run(in_classCommandLine);
    if(eCommandType == CMD_TEST && m_s32Result == 0)
    {
        cout << "Pass" << endl;
    }
    return m_s32Result;
    //return Run(in_classCommandLine);
    */
}


