#include "UARTOperation.hpp"

UARTOperation::UARTOperation(OPERATION_TYPE   in_eOprationType,
                             string           in_strHelpContent) : OperationBaseClass(in_eOprationType, in_strHelpContent)
{
    m_pUartDAPI = new UARTDAPI("uartDAPI");
}

UARTOperation::~UARTOperation()
{
    if(m_pUartDAPI != NULL)
    {
        delete m_pUartDAPI;
        m_pUartDAPI = NULL;
    }
}


int 
UARTOperation::doInternalLoopbackTest()
{
    //cout << "UARTOperation::doInternalLoopbackTest" << endl; //check 

    m_pErrorMsg = m_pUartDAPI->doInternalLoopbackTest(m_strParameter[0].c_str(),
                                                      false, //polling mode 
                                                      2048,  //buffer size
                                                      false, //use default pattern
                                                      UART_UNUSED);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(UART_DEBUG_LEVEL, m_pErrorMsg);
        return -1;
    }
    return 0;
}


int 
UARTOperation::testScratchPad()
{
    m_pErrorMsg = m_pUartDAPI->testScratchPad(m_strParameter[0].c_str());
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(UART_DEBUG_LEVEL, m_pErrorMsg);
        return -1;
    }
    return 0;;
}


int 
UARTOperation::showUartInfo()
{
    //cout << "UARTOperation::showUartInfo" << endl; //check 

    int             s32Result;
    SERIAL_INFO     uartInfo;
    ErrorMsg*       pErrorMsg;

    m_pErrorMsg = m_pUartDAPI->getUartInfoByName(m_uartName, &uartInfo);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(UART_DEBUG_LEVEL, m_pErrorMsg);
        return -1;
    }
    if(getOperationType() == OPERATION_ALL)
    {
        showIndividualItem(uartInfo, OPERATION_NODE);
        showIndividualItem(uartInfo, OPERATION_UART_PORT_TYPE);
        showIndividualItem(uartInfo, OPERATION_PORT);
        showIndividualItem(uartInfo, OPERATION_SHOW_IRQ);
    }
    else
    {
        s32Result = showIndividualItem(uartInfo, getOperationType());
    }

    return s32Result;
}


int 
UARTOperation::showIndividualItem(SERIAL_INFO       in_uartInfo,
                                  OPERATION_TYPE    in_eOprationType)
{
    //cout << "UARTOperation::showIndividualItem" << endl; //check 

    switch(in_eOprationType)
    {
        case OPERATION_NODE:
            cout << "Node:\t" << in_uartInfo.strUartNodeName << endl;
            break;
        case OPERATION_UART_PORT_TYPE:
            cout << "Type:\t" << in_uartInfo.strType << endl;
            //printf("type number: %d\n", in_uartInfo.serialInfo.type); //check 
            break;
        case OPERATION_PORT:
            printf("Port:\t0x%x\n", in_uartInfo.serialInfo.port);
            break;
        case OPERATION_SHOW_IRQ:
            cout << "IRQ:\t" << in_uartInfo.serialInfo.irq << endl;
            break;
        default:
            return -1;
    }

    return 0;
}


int
UARTOperation::executeOperation(StringParser in_classCommandLine)
{
    m_s32Result = -1;
    
    in_classCommandLine.getParameter(&m_strParameter[0], 0);
    if (m_strParameter[0] == HELP)
    {
        showHelp();
        return m_s32Result;
    }

    m_s32Result = checkSyntax(in_classCommandLine);
    if(m_s32Result != 0)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return m_s32Result;
    }
    
    sprintf(m_uartName, "%s", m_strParameter[0].c_str());
    //printf("m_uartName: %s\n", m_uartName);//check 
    in_classCommandLine.convertToLowcase(m_uartName);

    switch(getOperationType())
    {
        case OPERATION_ALL:
        case OPERATION_NODE:
        case OPERATION_UART_PORT_TYPE:
        case OPERATION_PORT:
        case OPERATION_SHOW_IRQ:
            return showUartInfo();
        
        case OPERATION_SCRATCHPAD_TEST:
            return testScratchPad();
        case OPERATION_INTERNAL_LOOPBACK_TEST:
            return doInternalLoopbackTest();
        
        default:
            break;
    }

    return m_s32Result;
}


void 
UARTOperation::showHelp()
{
    string strHelpContent;

    getHelp(&strHelpContent);
    cout << strHelpContent << endl;
    cout << "Parameters required:" << endl;

    switch(getOperationType())
    {
        case OPERATION_ALL:
        case OPERATION_NODE:
        case OPERATION_UART_PORT_TYPE:
        case OPERATION_PORT:
        case OPERATION_SHOW_IRQ:
        case OPERATION_SCRATCHPAD_TEST:
        case OPERATION_INTERNAL_LOOPBACK_TEST:
            cout << "  P1: uart name" << endl;
            break;
        default:
            break;
    }
}

int 
UARTOperation::checkSyntax(StringParser in_classCommandLine)
{
    int     s32Index;
    int     s32ParameterAmount;
    int     u32Result = 0;

    in_classCommandLine.getParameterNumber(&s32ParameterAmount);
    for(s32Index = 0; s32Index < s32ParameterAmount; s32Index++)
    {
        in_classCommandLine.getParameter(&m_strParameter[s32Index], s32Index);
    }
    
    switch(getOperationType())
    {
        case OPERATION_ALL:
        case OPERATION_NODE:
        case OPERATION_UART_PORT_TYPE:
        case OPERATION_PORT:
        case OPERATION_SHOW_IRQ:
            if(s32ParameterAmount != 1)
            {
                u32Result = -1;
            }
            break;
        default:
            break;
    }

    return u32Result;
}

