#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/blkdev.h>
#include <linux/sched.h>
#include <linux/workqueue.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/syscalls.h>
#include <linux/proc_fs.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,19))
#include <linux/aer.h>
#endif

#include "../mpi/mpi2_type.h"
#include "../mpi/mpi2_sas.h"
#include "../mpt3sas_base.h"
#include "inc/sasdiag.h"
#include "inc/diag_mpt.h"
#include "inc/diag_debug.h"

/*
 * Extern Declaration
 */
extern struct MPT3SAS_ADAPTER *mpt_softc[];
/*
 * Global
 */
unsigned int g_expander_phy_error;
EXPORT_SYMBOL(g_expander_phy_error);
unsigned int g_ioc_phy_error=0;
EXPORT_SYMBOL(g_ioc_phy_error);
unsigned int g_board_ioc_stress_err=0;
EXPORT_SYMBOL(g_board_ioc_stress_err);
unsigned int g_last_stress_type=10;
EXPORT_SYMBOL(g_last_stress_type);
unsigned int g_hd_io_error;
unsigned int g_sas_hic_stress_err=0;
EXPORT_SYMBOL(g_sas_hic_stress_err);
unsigned int g_sas_stress_pattern;
unsigned int g_peer_stress_failure_hd_num;
EXPORT_SYMBOL(g_peer_stress_failure_hd_num);
unsigned int g_expansion_port_linked;
unsigned int g_sas_stress_soe;
unsigned int g_sas_start_socket_srv;
struct       proc_dir_entry *g_mpt_proc = NULL;
int          g_MPTIocPatterns_count = 4; /* Num IOC patterns to be tested */
int          WRITE_TO_NVSRAM = 0; /* Debug only */
int          g_stop_expander_loopback_test = 0;
U8           g_front_end_io_cable_ok;
U8           g_sasdiag_skip_device_event = 0;
U32          g_component_test_flag = 0;
int          g_bring_up_ioc[NUM_IOC_TOTAL] = { 0, 0, 0 };  /* Indicate SAS Driver loaded/insmod */
bool         g_sasHicPresent = FALSE;
U8           g_numIocOnboard = NUM_IOC_ON_BASE_CTRL;
U8           g_sasEnclType = 0;
int          g_sasdiag_dbg_flag = (DBG_TEST);

/* Memory test patterns, Defined IOC testing patterns */
u32 g_MPTIocPatterns[4] =
{
        0xFFFFFFFF,
        0x00000000,
        0xAAAAAAAA, //FXIME?
        0x55555555
};

U16 BYTE_SWAP_16(U16 Data)
{
    U8 temp;
    U8 *ptrTemp = (U8 *)&Data;

    temp = ptrTemp[1];
    ptrTemp[1] = ptrTemp[0];
    ptrTemp[0] = temp;

    return Data;
}

U32 BYTE_SWAP_32(U32 Data)
{
    U8 temp;
    U8 *ptrTemp = (U8 *)&Data;

    temp = ptrTemp[3];
    ptrTemp[3] = ptrTemp[0];
    ptrTemp[0] = temp;

    temp = ptrTemp[2];
    ptrTemp[2] = ptrTemp[1];
    ptrTemp[1] = temp;

    return Data;
}

/*check SAS IO UNIT Page0 - NegotiatedLinkRate to check all phy status each IOC */
int sasdiag_dgIocPhyTest(U8 ioc_num)
{
	struct MPT3SAS_ADAPTER *ioc;
	U8                     i;
	U16                    sz, ioc_status;
	Mpi2ConfigReply_t      mpi_reply;
	Mpi2SasIOUnitPage0_t   *sasIoUnitPg0 = NULL;
	U8					   total_ioc_num;
	/* I really do not like to write suck ugly code, but I can not avoid it because it most less modify.
	   mfg ask to implement HIC test in current arch...........    Orz  - Dirk*/

	/* if input is 5,  it mean to test all on board IOC*/
	if(ioc_num==5){
		total_ioc_num = 3;		
		ioc_num = 0;
	}
	/* it mean test input ioc only */
	else{
		total_ioc_num = ioc_num + 1;
	}
	
	/* error handle */
	if(ioc_num>=6){
		printk(" ioc number is wrong ! Test FAIL... \n");
		return -1;
	}	
#
	/* allocate sas io unit pg0 memory */
	sz = offsetof(Mpi2SasIOUnitPage0_t, PhyData) +
		(MAX_IOC_PHYS * sizeof(Mpi2SasIOUnit0PhyData_t));

	for( ; ioc_num < total_ioc_num; ioc_num++)
	{
		ioc = mpt_softc[ioc_num] ;
		if(ioc == NULL){
			printk(" can not get correct IOC stuct... Test FAIL... \n ");
			return -1 ;
		}
		
		if ((sasIoUnitPg0 = kzalloc(sz, GFP_KERNEL)) == NULL) {
			printk(" fail to get sasIoUnitPg0 memory in sasdiag_dgIocPhyTest... \n ");
			return -1;
		}
		
		/* read sas io unit pg0  */
		if (!(mpt3sas_config_get_sas_iounit_pg0(ioc, &mpi_reply, sasIoUnitPg0, sz))) {
			ioc_status = le16_to_cpu(mpi_reply.IOCStatus) & MPI2_IOCSTATUS_MASK;
			if (ioc_status != MPI2_IOCSTATUS_SUCCESS){
				printk("IOC[%d] :get sas io unit pg0 , but ioc status is not ok... Test FAIL...\n", ioc_num);
				kfree(sasIoUnitPg0);
				return -1;
			}
		} else {
			printk(" IOC[%d] :fail to read sas io unit page0 in sasdiag_dgIocPhyTest... \n  ",ioc_num);
			kfree(sasIoUnitPg0);
			return -1;
		}

		for (i = 0; i < sasIoUnitPg0->NumPhys; i++) {
			if ((sasIoUnitPg0->PhyData[i].NegotiatedLinkRate == MPT_IOC_LINK_RATE_DISABLED) ||
			     (sasIoUnitPg0->PhyData[i].NegotiatedLinkRate == 0)){
				printk(" IOC[%d] : Check phy %d status, this phy may disable or do not connect device...   \n ", ioc_num, i );
				printk(" IOC Phy test - FAIL...\n");
				kfree(sasIoUnitPg0);
				return -1;
			}	
		}
		printk("IOC[%d] : all the phy status is OK \n", ioc_num);
	}
	
	kfree(sasIoUnitPg0);
	printk(" IOC Phy test - PASS\n");
	return 0;
}

// Revisit: should remove or not? If not, rmmod/insmod causes kernel crash
// because the "lsi" entry already exist at insmod (the second time)
// to do: check if "lsi" entry exit, then not create it and not remove at rmmod
void sasdiag_proc_entry_remove(void)
{
	if (g_mpt_proc)
	{
		remove_proc_entry("lsi", 0);
		g_mpt_proc = NULL;
	}
}


