/// @file       BaseClassAPI.hpp
/// @brief      BaseClassAPI class
/// @details    Provide base class for API common function
/// @version    1.0.0
/// @date       2014.01.21
/// @author     KenHu
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#ifndef BASE_CLASS_API_H
#define BASE_CLASS_API_H

// Include standard library header files
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
// Include Foxconn's header files
#include "config.hpp"
#include "common.hpp"
#include "error_code.hpp"
#include "ErrorMsg.hpp"
#include "ResourceLock.hpp"
#include "CommonFunctions.hpp"

/// @class      BaseClassAPI BaseClassAPI.hpp "BaseClassAPI.hpp"
/// @brief      Common function for all module APIs 
/// @param      m_szClassName is a char string for API class name
/// @param      m_szTempString is a char string for temp usage
/// @param      m_pErrorMsg is a pointer to ErrorMsg for error message list
class BaseClassAPI
{
    public:
                            BaseClassAPI(STRING in_szClassName);
        virtual             ~BaseClassAPI(void);

        void                addErrorMsg(ERRORCODE   in_eErrorCode, 
                                        const CHAR* in_szMessage);

        void                addErrorMsg(ERRORCODE   in_eErrorCode,
                                        const CHAR* in_szMessage,
                                        ErrorMsg*   in_pNextErrorMsg);

        void                clearErrorMsg(void);

        STRING              getClassName(void);
        
        ErrorMsg*           getErrorMsg(void);
        
        ErrorMsg*           returnErrorMsg(ERRORCODE    in_eErrorCode, 
                                           const CHAR*  in_szMessage = NULL,
                                           ErrorMsg*    in_pNextErrorMsg = NULL);
        
        ResourceLock        m_resourceLock;
    protected:            
        CHAR                m_szClassName[(CLASS_NAME_LENGTH + 1)];
        ERRORCODE           m_errorCode;            
        CHAR                m_szTempString[(DEBUG_MESSAGE_LENGTH + 1)];
    private:
        ErrorMsg            *m_pErrorMsg;
};


#endif

