/// @file       BaseClassDAPI.cpp
/// @brief      BaseClassDAPI class
/// @details    Provide DAPI base class for all modules
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#include "BaseClassDAPI.hpp"

/// @fn         BaseClassDAPI::BaseClassDAPI(STRING in_szClassName) : BaseClassAPI(in_szClassName)
/// @brief      BaseClassDAPI constructor
/// @param      in_szClassName is a char string for class name
/// @exception  Nothing
/// @return     Nothing
BaseClassDAPI::BaseClassDAPI(STRING in_szClassName) : BaseClassAPI(in_szClassName)
{   
/*    // create the version string and place in the database if necessary
    if (m_globalDatabase != NULL)
    {
        if (m_globalDatabase->getStringElement("$DFWAPIVER") == NULL)
        {
            sprintf(m_tempString, "DFW API Version:    %s  %s  %s", DFW_API_VERSION_STRING, __DATE__, __TIME__);
            m_globalDatabase->modifyElement("$DFWAPIVER", m_tempString);
        }
    }*/

    m_u64StepCount = 0;
    m_u64TotalStep = 0;
}

/// @fn         BaseClassDAPI::~BaseClassDAPI(void)
/// @brief      BaseClassDAPI destructor
/// @exception  Nothing
/// @return     Nothing
BaseClassDAPI::~BaseClassDAPI(void)
{
    // Save the error code
    m_errorCode = NO_ERROR;
}

/// @fn         BaseClassDAPI::getProgress(void)
/// @brief      Get DAPI's progress
/// @exception  Nothing
/// @return     A 32bits value for progress of DAPI
INT32
BaseClassDAPI::getProgress(void)
{
    UINT64 u64Percetage = 0;
    
    if (m_u64TotalStep == 0)
    {
        u64Percetage = 100;
    }
    else
    {
        u64Percetage = (m_u64StepCount * 100) / m_u64TotalStep;
    }

    return (INT32)u64Percetage;
}

/// @fn         BaseClassDAPI::initProgress(UINT64 in_u64TotalStep)
/// @brief      Initialize progress setting of DAPI
/// @param      in_u64TotalStep is 64bits value for amount of total steps for computing progress
/// @exception  Nothing
/// @return     Nothing
void
BaseClassDAPI::initProgress(UINT64 in_u64TotalStep)
{
    m_u64StepCount = 0;
    m_u64TotalStep = in_u64TotalStep;
}

/// @fn         BaseClassDAPI::updateProgress(UINT64 in_u64FinishedStep)
/// @brief      Update DAPI's progress with new finished steps
/// @param      in_u64FinishedStep is 64bits value for amount of finished steps for computing progress
/// @exception  Nothing
/// @return     Nothing
void
BaseClassDAPI::updateProgress(UINT64 in_u64FinishedStep)
{
    m_u64StepCount += in_u64FinishedStep;
}

