#include "EthernetDAPI.hpp"

EthernetDAPI :: EthernetDAPI
    (
    STRING className
    ) : BaseClassDAPI(className)
{
    m_pErrorMsg = NO_ERROR_MSG;

    if ((m_EthernetDriver = new EthernetDDAPI("ethDrv")) == NULL)
        m_errorCode = (ERRORCODE)(ERROR_ETHERNET_DAPI_CONSTRUCTOR | ERROR_DIAGNOSTIC_API_CONSTRUCTOR1_ERROR);
}



EthernetDAPI :: ~EthernetDAPI
    (
    void
    )
{
    // Save the error code
    if (m_EthernetDriver != NULL)
        delete m_EthernetDriver;
    m_errorCode = NO_ERROR;
}



ErrorMsg* EthernetDAPI::ReadMacAddress(string  ethernet_name,
                                       string  *mac_address)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->GetEthernetMAC(ethernet_name, mac_address);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_READ_MAC, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::GetDeviceNames(list<struct ethernet_data>* ethernet_name_list)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->GetEthernetNamesList(ethernet_name_list);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_GET_DEVICE_NAMES, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::ReadIPAddress(CHAR   *ethernet_name,
                                      UCHAR* ip)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->GetEthernetIP(ethernet_name, ip);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_READ_IP, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::ReadIPMask(CHAR *ethernet_name,
                                   UCHAR* mask)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->GetEthernetMask(ethernet_name, mask);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_READ_MASK, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::ReadMTU(CHAR  *ethernet_name,
                                INT32 *mtu)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->GetEthernetMTU(ethernet_name, mtu);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_READ_MTU, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::ReadEEPROM(CHAR  *ethernet_name,
                                   UCHAR *data,
                                   INT32 data_counter)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->GetEthernetEEPROM(ethernet_name, data, data_counter);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_READ_EEPROM, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::ReadEEPROM(string ethernet_name,
                                   INT32  StartAddress,
                                   INT32  Length,
                                   INT32  *data)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->GetEthernetEEPROM(ethernet_name, StartAddress, Length, data);

    return m_pErrorMsg;
}



ErrorMsg* EthernetDAPI::ReadEEPROMSize(CHAR  *ethernet_name,
                                       INT32 *data_counter)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->GetEthernetEEPROMSize(ethernet_name, data_counter);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_READ_EEPROM_SIZE, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::EepromValidationTest(CHAR  *ethernet_name,
                                             INT32 *test_result)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->SendIoctl(ethernet_name, EepromValidation, test_result);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_EEPROM_VALIDATION_TEST, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::EepromValidationTest(string ethernet_name,
                                             INT32  *test_result)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->SendIoctl(ethernet_name, EepromValidation, test_result);

    return m_pErrorMsg;
}



ErrorMsg* EthernetDAPI::MDIRegisterTest(string ethernet_name,
                                        INT32  *test_result)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->SendIoctl(ethernet_name, LinkStatus, test_result);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_MDI_REGISTER_TEST, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::InterruptTest(string  ethernet_name,
                                      INT32   *test_result)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->SendIoctl(ethernet_name, Interrupt, test_result);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_INTERRUPT_TEST, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::InternalLoopbackTest(string  ethernet_name,
                                             INT32   *test_result)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->SendIoctl(ethernet_name, InternalLoopback, test_result);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_INTERNAL_LOOPBACK_TEST, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::WriteEEPROM(CHAR  *ethernet_name,
                                    INT32 offset,
                                    INT32 length,
                                    UCHAR *data)
{
    INT32 i;

    m_pErrorMsg = NO_ERROR_MSG;

    for (i=0; i<length ;i++)
    {
        m_pErrorMsg = m_EthernetDriver->SetEthernetEEPROM(ethernet_name, offset+i, data[i], 1);
        if (m_pErrorMsg != NO_ERROR_MSG)
        {
            return returnErrorMsg(ERROR_ETHERNET_DAPI_WRITE_EEPROM, NULL, m_pErrorMsg);
        }
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::WriteEEPROM(string ethernet_name,
                                    INT32  StartAddress,
                                    INT32  *data,
                                    INT32  Length)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->SetEthernetEEPROM(ethernet_name, StartAddress, data, Length);

    return m_pErrorMsg;
}



ErrorMsg* EthernetDAPI::WriteMacAddress(string  ethernet_name,
                                        string  *mac_address)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->SetEthernetMAC(ethernet_name, mac_address);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_WRITE_MAC, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::Extrenal_loopback(string nic, int speed, float *throughput, string *statistics)
{
//    PCIDDAPI                    PCI_driver("PCI_driver");
//    PFA                         PCI_BDF_NIC1;
//    list<PCI_ERROR>             pciErrorList;
//    list<PCI_ERROR>::iterator   it;
//    char                        AER_string[100];
//    int                         domain_1, bus_1, device_1, func_1;
//    int                         flag = 0, AER_flag = 0;


    //Get nic 1 PCI BDF
//    m_EthernetDriver->GetPCIBusDeviceFunction(nic, &domain_1, &bus_1, &device_1, &func_1);

//    PCI_BDF_NIC1.u16Domain  = domain_1;
//    PCI_BDF_NIC1.u16Bus     = bus_1;
//    PCI_BDF_NIC1.u16Device  = device_1;
//    PCI_BDF_NIC1.u8Function = func_1;
    //Clean nic 1 ARE
//    PCI_driver.clearErrors(PCI_BDF_NIC1);


    m_EthernetDriver = new EthernetDDAPI(nic.c_str());

    //Enable loopback
    m_pErrorMsg = m_EthernetDriver->Enable_external_loopback(nic);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
       return returnErrorMsg(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK, NULL, m_pErrorMsg);
    }

    //Setup the speed
    m_pErrorMsg = m_EthernetDriver->SetupSpeed(nic, speed);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
       return returnErrorMsg(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK, NULL, m_pErrorMsg);
    }

    //Start speed test
    m_pErrorMsg = m_EthernetDriver->Execute_external_loopback(nic, nic, throughput);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
       return returnErrorMsg(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK, NULL, m_pErrorMsg);
    }

    //Disable loopback
    m_pErrorMsg = m_EthernetDriver->Disable_external_loopback();
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
       return returnErrorMsg(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK, NULL, m_pErrorMsg);
    }

    if (*throughput == 0)
    {
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK | ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK_ABNORMAL), NULL, NULL);
    }

    //Get statistics
//    m_pErrorMsg = m_EthernetDriver->getStatistics(nic, statistics);
//    if (m_pErrorMsg != NO_ERROR_MSG)
//    {
//        flag = 1;
//        //return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK ), NULL, m_pErrorMsg);
//    }

    //Check nic 1 PCI AER
//    PCI_driver.getErrorStatistics(PCI_BDF_NIC1, AER_ERROR, &pciErrorList);
//    for (it = pciErrorList.begin(); it != pciErrorList.end(); ++it)
//    {
//        sprintf(AER_string, "PCI AER %s: 0x%08x\n", it->strErrorName.c_str(), it->u32Data);
//        statistics->append(AER_string);
//        if (it->u32Data != 0)
//        {
//            AER_flag = 1;
//            //sprintf(AER_string, "%s %s=0x%08x\n", nic.c_str(), it->strErrorName.c_str(), it->u32Data); 
//            //return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK), NLL, NULL);
//        }
//    }

    //Check statistics error
//    if ((m_pErrorMsg != NO_ERROR_MSG) || (flag != 0))
//    {
//       return returnErrorMsg(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK, NULL, m_pErrorMsg);
//    }

    //Check PCI AER error flag
//    if (AER_flag == 1)
//    {
//       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK), "PCI AER error", NULL);
//    }

    return NO_ERROR_MSG;
}


ErrorMsg* EthernetDAPI::Extrenal_loopback(string nic_1, string nic_2, int speed, float *throughput, string *statistics)
{
    float throughput_1, throughput_2;
    int   flag = 0, AER_flag = 0;
    char  command_string[50];

//    PCIDDAPI                    PCI_driver("PCI_driver");
//    PFA                         PCI_BDF_NIC1, PCI_BDF_NIC2;
//    list<PCI_ERROR>             pciErrorList;
//    list<PCI_ERROR>::iterator   it;
//    char                        AER_string[100];
//    int domain_1, bus_1, device_1, func_1;
//    int domain_2, bus_2, device_2, func_2;

    //Get nic 1 PCI BDF
//    m_EthernetDriver->GetPCIBusDeviceFunction(nic_1, &domain_1, &bus_1, &device_1, &func_1);

//    PCI_BDF_NIC1.u16Domain  = domain_1;
//    PCI_BDF_NIC1.u16Bus     = bus_1;
//    PCI_BDF_NIC1.u16Device  = device_1;
//    PCI_BDF_NIC1.u8Function = func_1;
    //Clean nic 1 ARE
//    PCI_driver.clearErrors(PCI_BDF_NIC1);

    //Get nic 2 PCI BDF
//    m_EthernetDriver->GetPCIBusDeviceFunction(nic_2, &domain_2, &bus_2, &device_2, &func_2);

//    PCI_BDF_NIC2.u16Domain  = domain_2;
//    PCI_BDF_NIC2.u16Bus     = bus_2;
//    PCI_BDF_NIC2.u16Device  = device_2;
//    PCI_BDF_NIC2.u8Function = func_2;
    //Clean nic 2 ARE
//    PCI_driver.clearErrors(PCI_BDF_NIC2);


    m_EthernetDriver = new EthernetDDAPI(nic_1.c_str());

    //Setup ethernet speed
    //m_pErrorMsg = Setup_speed(nic_1, speed, 0);
    m_pErrorMsg = m_EthernetDriver->SetupSpeed(nic_1, speed);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
       //Reset ethernet 1 MTU to 1500
       sprintf(command_string, "ifconfig %s mtu 1500\n", nic_1.c_str());
       system(command_string);

       return returnErrorMsg(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK, NULL, m_pErrorMsg);
    }

    //m_pErrorMsg = Setup_speed(nic_2, speed, 1);
    m_pErrorMsg = m_EthernetDriver->SetupSpeed(nic_2, speed);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
       //Reset ethernet 2 MTU to 1500
       sprintf(command_string, "ifconfig %s mtu 1500\n", nic_2.c_str());
       system(command_string);

       return returnErrorMsg(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK, NULL, m_pErrorMsg);
    }

    //Start speed test nic1 to nin2
    m_pErrorMsg = m_EthernetDriver->Execute_external_loopback(nic_1, nic_2, &throughput_1);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
       //Reset ethernet MTU to 1500
       sprintf(command_string, "ifconfig %s mtu 1500\n", nic_1.c_str());
       system(command_string);
       sprintf(command_string, "ifconfig %s mtu 1500\n", nic_2.c_str());
       system(command_string);

       return returnErrorMsg(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK, NULL, m_pErrorMsg);
    }

    if (throughput_1 == 0)
    {
       //Reset ethernet MTU to 1500
       sprintf(command_string, "ifconfig %s mtu 1500\n", nic_1.c_str());
       system(command_string);
       sprintf(command_string, "ifconfig %s mtu 1500\n", nic_2.c_str());
       system(command_string);

       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK | ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK_ABNORMAL), NULL, NULL);
    }

    sleep(1);
    //Start speed test nic2 to nin1
    m_pErrorMsg = m_EthernetDriver->Execute_external_loopback(nic_2, nic_1, &throughput_2);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
       //Reset ethernet MTU to 1500
       sprintf(command_string, "ifconfig %s mtu 1500\n", nic_1.c_str());
       system(command_string);
       sprintf(command_string, "ifconfig %s mtu 1500\n", nic_2.c_str());
       system(command_string);

       return returnErrorMsg(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK, NULL, m_pErrorMsg);
    }

    if (throughput_2 == 0)
    {
       //Reset ethernet MTU to 1500
       sprintf(command_string, "ifconfig %s mtu 1500\n", nic_1.c_str());
       system(command_string);
       sprintf(command_string, "ifconfig %s mtu 1500\n", nic_2.c_str());
       system(command_string);

       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK | ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK_ABNORMAL), NULL, NULL);
    }

    *throughput = (throughput_1 + throughput_2) / 2;

    //Get nic1 statistics
//    statistics->append(nic_1);
//    statistics->append(":\n");
//    m_pErrorMsg = m_EthernetDriver->getStatistics(nic_1, statistics);
//    if (m_pErrorMsg != NO_ERROR_MSG)
//    {
//        flag = 1;
//    }

    //Get nic 1 PCI AER
//    PCI_driver.getErrorStatistics(PCI_BDF_NIC1, AER_ERROR, &pciErrorList);
//    for(it = pciErrorList.begin(); it != pciErrorList.end(); ++it)
//    {
//        sprintf(AER_string, "PCI AER %s: 0x%08x\n", it->strErrorName.c_str(), it->u32Data);
//        statistics->append(AER_string);
//        if (it->u32Data != 0)
//        {
//            AER_flag = 1;
//            //sprintf(AER_string, "%s %s=0x%08x\n", nic_1.c_str(), it->strErrorName.c_str(), it->u32Data); 
//            //return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK), AER_string, NULL);
//        }
//    }
//    pciErrorList.clear();


    //Get nic2 statistics
//    statistics->append(nic_2);
//    statistics->append(":\n");
//    m_pErrorMsg = m_EthernetDriver->getStatistics(nic_2, statistics);

//    if ((m_pErrorMsg != NO_ERROR_MSG) || (flag != 0))
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
       //Reset ethernet MTU to 1500
       sprintf(command_string, "ifconfig %s mtu 1500\n", nic_1.c_str());
       system(command_string);
       sprintf(command_string, "ifconfig %s mtu 1500\n", nic_2.c_str());
       system(command_string);

       return returnErrorMsg(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK, NULL, m_pErrorMsg);
    }

    //Get nic 2 PCI AER
//    PCI_driver.getErrorStatistics(PCI_BDF_NIC2, AER_ERROR, &pciErrorList);
//    for(it = pciErrorList.begin(); it != pciErrorList.end(); ++it)
//    {
//        sprintf(AER_string, "PCI AER %s: 0x%08x\n", it->strErrorName.c_str(), it->u32Data);
//        statistics->append(AER_string);
//        if (it->u32Data != 0)
//        {
//            AER_flag = 1;
//            //sprintf(AER_string, "%s %s=0x%08x\n", nic_2.c_str(), it->strErrorName.c_str(), it->u32Data); 
//            //return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK), AER_string, NULL);
//        }
//    }

    //Check PCI AER error flag
//    if (AER_flag == 1)
//    {
//       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_EXTRENAL_LOOPBACK), "PCI AER error", NULL);
//    }

    //Reset ethernet MTU to 1500
    sprintf(command_string, "ifconfig %s mtu 1500\n", nic_1.c_str());
    system(command_string);
    sprintf(command_string, "ifconfig %s mtu 1500\n", nic_2.c_str());
    system(command_string);

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::Setup_speed(string nic, int speed, int wait_link_up)
{

   int duplex = 0x01;
   int err = 0, fd;
   string link_up = "No";
   int sleep_count = 0;

   struct ifreq ifr;
   struct ethtool_cmd ecmd;

   fd = socket(AF_INET,SOCK_DGRAM,0);
   if (fd < 0)
   {
      //perror("socket");
      close(fd);
      return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED | ERROR_ETHERNET_DAPI_SOCKET_OPEN_FAIL), strerror(errno), NULL);
   }

   strncpy(ifr.ifr_name, nic.c_str(), sizeof(ifr.ifr_name));

   ecmd.cmd = ETHTOOL_GSET;
   ifr.ifr_data = (__caddr_t)&ecmd;
   err = ioctl(fd, SIOCETHTOOL, &ifr);
   if (err < 0)
   {
      //perror("Cannot get current device settings");
      close(fd);
      return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED | ERROR_ETHERNET_DAPI_GET_ETHERNET_DRIVER_SET_FAIL), strerror(errno), NULL);
   }

   //Speed setup
   ecmd.speed = (__u16)speed;
   ecmd.speed_hi = (__u16)(speed >> 16);

   //Duplex setup
   ecmd.duplex = duplex;  //0x01 for FULL duplex, 0x00 Half

   //Autonegotiation setup. 1G MUST enable.
   if (speed >= 1000)
      ecmd.autoneg = 0x01; //disable autonegotiation.  If this is set to enable, the forced link modes are completely ignored.
   else
      ecmd.autoneg = 0x00;

   /* Try to perform the update. */
   ecmd.cmd = ETHTOOL_SSET;
   err = ioctl(fd, SIOCETHTOOL, &ifr); 
   if (err < 0)
   {
      //perror("Cannot set new settings");
      close(fd);
      return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED | ERROR_ETHERNET_DAPI_SET_ETHERNET_DRIVER_SET_FAIL), strerror(errno), NULL);
   }
   close(fd);

   while (link_up.compare("No") == 0 && wait_link_up == 1)
   {
      //printf("Wait link UP!!!\n");
      sleep(1);
      sleep_count++;
      if (sleep_count > 6)
      {
         return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED | ERROR_ETHERNET_DAPI_SETUP_SPEED_TIMEOUT), NULL, NULL);
      }
      GetLinkStatus(nic, &link_up);
   }

   return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::GetSupportedLinkModes(string nic, string *modes, int *mode_count)
{
   int err = 0, fd;
   unsigned int i;
   int mask = 0;

   static const struct {
                int value;
                const char *name;
        } mode_defs[] = {
                {  ADVERTISED_10baseT_Half,       "10baseT/Half" },
                {  ADVERTISED_10baseT_Full,       "10baseT/Full" },
                {  ADVERTISED_100baseT_Half,      "100baseT/Half" },
                {  ADVERTISED_100baseT_Full,      "100baseT/Full" },
                {  ADVERTISED_1000baseT_Half,     "1000baseT/Half" },
                {  ADVERTISED_1000baseT_Full,     "1000baseT/Full" },
                {  ADVERTISED_1000baseKX_Full,    "1000baseKX/Full" },
                {  ADVERTISED_2500baseX_Full,     "2500baseX/Full" },
                {  ADVERTISED_10000baseT_Full,    "10000baseT/Full" },
                {  ADVERTISED_10000baseKX4_Full,  "10000baseKX4/Full" },
                {  ADVERTISED_10000baseKR_Full,   "10000baseKR/Full" },
                {  ADVERTISED_20000baseMLD2_Full, "20000baseMLD2/Full" },
                {  ADVERTISED_20000baseKR2_Full,  "20000baseKR2/Full" },
                {  ADVERTISED_40000baseKR4_Full,  "40000baseKR4/Full" },
                {  ADVERTISED_40000baseCR4_Full,  "40000baseCR4/Full" },
                {  ADVERTISED_40000baseSR4_Full,  "40000baseSR4/Full" },
                {  ADVERTISED_40000baseLR4_Full,  "40000baseLR4/Full" },
        };


   struct ifreq ifr;
   struct ethtool_cmd ecmd;


   *mode_count = 0;
   errno = 0;

   fd = socket(AF_INET, SOCK_DGRAM, 0);
   if (fd < 0)
   {
      //perror("socket");
      return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_SUPPORT_LINK_MODE | ERROR_ETHERNET_DAPI_SOCKET_OPEN_FAIL), strerror(errno), NULL);
   }

   strncpy(ifr.ifr_name, nic.c_str(), sizeof(ifr.ifr_name));

   ecmd.cmd = ETHTOOL_GSET;
   ifr.ifr_data = (__caddr_t)&ecmd;
   err = ioctl(fd, SIOCETHTOOL, &ifr);
   if (err < 0 && errno != ENOTSUP)
   {
      //perror("Cannot get current device settings"); printf("%d\n",errno);
      close(fd);
      return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_SUPPORT_LINK_MODE | ERROR_ETHERNET_DAPI_IOCTL_FAIL), strerror(errno), NULL);
   }

   if (errno == ENOTSUP)
   {
      mask = 0;
   }
   else
   {
      mask = ecmd.supported;
   }
   //printf("0x%x\n",mask);


   for (i = 0; i < ARRAY_SIZE(mode_defs); i++)
   {
      if (mask & mode_defs[i].value)
      {
         (modes+*mode_count)->append(mode_defs[i].name);
         (*mode_count)++;
         //printf("%s \n", mode_defs[i].name);
      }
   }
   close(fd);

   return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::GetSpeedDuplex(string nic, string *speed_result)
{
   int err = 0, fd;
   int duplex = 0;
   char buffer[50];

   struct ifreq ifr;
   struct ethtool_cmd ecmd;

   errno = 0;

   fd = socket(AF_INET, SOCK_DGRAM, 0);
   if (fd < 0)
   {
      //perror("socket");
      return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_SUPPORT_LINK_MODE | ERROR_ETHERNET_DAPI_SOCKET_OPEN_FAIL), strerror(errno), NULL);
   }

    //Covert well know name to system real name
    //m_EthernetDriver->ConverWellKnowNameToRealName(nic, &nic);
   strncpy(ifr.ifr_name, nic.c_str(), sizeof(ifr.ifr_name));

   ecmd.cmd = ETHTOOL_GSET;
   ifr.ifr_data = (__caddr_t)&ecmd;
   err = ioctl(fd, SIOCETHTOOL, &ifr);
   if (err < 0 && errno != ENOTSUP)
   {
      //perror("Cannot get current device settings");
      close(fd);
      return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_SUPPORT_LINK_MODE | ERROR_ETHERNET_DAPI_IOCTL_FAIL), strerror(errno), NULL);
   }

   if(ethtool_cmd_speed(&ecmd) == 65535 || ethtool_cmd_speed(&ecmd) == -1)
   {
      sprintf(buffer, "Unknown!");
   }
   else
   {
      if (ethtool_cmd_speed(&ecmd) >= 1000)
      {
          sprintf(buffer, "%dGb", ethtool_cmd_speed(&ecmd) / 1000);
      }
      else
      {
          sprintf(buffer, "%dMb", ethtool_cmd_speed(&ecmd));
      }
   }
   speed_result->append(buffer);
   duplex = ecmd.duplex;

   //if (duplex == 0)
   //{
   //   sprintf(buffer, "Half");
   //   speed_result->append(buffer);
   //}
   //else
   //{
   //   sprintf(buffer, "Full");
   //   speed_result->append(buffer);
   //}

   if (errno == ENOTSUP)
   {
      speed_result->clear();
   }
   close(fd);

   return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::GetLinkStatus(string nic, string *status_result)
{
    int err = 0, fd;
    struct ifreq ifr;
    struct ethtool_value edata;

    errno = 0;

    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
    {
       //perror("socket");
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_LINK_STATUS | ERROR_ETHERNET_DAPI_SOCKET_OPEN_FAIL), strerror(errno), NULL);
    }

    //Covert well know name to system real name
    //m_EthernetDriver->ConverWellKnowNameToRealName(nic, &nic);
    strncpy(ifr.ifr_name, nic.c_str(), sizeof(ifr.ifr_name));


    edata.cmd = ETHTOOL_GLINK;
    ifr.ifr_data = (__caddr_t)&edata;

    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0 && errno != ENOTSUP)
    {
       //perror("Cannot get current device link status");
       close(fd);
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_LINK_STATUS | ERROR_ETHERNET_DAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

   //printf("%d\n",edata.data);
   //printf("Link detected:");
   if (edata.data == 1)
   {
      //printf("Yes\n");
      *status_result = "Up";
   }
   else
   {
      //printf("No\n");
      *status_result = "Down";
   }
   close(fd);
   return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::GetStatistics(string nic, struct Ethetnet_statistics *eth_statis)
{
   string node_path = "/sys/class/net/" + nic + "/statistics/";

   m_pErrorMsg = GetNodeValue(node_path + "rx_packets", &(eth_statis->rx_packets));
   if (m_pErrorMsg != NO_ERROR_MSG)
   {
      return returnErrorMsg(ERROR_ETHERNET_DAPI_GET_STATISTICS, "rx_packets", m_pErrorMsg);
   }
   m_pErrorMsg = GetNodeValue(node_path + "rx_errors", &(eth_statis->rx_errors));
   if (m_pErrorMsg != NO_ERROR_MSG)
   {
      return returnErrorMsg(ERROR_ETHERNET_DAPI_GET_STATISTICS, "rx_errors", m_pErrorMsg);
   }
   m_pErrorMsg = GetNodeValue(node_path + "rx_dropped", &(eth_statis->rx_dropped));
   if (m_pErrorMsg != NO_ERROR_MSG)
   {
      return returnErrorMsg(ERROR_ETHERNET_DAPI_GET_STATISTICS, "rx_dropped", m_pErrorMsg);
   }
   m_pErrorMsg = GetNodeValue(node_path + "rx_frame_errors", &(eth_statis->rx_frame));
   if (m_pErrorMsg != NO_ERROR_MSG)
   {
      return returnErrorMsg(ERROR_ETHERNET_DAPI_GET_STATISTICS, "rx_frame_errors", m_pErrorMsg);
   }
   m_pErrorMsg = GetNodeValue(node_path + "tx_packets", &(eth_statis->tx_packets));
   if (m_pErrorMsg != NO_ERROR_MSG)
   {
      return returnErrorMsg(ERROR_ETHERNET_DAPI_GET_STATISTICS, "tx_packets", m_pErrorMsg);
   }
   m_pErrorMsg = GetNodeValue(node_path + "tx_errors", &(eth_statis->tx_errors));
   if (m_pErrorMsg != NO_ERROR_MSG)
   {
      return returnErrorMsg(ERROR_ETHERNET_DAPI_GET_STATISTICS, "tx_errors", m_pErrorMsg);
   }
   m_pErrorMsg = GetNodeValue(node_path + "tx_dropped", &(eth_statis->tx_dropped));
   if (m_pErrorMsg != NO_ERROR_MSG)
   {
      return returnErrorMsg(ERROR_ETHERNET_DAPI_GET_STATISTICS, "tx_dropped", m_pErrorMsg);
   }
   m_pErrorMsg = GetNodeValue(node_path + "tx_aborted_errors", &(eth_statis->tx_carrier));
   if (m_pErrorMsg != NO_ERROR_MSG)
   {
      return returnErrorMsg(ERROR_ETHERNET_DAPI_GET_STATISTICS, NULL, m_pErrorMsg);
   }

   return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::GetStatistics(string nic, string *statistic)
{
    m_pErrorMsg = m_EthernetDriver->getStatistics(nic, statistic);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_GET_STATISTICS, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::GetNodeValue(string node_path, string *value)
{
    ifstream infile;

    infile.open(node_path.c_str());
    if (infile.is_open())
    {
       getline(infile,*value); // Saves the line in STRING.
       infile.close();
    }
    else
    {
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_NODE_VALUE | ERROR_ETHERNET_DAPI_NODE_OPEN_FAIL), node_path.c_str(), NULL);
    }

   return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::GetControllerName(string nic, string *controller_name)
{
    string            vendor_ID;
    string            device_ID;
    int               vendor_ID_value, device_ID_value;
    PCIDAPI           my_pci_dapi("pci");
//    SelectedPCIInfo   PCI_Device_Info;
//    PCIDataStructDAPI *pci_list; 
    string            bus_type;

    m_EthernetDriver = new EthernetDDAPI(nic.c_str());

    m_pErrorMsg = m_EthernetDriver->get_device_bus_type(&bus_type);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
       return returnErrorMsg(ERROR_ETHERNET_DAPI_GET_CONTROLLER_NAME, NULL, m_pErrorMsg);
    }


    if (bus_type.compare("pci") == 0)
    {
       m_pErrorMsg = m_EthernetDriver->get_vendor_ID(&vendor_ID);
       if (m_pErrorMsg != NO_ERROR_MSG)
       {
          return returnErrorMsg(ERROR_ETHERNET_DAPI_GET_CONTROLLER_NAME, NULL, m_pErrorMsg);
       }

       m_pErrorMsg = m_EthernetDriver->get_device_ID(&device_ID);
       if (m_pErrorMsg != NO_ERROR_MSG)
       {
          return returnErrorMsg(ERROR_ETHERNET_DAPI_GET_CONTROLLER_NAME, NULL, m_pErrorMsg);
       }

       //cout << "vendor_ID: " << vendor_ID << "   device_ID: " << device_ID << endl;
       sscanf(vendor_ID.c_str(), "%x", &vendor_ID_value);
       sscanf(device_ID.c_str(), "%x", &device_ID_value);
    
//       my_pci_dapi.GetDeviceInfoByVD(vendor_ID_value, device_ID_value, &PCI_Device_Info);
//       pci_list = PCI_Device_Info.pDevInfoList;
       //printf("PCI:%d     %s\n",PCI_Device_Info.u16PCIAmount, pci_list->szDeviceName);
//       controller_name->append(pci_list->szDeviceName);
    }
    else
    {
       controller_name->append("Not support bus type");
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::GetFirmwareVersion(string nic, string *fw_version)
{
    int err = 0, fd;
    struct ifreq ifr;
    struct ethtool_drvinfo drvinfo;

    errno = 0;

    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
    {
       //perror("socket");
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_FIRMWARE_VERSION | ERROR_ETHERNET_DAPI_SOCKET_OPEN_FAIL), strerror(errno), NULL);
    }

    //Covert well know name to system real name
    //m_EthernetDriver->ConverWellKnowNameToRealName(nic, &nic);
    strncpy(ifr.ifr_name, nic.c_str(), sizeof(ifr.ifr_name));

    drvinfo.cmd = ETHTOOL_GDRVINFO;
    ifr.ifr_data = (__caddr_t)&drvinfo;

    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0 && errno != ENOTSUP)
    {
       //perror("Cannot get current device link status");
       close(fd);
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_FIRMWARE_VERSION | ERROR_ETHERNET_DAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    //printf("fw_version: %s\n",drvinfo.fw_version);
    *fw_version = drvinfo.fw_version;
    //if (fw_version->find_first_of(",") != -1)
    //   fw_version->erase(fw_version->find_first_of(","), fw_version->length() - fw_version->find_first_of(","));

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::GetNode(string nic, string *node)
{
    m_pErrorMsg = NO_ERROR_MSG;

    //Covert well know name to system real name
    //m_pErrorMsg = m_EthernetDriver->ConverWellKnowNameToRealName(nic, node);
    *node = nic;
    //m_pErrorMsg = m_EthernetDriver->GetEthernetNode(nic, node);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_WRITE_MAC, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::Program(string ethernet_name, string filePath)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->Program(ethernet_name, filePath);

    return m_pErrorMsg;
}



ErrorMsg* EthernetDAPI::ReadSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 Length, INT32 *data)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->ReadSFP(ethernet_name, DeviceAddress,StartAddress, Length, data);

    return m_pErrorMsg;
}



ErrorMsg* EthernetDAPI::writeSFP(string WellKnownName, INT32 DeviceAddress, INT32 StartAddress, INT32 data)
{
    m_pErrorMsg = NO_ERROR_MSG;

    m_pErrorMsg = m_EthernetDriver->WriteSFP(WellKnownName, DeviceAddress,StartAddress, data);

    return m_pErrorMsg;
}



ErrorMsg* EthernetDAPI::GetPCIError(string nic, string *AER)
{
    PCIDDAPI                    PCI_driver("PCI_driver");
    PFA                         PCI_BDF_NIC;
    char                        PCI_error_string[100];
    list<PCI_ERROR>             pciErrorList;
    list<PCI_ERROR>::iterator   it;
    int                         domain, bus, device, func;


    //Get nic PCI BDF
    m_EthernetDriver->GetPCIBusDeviceFunction(nic, &domain, &bus, &device, &func);

    PCI_BDF_NIC.u16Domain  = domain;
    PCI_BDF_NIC.u16Bus     = bus;
    PCI_BDF_NIC.u16Device  = device;
    PCI_BDF_NIC.u8Function = func;


    //PCI Compatible error
    AER->append("==== PCI-compatible ====\n");
    PCI_driver.getErrorStatistics(PCI_BDF_NIC, PCI_COMPATIBLE_ERROR, &pciErrorList);
    for (it = pciErrorList.begin(); it != pciErrorList.end(); ++it)
    {
        sprintf(PCI_error_string, "%s: 0x%08x\n", it->strErrorName.c_str(), it->u32Data);
        AER->append(PCI_error_string);
    }
    pciErrorList.clear();

    //PCIE erroe
    AER->append("==== PCIE ====\n");
    PCI_driver.getErrorStatistics(PCI_BDF_NIC, PCIE_ERROR, &pciErrorList);
    for (it = pciErrorList.begin(); it != pciErrorList.end(); ++it)
    {
        sprintf(PCI_error_string, "%s: 0x%08x\n", it->strErrorName.c_str(), it->u32Data);
        AER->append(PCI_error_string);
    }
    pciErrorList.clear();

    //PCI AER error
    AER->append("==== AER ====\n");
    PCI_driver.getErrorStatistics(PCI_BDF_NIC, AER_ERROR, &pciErrorList);
    for (it = pciErrorList.begin(); it != pciErrorList.end(); ++it)
    {
        sprintf(PCI_error_string, "%s: 0x%08x\n", it->strErrorName.c_str(), it->u32Data);
        AER->append(PCI_error_string);
    }
    pciErrorList.clear();

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::Clear(string nic)
{
    //Clear ethernet PCI AER
    m_pErrorMsg = m_EthernetDriver->clearPCIError(nic);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_CLEAR, NULL, m_pErrorMsg);
    }

    //Clear ethernet sat
    m_pErrorMsg = m_EthernetDriver->clearStatistics(nic);
    if (m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_ETHERNET_DAPI_CLEAR, NULL, m_pErrorMsg);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::GetPCIBDF(string nic, string *bdf)
{
    int    domain, bus, device, func;
    char   BDF_string[100];

    //Get nic PCI BDF
    m_EthernetDriver->GetPCIBusDeviceFunction(nic, &domain, &bus, &device, &func);

   sprintf(BDF_string, "%04x:%02x:%02x.%01x", domain, bus, device, func);
   bdf->append(BDF_string);

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::GetPCISpeed(string nic, string *PCISpeed)
{
    int       domain, bus, device, func;
    PCIDAPI   my_pci_dapi("pci");
    PFA       PCI_BDF_NIC;
    char      Speed_string[100];

    //Get nic PCI BDF
    m_EthernetDriver->GetPCIBusDeviceFunction(nic, &domain, &bus, &device, &func);

    PCI_BDF_NIC.u16Domain  = domain;
    PCI_BDF_NIC.u16Bus     = bus;
    PCI_BDF_NIC.u16Device  = device;
    PCI_BDF_NIC.u8Function = func;

    sprintf(Speed_string, "%.1f", my_pci_dapi.GetReadableSpeed(PCI_BDF_NIC , PCI_DATA_TYPE_CURRENT_SPEED));
    PCISpeed->append(Speed_string);

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetDAPI::GetPCIWidth(string nic, string *PCIWidth)
{
    int       domain, bus, device, func;
    PCIDAPI   my_pci_dapi("pci");
    PFA       PCI_BDF_NIC;
    char      Width_string[100];

    //Get nic PCI BDF
    m_EthernetDriver->GetPCIBusDeviceFunction(nic, &domain, &bus, &device, &func);

    PCI_BDF_NIC.u16Domain  = domain;
    PCI_BDF_NIC.u16Bus     = bus;
    PCI_BDF_NIC.u16Device  = device;
    PCI_BDF_NIC.u8Function = func;

    sprintf(Width_string, "%d", my_pci_dapi.GetDataByBDF(PCI_BDF_NIC , PCI_DATA_TYPE_CURRENT_WIDTH));
    PCIWidth->append(Width_string);

    return NO_ERROR_MSG;
}


