/// @file       MemoryDAPI_GetECCReport.cpp
/// @brief      Provide checksum test for MemoryDAPI
/// @details    Verfiy checksum value for the specified memory range with user expected value
/// @author     KenHu
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#include "MemoryDAPI.hpp"
#include "CommonFunctions.hpp"

/// @fn         MemoryDAPI::checksumTest(PHYSICAL_ADDRESS_T in_startAddress,
///                                      PHYSICAL_ADDRESS_T in_endAddress,
///                                      string             in_strChecksum,
///                                      CHECKSUM_TYPES     in_eChecksumType)
/// @brief      Verify memory by March C algorithm
/// @param      in_startAddress is a physcial address for start address
/// @param      in_endAddress is a physcial address for end address
/// @param      in_u8Checksum is 8bits value for expected checksum value
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ErrorMsg* 
MemoryDAPI::checksumTest(PHYSICAL_ADDRESS_T in_startAddress,
                         PHYSICAL_ADDRESS_T in_endAddress,
                         string             in_strChecksum,
                         CHECKSUM_TYPES     in_eChecksumType)
{
    ErrorMsg*   pErrorMsg = NULL;
    UINT8       u8Sum = 0;
    UINT8       u8ExpectedChecksum = 0;
    string      strErrorMsg;

    switch (in_eChecksumType)
    {
        case CHECKSUM_8BITS:            
            while (in_startAddress < in_endAddress)
            {            
                u8Sum += m_pMemoryDDAPI->read8(in_startAddress); 
                in_startAddress++;
            }

            pErrorMsg = m_pMemoryDDAPI->getErrorMsg();
            if (pErrorMsg != NO_ERROR_MSG)
            {
                return returnErrorMsg(ERROR_MEMORY_DAPI_CHECKSUM_TEST, NULL, pErrorMsg);
            }

            u8Sum &= 0xFF;
            u8Sum = (~u8Sum + 1);

            break;
        default:
            return returnErrorMsg((ERRORCODE)(ERROR_MEMORY_DAPI_CHECKSUM_TEST | ERROR_BAD_PARAMETER_FOUR), NULL, NULL);
            break;
    }

    u8ExpectedChecksum = (UINT8)std::stoi(in_strChecksum, NULL, 16);
    
    if (u8ExpectedChecksum != u8Sum)
    {
        strErrorMsg = "expected value: 0x" + Int2HexStr(u8ExpectedChecksum, 2) + ", actual value: 0x" + Int2HexStr(u8Sum, 2);

        return returnErrorMsg((ERRORCODE)(ERROR_MEMORY_DAPI_CHECKSUM_TEST | ERROR_UNEXPECTED_VALUE), strErrorMsg.c_str(), NULL);
    }
    
    return NO_ERROR_MSG;
}

