/// @file       SASDAPI_WritePerformanceTest.cpp
/// @brief      SAS write performance test
/// @details    This file defines the SAS DFW API class's Write Performance member functions.
/// @version    1.0.0
/// @date       2014.02.26
/// @author     JerryLee
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#include <sys/time.h>

#include "NVMeDAPI.hpp"
/*
 /// @fn         SASDAPI::readPerformanceTest
 /// @brief      Setting SAS device read performance test
 /// @param      The logical ID of this SAS device
 /// @param      The first sector number to test
 /// @param      The number of bytes to test
 /// @exception  Nothing
 /// @return     A pointer to ErrorMsg for error message list
 ErrorMsg*
 SASDAPI::readPerformanceTest(PerformanceInfo* performanceInformation)
 {
 ErrorMsg* errorMsg = NULL;
 // Initializations.
 if ((memDDAPI = new MemoryDDAPI("MEM_DRVR")) == NULL)
 {
 addErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_ERRORCODE | ERROR_DIAGNOSTIC_API_CONSTRUCTOR1_ERROR), "Create Memory DDAPI fail");
 }
 deviceNode = performanceInformation->DeviceNode;
 logicalSectorToTest = performanceInformation->StartLogicalSector;
 testSize = performanceInformation->TestSize;

 if ((errorMsg = readPerformanceTestSetup()) == NO_ERROR_MSG)
 {
 if ((errorMsg = readPerformanceTestExecute()) != NO_ERROR_MSG)
 {
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READ_PERFORMANCE_TEST), NULL, errorMsg);
 }
 performanceInformation->RunTimes.tv_sec = spendTime.tv_sec;
 performanceInformation->RunTimes.tv_usec = spendTime.tv_usec;
 performanceInformation->m_dSpeed = testSpeed;

 errorMsg = readPerformanceTestCleanup();
 }
 else
 {
 readPerformanceTestCleanup();
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READ_PERFORMANCE_TEST_SETUP), NULL, errorMsg);
 }

 if (errorMsg != NULL)
 {
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READ_PERFORMANCE_TEST), NULL, errorMsg);
 }
 return NO_ERROR_MSG;
 }


 /// @fn         SASDAPI::readPerformanceTestSetup
 /// @brief      Read performance test setting
 /// @param      IOC ID
 /// @param      The first logical sector or address to test
 /// @param      When transfer sizes are used, this number specifies this size.
 ///             Typical sizes are 512, 520, and 4K bytes for each burst.
 /// @exception  Nothing
 /// @return     A pointer to ErrorMsg for error message list
 ErrorMsg*
 SASDAPI::readPerformanceTestSetup(void)
 {
 ErrorMsg* errorMsg = NULL;
 UINT32    blockSize = 1024;

 // Reserve 1 MB buffer
 if((errorMsg = memDDAPI->getMemoryBuffer(SAS_RW_SIZE, (SYSTEM_ADDRESS_T*)&dataBuffer)) != NO_ERROR_MSG)
 {
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READ_PERFORMANCE_TEST), NULL, errorMsg);
 }
 // and set the test increment
 rwBlocks = (SAS_RW_SIZE / blockSize);

 return NO_ERROR_MSG;
 }


 /// @fn         SASDAPI::readPerformanceTestExecute
 /// @brief      Execute read performance test
 /// @param      IOC ID
 /// @param      The first logical sector or address to test
 /// @param      The amount of contiguous storage space to test in MB
 /// @exception  Nothing
 /// @return     A pointer to ErrorMsg for error message list
 ErrorMsg*
 SASDAPI::readPerformanceTestExecute(void)
 {
 ErrorMsg* errorMsg = NULL;
 ERRORCODE errorCode = NO_ERROR;
 struct    timeval startTime;
 struct    timeval endTime;
 UINT64    tempSize = 0;

 if((errorCode = scsiFactory->setFileHandle(deviceNode.c_str())) != NO_ERROR)
 {
 if(dataBuffer)
 {
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer);
 }
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READ_PERFORMANCE_TEST_EXECUTE | errorCode), NULL, NULL);
 }

 tempSize = testSize;
 gettimeofday(&startTime, NULL);

 while (tempSize)
 {
 if ((errorMsg = sasDDAPI->seekRead(dataBuffer, logicalSectorToTest, SAS_RW_SIZE)) != NULL)
 {
 if(dataBuffer)
 {
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer);
 }
 scsiFactory->closeFileHandle();

 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READ_PERFORMANCE_TEST_EXECUTE), NO_ERROR_MSG, errorMsg);
 }
 logicalSectorToTest += SAS_RW_SIZE;

 tempSize -= SAS_RW_SIZE;
 }
 gettimeofday(&endTime, NULL);

 spendTime.tv_sec = endTime.tv_sec - startTime.tv_sec;
 spendTime.tv_usec = endTime.tv_usec - startTime.tv_usec;

 if (spendTime.tv_usec < 0)
 {
 --spendTime.tv_sec;
 spendTime.tv_usec += 1000000;
 }
 double dDataSize = (double)(testSize/(MEGABYTES));
 double dCostTime = (double)(spendTime.tv_sec)+ ((double)(spendTime.tv_usec)/1000000);

 testSpeed = dDataSize/dCostTime;

 return NO_ERROR_MSG;
 }


 /// @fn         SASDAPI::readPerformanceTestCleanup
 /// @brief      Clean up read performance test settings
 /// @param      IOC ID
 /// @exception  Nothing
 /// @return     A pointer to ErrorMsg for error message list
 ErrorMsg*
 SASDAPI::readPerformanceTestCleanup(void)
 {
 ErrorMsg* errorMsg = NULL;
 // Release data buffer
 if ((errorMsg = memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer)) != NO_ERROR_MSG)
 {
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READ_PERFORMANCE_TEST), NULL, errorMsg);
 }
 scsiFactory->closeFileHandle();
 // delete PCI Driver API
 if (memDDAPI != NULL)
 delete memDDAPI;

 return NO_ERROR_MSG;
 }


 /// @fn         SASDAPI::writePerformanceTest
 /// @brief      Setting SAS device write performance test
 /// @param      The logical ID of this SAS device
 /// @param      The first sector number to test
 /// @param      The number of bytes to test
 /// @exception  Nothing
 /// @return     A pointer to ErrorMsg for error message list
 ErrorMsg*
 SASDAPI::writePerformanceTest(PerformanceInfo* performanceInformation)
 {
 ErrorMsg* errorMsg = NULL;
 // Initializations.
 if ((memDDAPI = new MemoryDDAPI("MEM_DRVR")) == NULL)
 {
 addErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_ERRORCODE | ERROR_DIAGNOSTIC_API_CONSTRUCTOR1_ERROR), "Create Memory DDAPI fail");
 }
 deviceNode = performanceInformation->DeviceNode;
 logicalSectorToTest = performanceInformation->StartLogicalSector;
 testSize = performanceInformation->TestSize;

 if ((errorMsg = writePerformanceTestSetup()) == NO_ERROR_MSG)
 {
 if ((errorMsg = writePerformanceTestExecute()) != NO_ERROR_MSG)
 {
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST), NULL, errorMsg);
 }
 performanceInformation->RunTimes.tv_sec = spendTime.tv_sec;
 performanceInformation->RunTimes.tv_usec = spendTime.tv_usec;
 performanceInformation->m_dSpeed = testSpeed;

 errorMsg = writePerformanceTestCleanup();
 }
 else
 {
 writePerformanceTestCleanup();
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST_SETUP), NULL, errorMsg);
 }

 if (errorMsg != NULL)
 {
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST), NULL, errorMsg);
 }
 return NO_ERROR_MSG;
 }


 /// @fn         SASDAPI::writePerformanceTestSetup
 /// @brief      Write performance test setting
 /// @param      IOC ID
 /// @param      The first logical sector or address to test
 /// @param      When transfer sizes are used, this number specifies this size.
 ///             Typical sizes are 512, 520, and 4K bytes for each burst.
 /// @exception  Nothing
 /// @return     A pointer to ErrorMsg for error message list
 ErrorMsg*
 SASDAPI::writePerformanceTestSetup(void)
 {
 ErrorMsg* errorMsg = NULL;
 UINT32    blockSize = 1024;

 // Reserve 1 MB buffer
 if((errorMsg = memDDAPI->getMemoryBuffer(SAS_RW_SIZE, (SYSTEM_ADDRESS_T*)&dataBuffer)) != NO_ERROR_MSG)
 {
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST_SETUP), NULL, errorMsg);
 }
 // and set the test increment
 rwBlocks = (SAS_RW_SIZE / blockSize);

 return NO_ERROR_MSG;
 }


 /// @fn         SASDAPI::writePerformanceTestExecute
 /// @brief      Execute write performance test
 /// @param      IOC ID
 /// @param      The first logical sector or address to test
 /// @param      The amount of contiguous storage space to test in MB
 /// @exception  Nothing
 /// @return     A pointer to ErrorMsg for error message list
 ErrorMsg*
 SASDAPI::writePerformanceTestExecute(void)
 {
 ErrorMsg* errorMsg = NULL;
 ERRORCODE errorCode = NO_ERROR;
 struct    timeval startTime;
 struct    timeval endTime;
 UINT64    tempSize = 0;

 if((errorCode = scsiFactory->setFileHandle(deviceNode.c_str())) != NO_ERROR)
 {
 if(dataBuffer)
 {
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer);
 }
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE | errorCode), NULL, NULL);
 }

 tempSize = testSize;

 while (tempSize)
 {
 if ((errorMsg = sasDDAPI->seekRead(dataBuffer, logicalSectorToTest, SAS_RW_SIZE)) != NULL)
 {
 if(dataBuffer)
 {
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer);
 }
 scsiFactory->closeFileHandle();

 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE), NO_ERROR_MSG, errorMsg);
 }
 gettimeofday(&startTime, NULL);

 if ((errorMsg = sasDDAPI->seekWrite(dataBuffer, logicalSectorToTest, SAS_RW_SIZE)) != NULL)
 {
 if(dataBuffer)
 {
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer);
 }
 scsiFactory->closeFileHandle();

 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST_EXECUTE), NO_ERROR_MSG, errorMsg);
 }
 gettimeofday(&endTime, NULL);

 spendTime.tv_sec += endTime.tv_sec - startTime.tv_sec;
 spendTime.tv_usec += endTime.tv_usec - startTime.tv_usec;

 logicalSectorToTest += SAS_RW_SIZE;

 tempSize -= SAS_RW_SIZE;
 }

 if (spendTime.tv_usec < 0)
 {
 --spendTime.tv_sec;
 spendTime.tv_usec += 1000000;
 }
 double dDataSize = (double)(testSize/(MEGABYTES));
 double dCostTime = (double)(spendTime.tv_sec)+ ((double)(spendTime.tv_usec)/1000000);

 testSpeed = dDataSize/dCostTime;

 return NO_ERROR_MSG;
 }


 /// @fn         SASDAPI::writePerformanceTestCleanup
 /// @brief      Clean up write performance test settings
 /// @param      IOC ID
 /// @exception  Nothing
 /// @return     A pointer to ErrorMsg for error message list
 ErrorMsg*
 SASDAPI::writePerformanceTestCleanup(void)
 {
 ErrorMsg* errorMsg = NULL;
 // Release data buffer
 if ((errorMsg = memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer)) != NO_ERROR_MSG)
 {
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_WRITE_PERFORMANCE_TEST_CLEAN), NULL, errorMsg);
 }
 scsiFactory->closeFileHandle();
 // delete PCI Driver API
 if (memDDAPI != NULL)
 delete memDDAPI;

 return NO_ERROR_MSG;
 }


 /// @fn         SASDAPI::writePerformanceTest
 /// @brief      Setting SAS device write performance test
 /// @param      The logical ID of this SAS device
 /// @param      The first sector number to test
 /// @param      The number of bytes to test
 /// @exception  Nothing
 /// @return     A pointer to ErrorMsg for error message list
 ErrorMsg*
 SASDAPI::readWriteTest(PerformanceInfo* performanceInformation)
 {
 ErrorMsg* errorMsg = NULL;
 // Initializations.
 if ((memDDAPI = new MemoryDDAPI("MEM_DRVR")) == NULL)
 {
 addErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_ERRORCODE | ERROR_DIAGNOSTIC_API_CONSTRUCTOR1_ERROR), "Create Memory DDAPI fail");
 }
 deviceNode = performanceInformation->DeviceNode;
 logicalSectorToTest = performanceInformation->StartLogicalSector;
 testSize = performanceInformation->TestSize;

 if ((errorMsg = writePerformanceTestSetup()) == NO_ERROR_MSG)
 {
 if ((errorMsg = writePerformanceTestExecute()) != NO_ERROR_MSG)
 {
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READWRITE_TEST), NULL, errorMsg);
 }
 errorMsg = writePerformanceTestCleanup();
 }
 else
 {
 writePerformanceTestCleanup();
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READWRITE_TEST), NULL, errorMsg);
 }

 if (errorMsg != NULL)
 {
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READWRITE_TEST), NULL, errorMsg);
 }

 return NO_ERROR_MSG;
 }


 /// @fn         SASDAPI::readWriteTestSetup
 /// @brief      Read write test setting
 /// @param      IOC ID
 /// @param      The first logical sector or address to test
 /// @param      When transfer sizes are used, this number specifies this size.
 ///             Typical sizes are 512, 520, and 4K bytes for each burst.
 /// @exception  Nothing
 /// @return     A pointer to ErrorMsg for error message list
 ErrorMsg*
 SASDAPI::readWriteTestSetup(void)
 {
 ERRORCODE errorCode = NO_ERROR;
 ErrorMsg* errorMsg  = NULL;
 UINT32    blockSize  = 1024;

 if((errorCode = scsiFactory->setFileHandle(deviceNode.c_str())) != NO_ERROR)
 {
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READWRITE_TEST_SETUP | errorCode), NULL, NULL);
 }
 // Reserve 1 MB buffer
 if((errorMsg = memDDAPI->getMemoryBuffer(SAS_RW_SIZE, (SYSTEM_ADDRESS_T*)&dataBuffer)) != NO_ERROR_MSG)
 {
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READWRITE_TEST_SETUP), NULL, errorMsg);
 }
 // and set the test increment
 rwBlocks = (SAS_RW_SIZE / blockSize);

 return NO_ERROR_MSG;
 }


 /// @fn         SASDAPI::readWriteTestExecute
 /// @brief      Execute read write test
 /// @param      IOC ID
 /// @param      The first logical sector or address to test
 /// @param      The amount of contiguous storage space to test in MB
 /// @exception  Nothing
 /// @return     A pointer to ErrorMsg for error message list
 ErrorMsg*
 SASDAPI::readWriteTestExecute(void)
 {
 ERRORCODE errorCode     = NO_ERROR;
 ErrorMsg* errorMsg      = NULL;
 UINT32*   patternBuffer = NULL;
 UINT32*   compareBuffer = NULL;
 UINT64    tempSize       = 0;
 UINT8     compareResult  = 0;

 tempSize = testSize;

 //allocate pattern buffer
 errorMsg = memDDAPI->getMemoryBuffer(SAS_RW_SIZE, (SYSTEM_ADDRESS_T*)&patternBuffer);
 if(errorMsg != NULL)
 {
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer);
 return returnErrorMsg(ERRORCODE(ERROR_SAS_DAPI_READWRITE_TEST_EXECUTE | ERROR_SAS_DAPI_ALLOCATE_PATTERN_BUFFER_FAIL), NULL, errorMsg);
 }
 //allocate compare buffer
 errorMsg = memDDAPI->getMemoryBuffer(SAS_RW_SIZE, (SYSTEM_ADDRESS_T*)&compareBuffer);
 if(errorMsg != NULL)
 {
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)patternBuffer);
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer);
 return returnErrorMsg(ERRORCODE(ERROR_SAS_DAPI_READWRITE_TEST_EXECUTE | ERROR_SAS_DAPI_ALLOCATE_PCOMPARE_BUFFER_FAIL), NULL, errorMsg);
 }

 srand(time(NULL));

 while (tempSize && (!compareResult))
 {
 //generate random pattern
 for(int i=0 ; i < SAS_RW_SIZE/sizeof(UINT32) ; i++)
 {
 *(patternBuffer+i) = rand()%UINT_MAX;
 }

 if ((errorMsg = sasDDAPI->seekRead(dataBuffer, logicalSectorToTest, SAS_RW_SIZE)) != NULL)
 {
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)compareBuffer);
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)patternBuffer);
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer);
 scsiFactory->closeFileHandle();

 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READWRITE_TEST_EXECUTE), NO_ERROR_MSG, errorMsg);
 }

 if ((errorMsg = sasDDAPI->seekWrite(patternBuffer, logicalSectorToTest, SAS_RW_SIZE)) != NULL)
 {
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)compareBuffer);
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)patternBuffer);
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer);
 scsiFactory->closeFileHandle();

 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READWRITE_TEST_EXECUTE), NO_ERROR_MSG, errorMsg);
 }

 if ((errorMsg = sasDDAPI->seekRead(compareBuffer, logicalSectorToTest, SAS_RW_SIZE)) != NULL)
 {
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)compareBuffer);
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)patternBuffer);
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer);
 scsiFactory->closeFileHandle();

 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READWRITE_TEST_EXECUTE), NO_ERROR_MSG, errorMsg);
 }
 //compare pattern
 compareResult = memcmp(patternBuffer, compareBuffer, SAS_RW_SIZE);
 //recovery original data to disk
 if ((errorMsg = sasDDAPI->seekWrite(dataBuffer, logicalSectorToTest, SAS_RW_SIZE)) != NULL)
 {
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)compareBuffer);
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)patternBuffer);
 memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer);
 scsiFactory->closeFileHandle();

 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READWRITE_TEST_EXECUTE), NO_ERROR_MSG, errorMsg);
 }

 if (!compareResult)
 errorCode = ERROR_SAS_DAPI_TEST_PATTERN_COMPARE_FAIL;

 logicalSectorToTest += SAS_RW_SIZE;

 tempSize -= SAS_RW_SIZE;
 }
 if (errorCode != NULL)
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READWRITE_TEST_EXECUTE | errorCode), NO_ERROR_MSG, NO_ERROR_MSG);

 return NO_ERROR_MSG;
 }


 /// @fn         SASDAPI::readWriteTestCleanup
 /// @brief      Clean up write performance test settings
 /// @param      IOC ID
 /// @exception  Nothing
 /// @return     A pointer to ErrorMsg for error message list
 ErrorMsg*
 SASDAPI::readWriteTestCleanup(void)
 {
 ErrorMsg* errorMsg = NULL;
 // Release data buffer
 if ((errorMsg = memDDAPI->releaseMemoryBuffer((SYSTEM_ADDRESS_T)dataBuffer)) != NO_ERROR_MSG)
 {
 return returnErrorMsg((ERRORCODE)(ERROR_SAS_DAPI_READWRITE_TEST_CLEAN), NULL, errorMsg);
 }
 scsiFactory->closeFileHandle();

 // delete PCI Driver API
 if (memDDAPI != NULL)
 delete memDDAPI;

 return NO_ERROR_MSG;
 }
 */
