#include "TPMDAPI.hpp"

TPMDAPI::TPMDAPI(STRING className) : BaseClassDAPI(className)
{
    m_pTpmDriverAPI = new TPMDDAPI("TPMDDAPI");
}

TPMDAPI::~TPMDAPI()
{
    if(m_pTpmDriverAPI != NULL)
    {
        delete m_pTpmDriverAPI;
        m_pTpmDriverAPI = NULL;
    }
}


ErrorMsg*
TPMDAPI::displayDevice()
{
    m_eErrorCode = m_pTpmDriverAPI->openDevice();
    if(m_eErrorCode == NO_ERROR)
    {
        cout << "TPM0" << endl;
    }
    else
    {
        cout << "No device be detected." << endl;
    }

    m_pTpmDriverAPI->closeDevice();

    return NO_ERROR_MSG;
}


ErrorMsg*
TPMDAPI::commandInterface(TPM_CMD_INPUT     in_cmdList,
                          UINT8*            out_u8pRspBuffer,
                          UINT32            in_u32RspBufferSize,
                          UINT32*           out_u32pRspSize) 
{
    UINT8*  u8pCmdBuffer;
    UINT32  u32CmdSize = 0;

    m_pTpmDriverAPI->getCmdSize(in_cmdList, &u32CmdSize);
    in_cmdList.u32CmdSize = u32CmdSize; 

    u8pCmdBuffer = (UINT8*)malloc(sizeof(UINT8)*u32CmdSize);
    
    m_pTpmDriverAPI->createCmdList(in_cmdList, u8pCmdBuffer);
   
    m_pErrorMsg = m_pTpmDriverAPI->commandInterface(u8pCmdBuffer, u32CmdSize, 
                                                    out_u8pRspBuffer, in_u32RspBufferSize, out_u32pRspSize);
    free(u8pCmdBuffer);

    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_TPM_DAPI_ACCESS_CMD, NULL, m_pErrorMsg);
    }
    return NO_ERROR_MSG;    
}


ErrorMsg*
TPMDAPI::sendCommandManually(STRING    in_cmdFileName,
                             UINT8*    out_u8pRspBuffer,
                             UINT32    in_u32RspBufferSize,
                             UINT32*   out_u32pRspSize)
{
    m_pErrorMsg = m_pTpmDriverAPI->sendCommandManually(in_cmdFileName, 
                                                       out_u8pRspBuffer,
                                                       in_u32RspBufferSize,
                                                       out_u32pRspSize);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_TPM_DAPI_SEND_CMD_MANUALLY, NULL, m_pErrorMsg);
    }
    return NO_ERROR_MSG;
}






