/// @file       IOCTL.cpp
/// @brief      IOCTL base class
/// @details    This is an abstract class which define IOCTL low layer utility API
/// @version    1.0.0
/// @date       2014.02.11
/// @author     JerryLee
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#include "IOCTL.hpp"
#include "CommonFunctions.hpp"

/// @fn         ICOTL::IOCTL(STRING in_fileName)
/// @brief      ICOTL constructor
/// @param      in_fileName is a constant string for file name
/// @exception  Nothing
/// @return     Nothing
IOCTL::IOCTL(STRING fileName)
{
    // TODO Auto-generated constructor stub
    myFileHandle = 0;
    targetControllerID = INVALID_CONTROLLER_ID;
    targetPHYNumber = INVALID_PHY_NUMBER;
    myIOCTLArgument = NULL;
    setFileHandle(fileName);
}

/// @fn         IOCTL::~IOCTL(void)
/// @brief      IOCTL destructor
/// @exception  Nothing
/// @return     Nothing
IOCTL::~IOCTL()
{
    // TODO Auto-generated destructor stub
    if(this->myFileHandle > 0)
        close(this->myFileHandle);

    if(myIOCTLArgument)
        free(myIOCTLArgument);
}

ERRORCODE IOCTL::getFileHandle(INT8* fileHandle)
{
    *fileHandle = this->myFileHandle;
    return NO_ERROR;
}

ERRORCODE IOCTL::getFileHandle(STRING fileName, INT8* fileHandle)
{
    if(myFileHandle == 0)
        setFileHandle(fileName);
    else if(myFileHandle < 0)
        return NO_ERROR;
    else
        *fileHandle = this->myFileHandle;

    return NO_ERROR;
}

UINT8 IOCTL::getControllerID(void)
{
    return this->targetControllerID;
}

UINT8 IOCTL::getPHYNumber(void)
{
    return this->targetPHYNumber;
}

void IOCTL::setControllerID(UINT8 iocNumber)
{
    this->targetControllerID = iocNumber;
}

void IOCTL::setPHYNumber(UINT8 phyNumber)
{
    this->targetPHYNumber = phyNumber;
}

ERRORCODE IOCTL::setFileHandle(STRING fileName)
{
    this->myFileHandle = open(fileName, O_RDWR);

    if(this->myFileHandle)
        return NO_ERROR;
    else
        return ERROR_IOCTL_GET_FILE_HANDLE_FAIL;
}

ERRORCODE IOCTL::closeFileHandle()
{
    if(this->myFileHandle > 0)
        close(this->myFileHandle);

    return NO_ERROR;
}

ERRORCODE IOCTL::allocateIOCTLArgument(UINT8 IOCTLOperation)
{
    return ERROR_IOCTL_UNSUPPORT_CLASS;
}

/// @fn         IOCTL::setIOCTLOperationArgument
/// @brief      Set parameter of specific IOCTL operation
/// @param      IOCTLOperation enum defined several IOCTL request command type
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ERRORCODE IOCTL::setIOCTLOperationArgument(UINT8 IOCTLOperation, void* data)
{
    return ERROR_IOCTL_UNSUPPORT_CLASS;
}

/// @fn         IOCTL:executeIOCTL
/// @brief      Execute IOCTL and get result
/// @param      IOCTLOperation enum defined several IOCTL request command type
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ERRORCODE IOCTL::executeIOCTL(UINT8 IOCTLOperation)
{
    return ERROR_IOCTL_UNSUPPORT_CLASS;
}

/// @fn         IOCTL:executeIOCTL
/// @brief      Execute IOCTL and get result
/// @param      IOCTLOperation enum defined several IOCTL request command type
/// @param      Get result from specific IOCTL operation
/// @exception  Nothing
/// @return     A pointer to ErrorMsg for error message list
ERRORCODE IOCTL::executeIOCTL(UINT8 IOCTLOperation, void* out_pIOCTLResult)
{
    return ERROR_IOCTL_UNSUPPORT_CLASS;
}
