#ifndef ProcessorDDAPI_H
#define ProcessorDDAPI_H

#include "BaseClassDDAPI.hpp"
#include <list>
#include <fcntl.h>
#include <unistd.h>
#include "BroadwellDE.hpp"

using namespace std;

extern "C"{
#include <pci/pci.h>
}

enum {
//    READ = 0,
//    WRITE = -1,

    RW_BYTE = 1,
    RW_WORD = 2,
    RW_DWORD = 4,
};



enum CPU_TEMPERATURE_OPERATION_ID
{
    CPU_CURRENT_TEMPERATURE,
    CPU_THRESHOLD_HIGH,
    CPU_THRESHOLD_LOW,
};



struct pci_device {
    unsigned short domain;
    unsigned short bus:8;
    unsigned short dev:8;
    unsigned short func:8;
    unsigned short classid;
    unsigned short revision:8;  /* pci revision */
    unsigned short vendor;
    unsigned short device;
    unsigned short sub_vendor;
    unsigned short sub_device;
    unsigned short addr;        /* the register's address */
    unsigned int   data;        /* read/write value */
    unsigned short rw_size:8;   /* specified read/write data's size: 1 - byte, 2 - word, 4 - double word */
    unsigned short slave;       /* i2c slave address */
    int            len;
    unsigned int   reserved[0];
};



class CPUInformation
{
    public:
                        CPUInformation(void);
          virtual       ~CPUInformation(void);

          INT32         m_processor_id;
          string        m_vendor_id;
          INT32         m_cpu_family;
          INT32         m_model;
          string        m_model_name;
          INT32         m_stepping;
          FLOAT         m_cpu_frequency;
          INT32         m_cache_size;
          INT32         m_physical_id;
          INT32         m_siblings;
          INT32         m_core_id;
          INT32         m_cpu_cores;
          INT32         m_apicid;
          INT32         m_initial_apicid;
          BOOL          m_fdiv_bug;
          BOOL          m_hlt_bug;
          BOOL          m_f00f_bug;
          BOOL          m_coma_bug;
          BOOL          m_fpu;
          BOOL          m_fpu_exception;
          INT32         m_cpuid_level;
          BOOL          m_wp;
          string        m_flags;
          FLOAT         m_bogomips;
          INT32         m_clflush_size;
          INT32         m_cache_alignment;
          INT32         m_address_sizes_bits_physical;
          INT32         m_address_sizes_bits_virtual;
          string        m_power_management;
};



class ProcessorDDAPI : public BaseClassDDAPI
{
    public:
                    ProcessorDDAPI(STRING className);
    virtual         ~ProcessorDDAPI(void);

//                    ErrorMsg*   GetCPUInformation(XML_CPU* CPU_data);
                    ErrorMsg*   GetCPUInformation(list<CPUInformation> *CPU_data);
                    CHAR *      RTrim(CHAR *string, CHAR junk);
                    CHAR *      LTrim(CHAR *string, CHAR junk);
                    ErrorMsg*   GetCPUNumber(INT32* CPU_number);
                    ErrorMsg*   GetCPUPhysicalNumber(INT32* CPU_number);
                    ErrorMsg*   RegRead(struct pci_device *pci);
                    //ErrorMsg*   RegWrite(struct pci_device *pci);
                    ErrorMsg*   GetErrorRegistersExecute(INT32  *Test_result);
                    //ErrorMsg*   IOCTL(INT32 cmd);
                    ErrorMsg*   ReadCPURegister(string wellKnownName, int CPU_register, int count, list<unsigned long long> *val);
                    ErrorMsg*   WriteCPURegister(string wellKnownName, int CPU_register, unsigned long long Data);
                    ErrorMsg*   GetCPUIDByWellKnowName(string wellKnownName, int *ID);
                    ErrorMsg*   GetCoreIDByPhycailID(int ID, int *CordID);
                    ErrorMsg*   getTemperatureDataByID(string well_know_name, CPU_TEMPERATURE_OPERATION_ID ID, string *result);
                    ErrorMsg*   setTemperatureDataByID(string well_know_name, CPU_TEMPERATURE_OPERATION_ID ID, string data);
                    ErrorMsg*   get_device_by_well_known_name(string wellKnownName, CPUInformation *device);

    protected:

    private:
            ErrorMsg*  m_errorCode;
};

#endif
