#include "EthernetBaseClass.hpp"


EthernetBaseClass::EthernetBaseClass(STRING nic_name):BaseClassDDAPI(nic_name)
{
   m_node = "/sys/class/net/";
   m_node.append(nic_name);
   //printf("%s\n",m_node.c_str());
}


EthernetBaseClass::~EthernetBaseClass()
{

}


ErrorMsg* EthernetBaseClass::Set_external_loopback()
{
   //printf("I'm Ethernet base class. Set_external_loopback\n");
   return NO_ERROR_MSG;
}


ErrorMsg* EthernetBaseClass::Remove_external_loopback()
{
   //printf("I'm Ethernet base class. Remove_external_loopback\n");
   return NO_ERROR_MSG;
}


string EthernetBaseClass::Get_Node()
{
   return m_node;
}


ErrorMsg* EthernetBaseClass::Append_Node(string bass_address)
{
   m_node.append(bass_address);
   return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EXTERNAL_LOOPBACK | ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL), NULL, NULL);//NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::Get_self_test_result(struct ethtool_test *test, TestType index, INT32 *test_result)
{
    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::SetupSpeed(string ethernet_name, INT32 speed)
{
    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::GetLinkStatus(string nic, string *status_result)
{
    int err = 0, fd;
    struct ifreq ifr;
    struct ethtool_value edata;

    errno = 0;

    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
    {
       //perror("socket");
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_LINK_STATUS | ERROR_ETHERNET_DAPI_SOCKET_OPEN_FAIL), strerror(errno), NULL);
    }

    //Covert well know name to system real name
    //m_EthernetDriver->ConverWellKnowNameToRealName(nic, &nic);
    strncpy(ifr.ifr_name, nic.c_str(), sizeof(ifr.ifr_name));


    edata.cmd = ETHTOOL_GLINK;
    ifr.ifr_data = (__caddr_t)&edata;

    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0 && errno != ENOTSUP)
    {
       //perror("Cannot get current device link status");
       close(fd);
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_LINK_STATUS | ERROR_ETHERNET_DAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

   //printf("%d\n",edata.data);
   //printf("Link detected:");
   if (edata.data == 1)
   {
      //printf("Yes\n");
      *status_result = "Up";
   }
   else
   {
      //printf("No\n");
      *status_result = "Down";
   }
   close(fd);
   return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::CheckAdvertisedSpeed(string nic, INT32 speed)
{
    int    fd, err;
    struct ifreq ifr;
    struct ethtool_cmd ecmd;
    string Error_meg;

    switch (speed)
    {
        case 10: //10M
            speed = ADVERTISED_10baseT_Full;
            break;
        case 100: //100M
            speed = ADVERTISED_100baseT_Full;
            break;
        case 1000: //1G
            speed = ADVERTISED_1000baseT_Full;
            break;
        case 10000: //10G
            speed = ADVERTISED_10000baseT_Full;
            break;
        case 40000: //40G
            speed = ADVERTISED_40000baseCR4_Full;
            break;
        default:
            return returnErrorMsg((ERRORCODE)ERROR_ETHERNET_DAPI_SETUP_SPEED, "Not difned speed", NULL);
    }

    fd = socket(AF_INET,SOCK_DGRAM,0);
    if (fd < 0)
    {
       //perror("socket");
       close(fd);
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED | ERROR_ETHERNET_DAPI_SOCKET_OPEN_FAIL), strerror(errno), NULL);
    }

    strncpy(ifr.ifr_name, nic.c_str(), sizeof(ifr.ifr_name));

    //Get the driver speed support information
    ecmd.cmd = ETHTOOL_GSET;
    ifr.ifr_data = (__caddr_t)&ecmd;
    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0)
    {
       //perror("Cannot get current device settings");
       close(fd);
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED | ERROR_ETHERNET_DAPI_GET_ETHERNET_DRIVER_SET_FAIL), strerror(errno), NULL);
    }

    //Transform speed index, if port type is FIBER
    if (ecmd.port == PORT_FIBRE && speed == ADVERTISED_1000baseT_Full)
    {
        speed = ADVERTISED_1000baseKX_Full;
    }

    //check the driver support or not
    if ((ecmd.supported & speed) == 0)
    {
        Error_meg = nic + " not support speed";
        return returnErrorMsg((ERRORCODE)ERROR_ETHERNET_DAPI_SETUP_SPEED, Error_meg.c_str(), NULL);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::GetMAC(string ethernet_name, string *mac_address)
{
    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::SetMAC(string ethernet_name, string *mac_address)
{
    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::GetEEPROM(string ethernet_name, INT32 StartAddress, INT32 Length, INT32 *data)
{
    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::SetEEPROM(string ethernet_name, INT32 StartAddress, INT32 *data, INT32 Length)
{
    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::GetPCIFunction(string nic, int *function_id)
{
    int                    err = 0, fd;
    string                 function_string;
    struct ifreq           ifr;
    struct ethtool_drvinfo drvinfo;

    errno = 0;

    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
    {
       //perror("socket");
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_FIRMWARE_VERSION | ERROR_ETHERNET_DAPI_SOCKET_OPEN_FAIL), strerror(errno), NULL);
    }

    strncpy(ifr.ifr_name, nic.c_str(), sizeof(ifr.ifr_name));

    drvinfo.cmd = ETHTOOL_GDRVINFO;
    ifr.ifr_data = (__caddr_t)&drvinfo;

    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0 && errno != ENOTSUP)
    {
       //perror("Cannot get current device link status");
       close(fd);
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_FIRMWARE_VERSION | ERROR_ETHERNET_DAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    function_string = drvinfo.bus_info;
    //splitte PCI function number only
    function_string = function_string.substr(function_string.find(".") + 1);

    //Turn string to int.
    sscanf(function_string.c_str(), "%d", function_id);

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::GetPCIBusDeviceFunction(string ethernet_name, int *domain, int *bus, int *device, int *func)
{
    int                    err = 0, fd;
    string                 function_string;
    struct ifreq           ifr;
    struct ethtool_drvinfo drvinfo;

    errno = 0;

    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
    {
       //perror("socket");
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_FIRMWARE_VERSION | ERROR_ETHERNET_DAPI_SOCKET_OPEN_FAIL), strerror(errno), NULL);
    }

    strncpy(ifr.ifr_name, ethernet_name.c_str(), sizeof(ifr.ifr_name));

    drvinfo.cmd = ETHTOOL_GDRVINFO;
    ifr.ifr_data = (__caddr_t)&drvinfo;

    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0 && errno != ENOTSUP)
    {
       //perror("Cannot get current device link status");
       close(fd);
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_GET_FIRMWARE_VERSION | ERROR_ETHERNET_DAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    sscanf(drvinfo.bus_info, "%x:%x:%x.%x", domain, bus, device, func);

    close(fd);

    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::Program(string ethernet_name, string filePath)
{
    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::ReadSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 Length, INT32 *data)
{
    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::WriteSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 data)
{
    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::getLEDName(string nic, list<string> *LED)
{
    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::setLEDStatus(string ethernet_name, string in_strLEDName, string in_strStatus)
{
    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::getLEDStatus(string ethernet_name, string in_strLEDName, string *out_strpStatus)
{
    return NO_ERROR_MSG;
}



ErrorMsg* EthernetBaseClass::GetPCIDeviceID(string ethernet_name, string *vendor_ID)
{
    string   node_path;
    ifstream infile;

    node_path = "/sys/class/net/" + ethernet_name + "/device/device";

    infile.open(node_path.c_str());
    if (infile.is_open())
    {
       getline(infile,*vendor_ID); // Saves the line in STRING.
       infile.close();
    }
    else
    {
       return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_DEVICE_ID | ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL), node_path.c_str(), NULL);
    }

    return NO_ERROR_MSG;
}


