#include "INTEL_XL710.hpp"



INTEL_XL710::INTEL_XL710(STRING nic_name):EthernetBaseClass(nic_name)
{
   Append_Node("/device/resource0");
   //printf("Intel XL710: %s\n",Get_Node().c_str());
   m_errorCode = NO_ERROR_MSG;
}



INTEL_XL710::~INTEL_XL710()
{

}



ErrorMsg* INTEL_XL710::Set_external_loopback()
{
   //printf("INTEL XL710 Set_external_loopback\n");

   unsigned int *addr;
   int fd;

   fd = open(Get_Node().c_str(), O_RDWR | O_SYNC);
   if (fd < 0)
   {
       //printf("%s file open error!\n",Get_Node().c_str());
       return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EXTERNAL_LOOPBACK | ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL), NULL, NULL);
   }

   addr = (unsigned int *)mmap(NULL, 0x1000, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0x00);
   MDIO_point = addr + (MDIO_CONTROL_REGISTER / 4);

   //printf("0x%04x\n",read_PHY_reg(0, 0));
   //1G setup
   write_PHY_reg(18, 6, 0x0008);
   write_PHY_reg(0, 0, 0x9340);

   //100/10M setup
   //write_PHY_reg(18, 6, 0x0000);
   //write_PHY_reg(0, 0, 0x1140);

   munmap(addr, 0x1000);
   close(fd);

   return NO_ERROR_MSG;
}



ErrorMsg* INTEL_XL710::Remove_external_loopback()
{
   //printf("INTEL XL710 Remove_external_loopback\n");

   unsigned int *addr;
   int fd;

   fd = open(Get_Node().c_str(), O_RDWR | O_SYNC);
   if (fd < 0)
   {
       //printf("%s file open error!\n",Get_Node().c_str());
       return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EXTERNAL_LOOPBACK | ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL), NULL, NULL);
   }

   addr = (unsigned int *)mmap(NULL, 0x1000, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0x00);
   MDIO_point = addr + (MDIO_CONTROL_REGISTER / 4);

   //printf("0x%04x\n",read_PHY_reg(0, 0));

   //100/10M setup
   write_PHY_reg(18, 6, 0x0000);
   write_PHY_reg(0, 0, 0x1140);

   munmap(addr, 0x1000);
   close(fd);

   return NO_ERROR_MSG;
}



int INTEL_XL710::read_PHY_reg(int reg, int page)
{
   //write page(Register 22)
   if (reg != 22)
   {
      *MDIO_point =  MDIO_WRITE | ((0x16 & PHY_REG_NUM) << 16) | (page & PHY_DATA_LEN);
      usleep(1);
   }

   *MDIO_point =  MDIO_READ | ((reg & PHY_REG_NUM) << 16);
   usleep(1);
   return *MDIO_point & 0x0FFFF;
}



int INTEL_XL710::write_PHY_reg(int reg, int page , int data)
{
   //write page(Register 22)
   if (reg != 22)
   {
      *MDIO_point =  MDIO_WRITE | ((0x16 & PHY_REG_NUM) << 16) | (page & PHY_DATA_LEN);
      usleep(1);
   }

   *MDIO_point =  MDIO_WRITE | ((reg & PHY_REG_NUM) << 16) | (data & PHY_DATA_LEN);
   usleep(1);

   return 0;
}



ErrorMsg* INTEL_XL710::Get_self_test_result(struct ethtool_test *test, TestType index, INT32 *test_result)
{
    //printf("test->data[0]:%d  test->data[1]:%d  test->data[2]:%d  test->data[3]:%d  test->data[4]:%d\n"
    //       ,test->data[0], test->data[1], test->data[2], test->data[3], test->data[4]);

    switch(index)
    {
        case Register:
            *test_result = test->data[0];
            if (*test_result == 1)
                return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SENDIOCTL | ERROR_ETHERNET_DDAPI_REGISTER_TEST_FAIL), NULL, NULL);
            break;
        case EepromValidation:
            *test_result = test->data[1];
            if (*test_result == 1)
                return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SENDIOCTL | ERROR_ETHERNET_DDAPI_EEPROM_VALIDATION_TEST_FAIL), NULL, NULL);
            break;
        case Interrupt:
            *test_result = test->data[2];
            if (*test_result == 1)
                return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SENDIOCTL | ERROR_ETHERNET_DDAPI_INTERRUPT_TEST_FAIL), NULL, NULL);
            break;
        case InternalLoopback:
            *test_result = test->data[3];
            if (*test_result == 1)
                return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SENDIOCTL | ERROR_ETHERNET_DDAPI_INTERNAL_LOOPBACK_TEST_FAIL), NULL, NULL);
            break;
        case LinkStatus:
            *test_result = test->data[4];
            if (*test_result == 1)
                return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SENDIOCTL | ERROR_ETHERNET_DDAPI_MDI_REGISTER_TEST_FAIL), NULL, NULL);
            break;
        default:
            return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SENDIOCTL | ERROR_ETHERNET_DDAPI_SENDIOCTL_UNDEFINED), NULL, NULL);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_XL710::SetupSpeed(string nic, INT32 speed)
{
    char command_string[50];

    sprintf(command_string, "ifconfig %s mtu 9700\n", nic.c_str());
    system(command_string);

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_XL710::GetMAC(string ethernet_name, string *mac_address)
{
    INT32  err, fd, i, offset, eeprom_address, PCI_function_id;
    struct ethtool_eeprom *eeprom;
    struct ifreq ifr;
    char   string_temp[3];

    errno = 0;

    if (ethernet_name.length() == 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DRIVER_XL710_GET_MAC | ERROR_ETHERNET_DDAPI_NO_SPECIFY_NIC), NULL, NULL);
    }

    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DRIVER_XL710_GET_MAC | ERROR_ETHERNET_DDAPI_SOCKET_OPEN_FAIL), NULL, NULL);
    }

    strncpy(ifr.ifr_name, ethernet_name.c_str(), sizeof(ifr.ifr_name));

    eeprom = (ethtool_eeprom *)calloc(1, sizeof(*eeprom) + 6);
    if (!eeprom)
    {
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DRIVER_XL710_GET_MAC | ERROR_ETHERNET_DDAPI_ALLOCAT_EEPROM_FAIL), NULL, NULL);
    }

    eeprom_address = 0x90; //EMP SR Settings Pointer

    eeprom->cmd = ETHTOOL_GEEPROM;
    eeprom->len = 2;
    eeprom->offset = eeprom_address;

    ifr.ifr_data = (__caddr_t)eeprom;

    //Get eeprom content
    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0)
    {
        //perror("Cannot do get EEPROM");
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DRIVER_XL710_GET_MAC | ERROR_ETHERNET_DDAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    eeprom_address = ((eeprom->data[1] << 8) + eeprom->data[0]) * 2; //EMP SR Settings address 
    eeprom_address = eeprom_address + 0x30; //PF MAC Address Pointer
    
    eeprom->offset = eeprom_address;

    //Get eeprom content
    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0)
    {
        //perror("Cannot do get EEPROM");
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DRIVER_XL710_GET_MAC | ERROR_ETHERNET_DDAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    offset = ((eeprom->data[1] << 8) + eeprom->data[0]) * 2; //PF MAC Address offset
    eeprom_address = eeprom_address + offset;

    //Get PCI function ID,
    //0 for prot 1
    //1 for prot 2
    //2 for prot 3
    //3 for prot 4
    GetPCIFunction(ethernet_name, &PCI_function_id);
    eeprom_address = eeprom_address + (8 * PCI_function_id);
    
    eeprom->len = 6;
    eeprom->offset = eeprom_address + 2;

    //Get eeprom content
    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0)
    {
        //perror("Cannot do get EEPROM");
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DRIVER_XL710_GET_MAC | ERROR_ETHERNET_DDAPI_IOCTL_FAIL), strerror(errno), NULL);
    }   


    for (i = 0; i < eeprom->len; i++)
    {
        sprintf(string_temp, "%02X",eeprom->data[i]);
        *mac_address += string_temp;
        if ( i != 5 )
        {
            *mac_address += ":";
        }
    }

    free(eeprom);
    close(fd);

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_XL710::SetMAC(string ethernet_name, string *mac_address)
{
    int     domain, bus, device, func, result = 0;
    string  command = "eeupdate64e";
    char    parameter[512];


    //Get XL710 PCI BDF
    m_errorCode = GetPCIBusDeviceFunction(ethernet_name, &domain, &bus, &device, &func);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED), "XL710 get PCI BDF fail", m_errorCode);
    }
    //printf("bus:%x  device:%x  func:%x\n",bus, device, func);

    sprintf(parameter, " /bus=%d /dev=%d /fun=%d /MAC=%s%s%s%s%s%s >> /dev/null",bus, device, func, mac_address[0].c_str(), 
    mac_address[1].c_str(), mac_address[2].c_str(), mac_address[3].c_str(), mac_address[4].c_str(), mac_address[5].c_str());

    command.append(parameter);
    //printf("%s\n", command.c_str());
    result = system(command.c_str());
    if (result != 0)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED), "I210 program EEPROM fail", m_errorCode);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_XL710::GetEEPROM(string ethernet_name, INT32 StartAddress, INT32 Length, INT32 *data)
{
    struct ethtool_eeprom *eeprom;
    struct ifreq          ifr;
           INT32          err, fd, i;

    errno = 0;

    if (ethernet_name.length() == 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_NO_SPECIFY_NIC), NULL, NULL);
    }

    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_SOCKET_OPEN_FAIL), NULL, NULL);
    }

    strncpy(ifr.ifr_name, ethernet_name.c_str(), sizeof(ifr.ifr_name));

    eeprom = (ethtool_eeprom *)calloc(1, sizeof(*eeprom) + Length);
    if (!eeprom)
    {
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_ALLOCAT_EEPROM_FAIL), NULL, NULL);
    }

    eeprom->cmd    = ETHTOOL_GEEPROM;
    eeprom->len    = Length;
    eeprom->offset = StartAddress;

    ifr.ifr_data = (__caddr_t)eeprom;

    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0)
    {
        //perror("Cannot do get EEPROM");
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    for (i = 0; i < eeprom->len; i++)
    {
        data[i] = eeprom->data[i];
    }

    free(eeprom);
    close(fd);

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_XL710::SetEEPROM(string ethernet_name, INT32 StartAddress, INT32 *data, INT32 Length)
{
    int     domain, bus, device, func, result = 0;
    string  command;
    char    parameter[512];
    int     word_address = 0, remainder = 0, read_byte_data[2], word_data = 0, i;

    //Get XL710 PCI BDF
    m_errorCode = GetPCIBusDeviceFunction(ethernet_name, &domain, &bus, &device, &func);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED), "XL710 get PCI BDF fail", m_errorCode);
    }

    for(i = 0; i < Length; i++)
    {
        //Transform address to wrod formate
        word_address = (StartAddress + i) / 2;
        remainder = (StartAddress + i) % 2;
        //printf("word_address:0x%02x   remainder:%d\n", word_address, remainder);

        //Get the word data
        GetEEPROM(ethernet_name, word_address * 2, 2, read_byte_data);
        //printf("read_byte_data[0]:0x%02x  read_byte_data[1]:0x%02x\n", read_byte_data[0], read_byte_data[1]);
        word_data = (read_byte_data[1] << 8) | read_byte_data[0];
        //printf("word_data:0x%04x\n", word_data);

        if(remainder == 0)
        {   //Write low byte
            word_data = word_data & 0xff00;
            word_data = word_data | data[i];
        }
        else
        {   //Write high byte
            word_data = word_data & 0x00ff;
            word_data = word_data | (data[i] << 8);
        }
        //printf("Modify word_data:0x%04x\n", word_data);

        //Ready to write word
        command.clear();
        command = "eeupdate64e";
        sprintf(parameter, " /bus=%d /dev=%d /fun=%d /WW 0x%x 0x%x >> /dev/null",bus, device, func, word_address, word_data);
        command.append(parameter);
        //printf("%s\n", command.c_str());
        result = system(command.c_str());
        if (result != 0)
        {
            return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DRIVER_XL710_SET_EEPROM), "XL710 set EEPROM fail", m_errorCode);
        }
    }

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_XL710::Program(string ethernet_name, string filePath)
{
    int     domain, bus, device, func, result = 0;
    string  command = "eeupdate64e";
    char    parameter[512];

    //Get I210 PCI BDF
    m_errorCode = GetPCIBusDeviceFunction(ethernet_name, &domain, &bus, &device, &func);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED), "I210 get PCI BDF fail", m_errorCode);
    }
    //printf("bus:%x  device:%x  func:%x\n",bus, device, func);

    sprintf(parameter, " /bus=%d /dev=%d /fun=%d /D %s >> /dev/null",bus, device, func, filePath.c_str());


    command.append(parameter);
    //printf("%s\n", command.c_str());
    result = system(command.c_str());
    if (result != 0)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED), "I210 program EEPROM fail", m_errorCode);
    }

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_XL710::ReadSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 Length, INT32 *data)
{
    unsigned int *addr, *BAR_point;
    int fd, i, PCI_function_id;


    fd = open(Get_Node().c_str(), O_RDWR | O_SYNC);
    if (fd < 0)
    {
        //printf("%s file open error!\n",Get_Node().c_str());
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EXTERNAL_LOOPBACK | ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL), NULL, NULL);
    }

    addr = (unsigned int *)mmap(NULL, 0x100000, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0x00);
    GetPCIFunction(ethernet_name, &PCI_function_id);
    BAR_point = addr + ((GLGEN_I2CCMD + 4 * PCI_function_id) / 4);

    for (i = 0; i < Length; i++)
    {           //bit 27:Read op    bit 26~24:PHYADD               bit 23~16:Register Address
        *BAR_point = 0x08000000 | ((DeviceAddress & 0x0f) << 24) | ((StartAddress + i) << 16);
        //Wait for operation on the I2C interface has completed.
        while ((0x20000000 & *BAR_point) == 0) {usleep(1);}
        data[i] = *BAR_point & 0xff;
    }

    munmap(addr, 0x100000);
    close(fd);

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_XL710::WriteSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 data)
{
    unsigned int *addr, *BAR_point, SFP_data;
    int fd, PCI_function_id;


    fd = open(Get_Node().c_str(), O_RDWR | O_SYNC);
    if (fd < 0)
    {
        //printf("%s file open error!\n",Get_Node().c_str());
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EXTERNAL_LOOPBACK | ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL), NULL, NULL);
    }

    addr = (unsigned int *)mmap(NULL, 0x100000, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0x00);
    GetPCIFunction(ethernet_name, &PCI_function_id);
    BAR_point = addr + ((GLGEN_I2CCMD + 4 * PCI_function_id) / 4);

    //First, read data out
              //bit 27:Read op    bit 26~24:PHYADD               bit 23~16:Register Address
    *BAR_point = 0x08000000 | ((DeviceAddress & 0x0f) << 24) | ((StartAddress) << 16);
    //Wait for operation on the I2C interface has completed.
    while ((0x20000000 & *BAR_point) == 0) {usleep(1);}
    SFP_data = *BAR_point & 0xffff;
    //printf("Read data: 0x%04x\n", SFP_data);

    //Then write data
              //bit 27:Wead op   bit 26~24:PHYADD             bit 23~16:Register Address   bit 0~15: date to write
    *BAR_point = 0x00000000 | ((DeviceAddress & 0x0f) << 24) | ((StartAddress) << 16) |    ((SFP_data & 0xff00) | data);
    //Wait for operation on the I2C interface has completed.
    while ((0x20000000 & *BAR_point) == 0) {usleep(1);}

    munmap(addr, 0x100000);
    close(fd);

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_XL710::getLEDName(string nic, list<string> *LED)
{

    LED->push_back(nic + "_led0");
    LED->push_back(nic + "_led1");

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_XL710::setLEDStatus(string ethernet_name, string in_strLEDName, string in_strStatus)
{
    unsigned int *addr, *LED_point;
    int fd, PCI_function_id;


    fd = open(Get_Node().c_str(), O_RDWR | O_SYNC);
    if (fd < 0)
    {
        //printf("%s file open error!\n",Get_Node().c_str());
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EXTERNAL_LOOPBACK | ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL), NULL, NULL);
    }

    addr = (unsigned int *)mmap(NULL, 0x90000, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0x00);
    GetPCIFunction(ethernet_name, &PCI_function_id);

    if (in_strLEDName == "led0")
    {
        LED_point = addr + (GLGEN_GPIO_CTL + (0x04 * (22 + (PCI_function_id * 2) + 0))) / 4;
        //printf("%x\n", GLGEN_GPIO_CTL + (0x04 * (22 + (PCI_function_id * 2) + 0)));
    }
    if (in_strLEDName == "led1")
    {
        LED_point = addr + (GLGEN_GPIO_CTL + (0x04 * (22 + (PCI_function_id * 2) + 1))) / 4;
        //printf("%x\n", GLGEN_GPIO_CTL + (0x04 * (22 + (PCI_function_id * 2) + 1)));
    }

    if (in_strStatus == "on")
    {
        *LED_point = (*LED_point & 0xFFFE07FF) | 0x0000F000;
    }
    else if (in_strStatus == "off")
    {
        *LED_point = (*LED_point & 0xFFFE07FF);
    }
    else
    {
        munmap(addr, 0x90000);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL), "Undefine LED status" , NULL);
    }

    munmap(addr, 0x90000);
    close(fd);

    return NO_ERROR_MSG;
}



ErrorMsg* INTEL_XL710::getLEDStatus(string ethernet_name, string in_strLEDName, string *out_strpStatus)
{
    unsigned int *addr, *LED_point;
    int fd, PCI_function_id;


    fd = open(Get_Node().c_str(), O_RDWR | O_SYNC);
    if (fd < 0)
    {
        //printf("%s file open error!\n",Get_Node().c_str());
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EXTERNAL_LOOPBACK | ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL), NULL, NULL);
    }

    addr = (unsigned int *)mmap(NULL, 0x90000, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0x00);
    GetPCIFunction(ethernet_name, &PCI_function_id);

    if (in_strLEDName == "led0")
    {
        LED_point = addr + (GLGEN_GPIO_CTL + (0x04 * (22 + (PCI_function_id * 2) + 0))) / 4;
        //printf("%x\n", GLGEN_GPIO_CTL + (0x04 * (22 + (PCI_function_id * 2) + 0)));
    }
    if (in_strLEDName == "led1")
    {
        LED_point = addr + (GLGEN_GPIO_CTL + (0x04 * (22 + (PCI_function_id * 2) + 1))) / 4;
        //printf("%x\n", GLGEN_GPIO_CTL + (0x04 * (22 + (PCI_function_id * 2) + 1)));
    }

    //printf("%x\n", *LED_point);
    //printf("%x\n", (*LED_point & 0xF000) >> 12);
    //Check bit 12~15(LED_MODE).
    //0x0 => LED OFF, 0xF => LED ON
    if (((*LED_point & 0xF000) >> 12) == 0x0)
    {
        *out_strpStatus = "off";
    }
    else if (((*LED_point & 0xF000) >> 12) == 0xF)
    {
        *out_strpStatus = "on";
    }
    else
    {
        munmap(addr, 0x90000);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_OPEN_NODE_FAIL), "Undefine LED status" , NULL);
    }

    munmap(addr, 0x90000);
    close(fd);

    return NO_ERROR_MSG;
}


