#ifndef QLOGIC_EP8324_H
#define QLOGIC_EP8324_H

#include "EthernetBaseClass.hpp"
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include "PCIDDAPI.hpp"
#include "MemoryDDAPI.hpp"


//EP8324 define
#define FLASH_CNA_BOARD_CONFIG_START       0x370000  //Board Configuration
#define FLASH_CNA_BOARD_CONFIG_CHECKSUM    0x37001C
#define FUNCTION_0_MAC                     0x370400  //FC port 1
#define FUNCTION_1_MAC                     0x370408  //FC port 2
#define FUNCTION_2_MAC                     0x370410  //Ethernet port 1
#define FUNCTION_3_MAC                     0x370418  //Ethernet port 2
#define FUNCTION_4_MAC                     0x370420
#define FUNCTION_5_MAC                     0x370428
#define FLASH_CNA_BOARD_CONFIG_SIZE        0x5000


#define SPI_FLASH_START_ADDRESS   0x7F800000
#define FLASH_ACCESS_DATA_FLAG    0x80000000

#define CNA_FLASH_SECTOR_SIZE (64*1024)
#define EP8324_FLASH_SIZE     (16*1024*1024)


#define MAILBOX_REGISTER_COUNT    32
#define SFP_EEPROM_SIZE           256


typedef struct {
    uint32_t    out_mb;                     /* outbound from driver */
    uint32_t    in_mb;                      /* Incoming from RISC */
    uint16_t    mb[MAILBOX_REGISTER_COUNT];
#define MBX_TOV_SECONDS 30
    uint32_t    tov;
} mbx_cmd_t;


#define NAME_STR_SIZE    20

struct ep8324_ioctl_cb {
    char well_known_name[NAME_STR_SIZE + 1];
    union {
        mbx_cmd_t    mc;

        struct {
            int         fwloadbin;
            uint32_t    srisc_address;
        } lrcb;

        struct {
            uint16_t    mb0;
            uint8_t     device_address;
            uint16_t    offset;
            uint16_t    size;
            uint8_t     data[SFP_EEPROM_SIZE];
        } sfpcb;

        struct {
            uint32_t     faddr;
            uint32_t     dwords;
            //uint32_t    *dword_buf;
            uint32_t     dword_buf[64];
        } spicb;

        struct {
#define EP8324_LOOPBACK_SETUP      1
#define EP8324_LOOPBACK_EXECUTE    2
#define EP8324_LOOPBACK_CLEANUP    3
            uint32_t    cmd;
#define EP8324_LOOPBACK_NO_ERROR       0
#define EP8324_LOOPBACK_TEST_ERROR     1
#define EP8324_LOOPBACK_NO_MEMORY      2
#define EP8324_LOOPBACK_SETUP_ERROR    3
            uint32_t    result;
#define EP8324_EXTERNAL_LOOPBACK    0
#define EP8324_INTERNAL_LOOPBACK    1
            uint32_t    path;
#define EP8324_LINK_SPEED_AUTO    1
#define EP8324_LINK_SPEED_4G      2
#define EP8324_LINK_SPEED_8G      3
#define EP8324_LINK_SPEED_16G     4
            uint32_t    link_speed;
            uint32_t    seg_count;    /* tx/rx segment count */
            uint32_t    seg_length;   /* segment length */
            uint32_t    pattern;
            uint16_t    mb[6]; /* mailbox status 0, 1, 2 ,3, 18, 19 */
        } loopbackcb;

        struct {
            uint16_t    mb[3]; /* mailbox status 0, 1, 3 */
        } echocb;
    };
};



#define EP8324_IOCTL_LIST_FUNC            _IO('e', 1)
#define EP8324_IOCTL_SETUP_CHIP           _IO('e', 2)
#define EP8324_IOCTL_RESET_CHIP           _IO('e', 3)
#define EP8324_IOCTL_SEND_MBX             _IOWR('e', 4, struct ep8324_ioctl_cb)
#define EP8324_IOCTL_LOAD_RISC            _IOWR('e', 5, struct ep8324_ioctl_cb)
#define EP8324_IOCTL_SFP_READ             _IOWR('e', 6, struct ep8324_ioctl_cb)
#define EP8324_IOCTL_SFP_WRITE            _IOWR('e', 7, struct ep8324_ioctl_cb)
#define EP8324_IOCTL_SPI_READ             _IOWR('e', 8, struct ep8324_ioctl_cb)
#define EP8324_IOCTL_SPI_WRITE            _IOWR('e', 9, struct ep8324_ioctl_cb)
#define EP8324_IOCTL_LOOPBACK_TEST        _IOWR('e', 10, struct ep8324_ioctl_cb)
#define EP8324_IOCTL_ECHO_TEST            _IOWR('e', 11, struct ep8324_ioctl_cb)




class QLOGIC_EP8324 : public EthernetBaseClass
{
    public:
                  QLOGIC_EP8324(STRING);
                  ~QLOGIC_EP8324();

        ErrorMsg* Get_self_test_result(struct ethtool_test *test, TestType index, INT32 *test_result);
        ErrorMsg* SetupSpeed(string ethernet_name, INT32 speed);
        ErrorMsg* GetMAC(string ethernet_name, string *mac_address);
        ErrorMsg* SetMAC(string ethernet_name, string *mac_address);
        ErrorMsg* SPIRead(string ethernet_name, UINT32 spiAddress, UINT32 *returnedDataValue);
        ErrorMsg* SPIWrite(string ethernet_name, UINT32 spiAddress, UINT32 dataToWrite);
        ErrorMsg* SPIBufferWrite(string ethernet_name, UINT32 spiAddress, UINT32 *dataBuffer, UINT32 dwordsToWrite, UINT32 showProgress);
        ErrorMsg* UpdateFlashChecksum(string ethernet_name, UINT32 startAddress, UINT32 endAddress, UINT32 checksumAddress);
        ErrorMsg* OpenDevice(void);
        ErrorMsg* GetEEPROM(string ethernet_name, INT32 StartAddress, INT32 Length, INT32 *data);
        ErrorMsg* SetEEPROM(string ethernet_name, INT32 StartAddress, INT32 *data, INT32 Length);
        ErrorMsg* Program(string ethernet_name, string filePath);
        ErrorMsg* ReadSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 Length, INT32 *data);
        ErrorMsg* WriteSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 data);

    protected:

    private:
        unsigned int *MDIO_point;
        ErrorMsg     *m_errorCode;

                 int deviceHandle;
};

#endif

