#include "ExpanderDDAPI.hpp"
#include <unistd.h>
#include <sys/io.h>
#include <fcntl.h>
#include <linux/rtc.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <stdio.h>

ExpanderDDAPI :: ExpanderDDAPI(STRING className) : BaseClassDDAPI(className)
{
    FILE            *pControllerFile = NULL;
    FILE            *pPHYFile = NULL;
    FILE            *pFile = NULL;
    CHAR            strTemp[EXPANDER_DEVICE_NAME_LENGTH + 1];
    CHAR            cmd[170];
    CHAR            strPhyID[3];
    CHAR            strControllerName[EXPANDER_DEVICE_NAME_LENGTH + 1];
    EXP_DEV_CONFIG  expDevice;
    CHAR            *pch;
    BOOL            bIsInsert;
    UINT32          logicalID;
    UINT32          controllerNum;
    UINT32          driveID;
    UINT32          phyMap[] = {16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39};

    logicalID = 0;
    controllerNum = 0;
    
    if((pControllerFile = popen("ls /sys/class/sas_expander/", "r")) == NULL)
    {
        addErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_CONSTRUCTOR | ERROR_EXPANDER_DDAPI_POPEN_FAIL), NULL);
        return;
    }

    //Get Expander Controller
    while((fgets(strControllerName, sizeof(strControllerName), pControllerFile)) != NULL)
    {
        expDevice.devType = EXP_CONTROLLER;
        strControllerName[strlen(strControllerName) - 1] = '\0';
        memset(expDevice.deviceName, 0, EXPANDER_DEVICE_NAME_LENGTH);
        strncpy(expDevice.deviceName, strControllerName, EXPANDER_DEVICE_NAME_LENGTH);
        sprintf(expDevice.wellKnownName, "EXP%d", controllerNum);
        expDevice.logicalID = logicalID;
        m_expDevice.push_back(expDevice);
        logicalID++;
        
        //Get PHY
        sprintf(cmd, "ls /sys/class/sas_expander/%s/device/", strControllerName);
        if((pPHYFile = popen(cmd, "r")) == NULL)
        {
            addErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_CONSTRUCTOR | ERROR_EXPANDER_DDAPI_POPEN_FAIL), NULL);
            return;
        }
        
        while((fgets(strTemp, sizeof(strTemp), pPHYFile)) != NULL)
        {
            if(strstr(strTemp, "phy") != NULL)
            {
                strTemp[strlen(strTemp) - 1] = '\0';
                
                expDevice.devType = EXP_PHY;
                memset(expDevice.deviceName, 0, EXPANDER_DEVICE_NAME_LENGTH);
                strncpy(expDevice.deviceName, strTemp, EXPANDER_DEVICE_NAME_LENGTH);
                
                pch = strtok (strTemp, ":");
                if (pch != NULL)
                    pch = strtok (NULL, ":");
                if (pch != NULL)
                    pch = strtok (NULL, ":");
                sprintf(expDevice.wellKnownName, "EXP%d-PHY%d", controllerNum, atoi(pch));
                expDevice.logicalID = logicalID;
                
                bIsInsert = FALSE;
				if (m_expDevice.begin() != m_expDevice.end())
				{
	                for (list<EXP_DEV_CONFIG>::iterator iter = m_expDevice.begin(); iter != m_expDevice.end(); iter++)
	                {
	                    if((compareStrNum(*iter, expDevice)) == FALSE)
	                    {
	                        m_expDevice.insert(iter, expDevice);
	                        bIsInsert = TRUE;
	                        break;
	                    }
	                }
				}
				
                if (bIsInsert == FALSE)
                {
                    m_expDevice.push_back(expDevice);
                }
                logicalID++;
            }
            else if(strstr(strTemp, "port") != NULL)
            {
                strTemp[strlen(strTemp) - 1] = '\0';
                
                expDevice.devType = EXP_PORT;
                memset(expDevice.deviceName, 0, EXPANDER_DEVICE_NAME_LENGTH);
                strncpy(expDevice.deviceName, strTemp, EXPANDER_DEVICE_NAME_LENGTH);
                
                sprintf(cmd, "cat /sys/class/sas_expander/%s/device/%s/end_device%s/sas_device/end_device%s/phy_identifier", strControllerName, expDevice.deviceName, strstr(expDevice.deviceName, "-"), strstr(expDevice.deviceName, "-"));

                if((pFile = popen(cmd, "r")) == NULL)
                {
                    addErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_CONSTRUCTOR | ERROR_EXPANDER_DDAPI_POPEN_FAIL), NULL);
                    return;
                }
                
                fgets(strPhyID, sizeof(strPhyID), pFile);
                pclose(pFile);

                for (driveID = 0; driveID < sizeof(phyMap) / 4; driveID++)
                {
                    if (phyMap[driveID] == atoi(strPhyID))
                    {
                        break;
                    }
                }
                
                if (driveID < sizeof(phyMap) / 4)
                {
                    sprintf(expDevice.wellKnownName, "EXP%d-PORT%d", controllerNum, driveID + 1);
                    expDevice.logicalID = logicalID;
                    bIsInsert = FALSE;
                    if (m_expDevice.begin() != m_expDevice.end())
                    {
                        for (list<EXP_DEV_CONFIG>::iterator iter = m_expDevice.begin(); iter != m_expDevice.end(); iter++)
                        {
                            if((compareStrNum(*iter, expDevice)) == FALSE)
                            {
                                m_expDevice.insert(iter, expDevice);
                                bIsInsert = TRUE;
                                break;
                            }
                        }
                    }
                    
                    if (bIsInsert == FALSE)
                    {
                        m_expDevice.push_back(expDevice);
                    }
                    logicalID++;
                }

            }
            
            
            
        }
        if (pPHYFile != NULL)
            pclose(pPHYFile);
        controllerNum++;
    }

	if (pControllerFile != NULL)
	    pclose(pControllerFile);
}

ExpanderDDAPI :: ~ExpanderDDAPI()
{

    
}

void ExpanderDDAPI :: getDeviceNameByID(EXP_DEVICE devType, UINT32 index, string *strWellKnownName)
{
    UINT32 countIndex = 0;
    
    *strWellKnownName = "";

	if (m_expDevice.begin() != m_expDevice.end())
	{
	    for (list<EXP_DEV_CONFIG>::iterator iter = m_expDevice.begin(); iter != m_expDevice.end(); iter++)
	    {
	        if (iter->devType == devType)
	        {
	            if (countIndex == index)
	            {
	                *strWellKnownName = iter->wellKnownName;
	                break;
	            }
	            countIndex++;
	        }
	    }
	}
}

ErrorMsg * ExpanderDDAPI :: getLogicalIDByName(string wellKnownName, UINT32 *logicalID)
{
    ERRORCODE   eErrorCode;
    
    wellKnownName = convertStringToUpperCase(wellKnownName);
    
    eErrorCode = ERROR_EXPANDER_DDAPI_UNKNOWN_NAME;
	if (m_expDevice.begin() != m_expDevice.end())
	{
	    for (list<EXP_DEV_CONFIG>::iterator iter = m_expDevice.begin(); iter != m_expDevice.end(); iter++)
	    {
	        if (wellKnownName.compare(iter->wellKnownName) == 0)
	        {
	            *logicalID = iter->logicalID;
	            eErrorCode = NO_ERROR;
	        }
	    }
	}
    
    if (eErrorCode != NO_ERROR)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_GET_ID_BY_NAME | eErrorCode), wellKnownName.c_str());
    }
    
    return NO_ERROR_MSG;
}

ErrorMsg * ExpanderDDAPI :: getDataByID(UINT32 logicalID, EXP_DATAELEMENT dataElement, string *strReturnData)
{
    ERRORCODE                       eErrorCode;
    list<EXP_DEV_CONFIG>::iterator  iter;
    CHAR                            cmd[200];
    CHAR                            controllerDevName[EXPANDER_DEVICE_NAME_LENGTH + 1];
    CHAR                            strTemp[100];
    CHAR                            *pFirstCh, *pLastCh;
    FILE                            *pDirFile, *pFile;
    UINT32                          phyCount;
    
    phyCount = 0;
    pDirFile = NULL;
    pFile    = NULL;
    strReturnData->assign("");
    eErrorCode = ERROR_EXPANDER_DDAPI_UNKNOWN_ID;
	if (m_expDevice.begin() != m_expDevice.end())
	{
	    for (iter = m_expDevice.begin(); iter != m_expDevice.end(); iter++)
	    {
	        if (iter->logicalID == logicalID)
	        {
	            eErrorCode = NO_ERROR;
	            break;
	        }
	    }
	}

    if (eErrorCode == NO_ERROR)
    {
        if (iter->devType == EXP_CONTROLLER)
        {
            switch(dataElement)
            {
                case EXP_DATAELEMENT_ATTACHED_TYPE:
                    sprintf(cmd, "cat /sys/class/sas_expander/%s/device/sas_device/%s/device_type", iter->deviceName, iter->deviceName);
                    break;
                case EXP_DATAELEMENT_LINK:
                case EXP_DATAELEMENT_SPEED:
                    strReturnData->assign("UNSUPPORT");
                    return NO_ERROR_MSG;
                    break;
                case EXP_DATAELEMENT_ADDRESS:
                    sprintf(cmd, "cat /sys/class/sas_expander/%s/device/sas_device/%s/sas_address", iter->deviceName, iter->deviceName);
                    break;
                default:
                    break;
            }

            if((pFile = popen(cmd, "r")) == NULL)
            {
                return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_GET_DATA_BY_ID | ERROR_EXPANDER_DDAPI_POPEN_FAIL));
            }

            fgets(strTemp, sizeof(strTemp), pFile);
            strTemp[strlen(strTemp) - 1] = '\0';
            
            for (INT32 index = 0; strTemp[index] && (dataElement != EXP_DATAELEMENT_ADDRESS); index++)
            {
                strTemp[index] = toupper(strTemp[index]);
            }
            strReturnData->assign(strTemp);
        }
        else if (iter->devType == EXP_PHY)
        {
            pFirstCh = strrchr(iter->deviceName,'-') + 1;
            pLastCh = strrchr(pFirstCh,':');
            strncpy(strTemp, pFirstCh, (pLastCh - pFirstCh));
            strTemp[pLastCh - pFirstCh] = '\0';
            sprintf(controllerDevName, "expander-%s", strTemp);
            
            switch(dataElement)
            {
                case EXP_DATAELEMENT_ATTACHED_TYPE:
                    sprintf(cmd, "cat /sys/class/sas_expander/%s/device/%s/sas_phy/%s/device_type", controllerDevName, iter->deviceName, iter->deviceName);
                    break;
                case EXP_DATAELEMENT_LINK:
                    sprintf(cmd, "cat /sys/class/sas_expander/%s/device/%s/sas_phy/%s/negotiated_linkrate", controllerDevName, iter->deviceName, iter->deviceName);
                    break;
                case EXP_DATAELEMENT_SPEED:
                    sprintf(cmd, "cat /sys/class/sas_expander/%s/device/%s/sas_phy/%s/negotiated_linkrate", controllerDevName, iter->deviceName, iter->deviceName);
                    break;
                case EXP_DATAELEMENT_ADDRESS:
                    sprintf(cmd, "cat /sys/class/sas_expander/%s/device/%s/sas_phy/%s/sas_address", controllerDevName, iter->deviceName, iter->deviceName);
                    break;
                default:
                    break;
            }
            
            if((pFile = popen(cmd, "r")) == NULL)
            {
                return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_GET_DATA_BY_ID | ERROR_EXPANDER_DDAPI_POPEN_FAIL));
            }

            fgets(strTemp, sizeof(strTemp), pFile);
            strTemp[strlen(strTemp) - 1] = '\0';
            
            for (INT32 index = 0; strTemp[index] && (dataElement != EXP_DATAELEMENT_ADDRESS); index++)
            {
                strTemp[index] = toupper(strTemp[index]);
            }
            
            switch(dataElement)
            {
                case EXP_DATAELEMENT_LINK:
                    if (strstr(strTemp, "GBIT") == NULL)
                    {
                        strReturnData->assign("DOWN");
                    }
                    else
                    {
                        strReturnData->assign("UP");
                    }
                    break;
                default:
                    strReturnData->assign(strTemp);
                    break;
            }
        }
        else if (iter->devType == EXP_PORT)
        {
            pFirstCh = strrchr(iter->deviceName,'-') + 1;
            pLastCh = strrchr(pFirstCh,':');
            strncpy(strTemp, pFirstCh, (pLastCh - pFirstCh));
            strTemp[pLastCh - pFirstCh] = '\0';
            sprintf(controllerDevName, "expander-%s", strTemp);
       
            sprintf(cmd, "ls /sys/class/sas_expander/%s/device/%s", controllerDevName, iter->deviceName);
            if((pDirFile = popen(cmd, "r")) == NULL)
            {
                return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_GET_DATA_BY_ID | ERROR_EXPANDER_DDAPI_POPEN_FAIL));
            }
            
            while ((fgets(strTemp, sizeof(strTemp), pDirFile)) != NULL)
            {
                if (strchr(strTemp, ':') != NULL)
                {
                    strTemp[strlen(strTemp) - 1] = '\0';

                    if (strstr(strTemp, "phy") != NULL)
                    {
                        switch(dataElement)
                        {
                            case EXP_DATAELEMENT_LINK:
                            case EXP_DATAELEMENT_SPEED:
                                sprintf(cmd, "cat /sys/class/sas_expander/%s/device/%s/%s/sas_phy/%s/negotiated_linkrate", 
                                             controllerDevName, iter->deviceName, strTemp, strTemp);
                                break;
                            default:
                                break;
                        }
                        
                        if((pFile = popen(cmd, "r")) == NULL)
                        {
                            pclose(pDirFile);
                            return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_GET_DATA_BY_ID | ERROR_EXPANDER_DDAPI_POPEN_FAIL));
                        }

                        fgets(strTemp, sizeof(strTemp), pFile);
                        strTemp[strlen(strTemp) - 1] = '\0';
                        
                        for (INT32 index = 0; strTemp[index] && (dataElement != EXP_DATAELEMENT_ADDRESS); index++)
                        {
                            strTemp[index] = toupper(strTemp[index]);
                        }
                        
                        if ((phyCount > 0) && (strcmp(strTemp, strReturnData->c_str()) != 0))
                        {
                            pclose(pDirFile);
                            pclose(pFile);
                            return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_GET_DATA_BY_ID | ERROR_EXPANDER_DDAPI_DIFF_PHY_SPEED));
                        }
                        
                        switch(dataElement)
                        {
                            case EXP_DATAELEMENT_LINK:
                                if (strstr(strTemp, "GBIT") == NULL)
                                {
                                    strReturnData->assign("DOWN");
                                }
                                else
                                {
                                    strReturnData->assign("UP");
                                }
                                break;
                            default:
                                strReturnData->assign(strTemp);
                                break;
                        }
                        phyCount++;
                    }
                    else
                    {
                        switch(dataElement)
                        {
                            case EXP_DATAELEMENT_ATTACHED_TYPE:
                                sprintf(cmd, "cat /sys/class/sas_expander/%s/device/%s/%s/sas_device/%s/device_type", 
                                             controllerDevName, iter->deviceName, strTemp, strTemp);
                                break;
                            case EXP_DATAELEMENT_ADDRESS:
                                sprintf(cmd, "cat /sys/class/sas_expander/%s/device/%s/%s/sas_device/%s/sas_address", 
                                             controllerDevName, iter->deviceName, strTemp, strTemp);
                                break;
                            default:
                                break;
                        }
                        
                        if((pFile = popen(cmd, "r")) == NULL)
                        {
                            pclose(pDirFile);
                            return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_GET_DATA_BY_ID | ERROR_EXPANDER_DDAPI_POPEN_FAIL));
                        }

                        fgets(strTemp, sizeof(strTemp), pFile);
                        strTemp[strlen(strTemp) - 1] = '\0';
                        
                        for (INT32 index = 0; strTemp[index] && (dataElement != EXP_DATAELEMENT_ADDRESS); index++)
                        {
                            strTemp[index] = toupper(strTemp[index]);
                        }
                        strReturnData->assign(strTemp);
                    }
                }
            }
            
            if (pDirFile != NULL)
                pclose(pDirFile);
        }
    }

    if (pFile != NULL)
        pclose(pFile);

    if (eErrorCode != NO_ERROR)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_GET_DATA_BY_ID | eErrorCode));
    }
    else
    {
        return NO_ERROR_MSG;
    }
}

ErrorMsg * ExpanderDDAPI :: reset(UINT32 logicalID)
{
    ErrorMsg    *pErrorMsg;
    string      strExpAddr;
    FILE        *pFile = NULL;
    CHAR        strTemp[100];
    CHAR        cmd[100];

    if ((pErrorMsg = getDataByID(logicalID, EXP_DATAELEMENT_ADDRESS, &strExpAddr)) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_EXPANDER_DDAPI_RESET, NULL, pErrorMsg);
    }
    
    sprintf(cmd, "smp_reset_expander -vvvv -s %s -I mpt -a 0xc3800200 -d 0x0c0b080a /dev/mpt3ctl 2>&1", strExpAddr.c_str());

    if((pFile = popen(cmd, "r")) == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_RESET | ERROR_EXPANDER_DDAPI_POPEN_FAIL));
    }
    
    while((fgets(strTemp, sizeof(strTemp), pFile)) != NULL)
    {
        strTemp[strlen(strTemp) - 1] = '\0';
        if (strstr(strTemp, "(IOCStatus)") != NULL)
        {
            break;
        }
        else if (strstr(strTemp, "IOCStatus=0x8090") != NULL)
        {
            memset(strTemp, 0, strlen(strTemp));
            break;
        }
        
    }
    
    if (strTemp[0] != 0)
    {
        pclose(pFile);
        return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_RESET | ERROR_EXPANDER_DDAPI_SMP_CMD_FAIL), strTemp);
    }
        
	pclose(pFile);
        
    return NO_ERROR_MSG;
}

ErrorMsg * ExpanderDDAPI :: getErrorDataByID(UINT32 logicalID, EXP_ERRORELEMENT dataElement, string *strReturnData)
{
    ERRORCODE                       eErrorCode;
    list<EXP_DEV_CONFIG>::iterator  iter;
    CHAR                            cmd[200];
    CHAR                            controllerDevName[EXPANDER_DEVICE_NAME_LENGTH + 1];
    CHAR                            strTemp[100];
    CHAR                            *pFirstCh, *pLastCh;
    FILE                            *pFile;

    strReturnData->assign("");
    
    eErrorCode = ERROR_EXPANDER_DDAPI_UNKNOWN_ID;
	if (m_expDevice.begin() != m_expDevice.end())
	{
	    for (iter = m_expDevice.begin(); iter != m_expDevice.end(); iter++)
	    {
	        if (iter->logicalID == logicalID)
	        {
	            eErrorCode = NO_ERROR;
	            break;
	        }
	    }
	}
    
    if (eErrorCode == NO_ERROR)
    {
        if (iter->devType != EXP_PHY)
        {
            eErrorCode = ERROR_EXPANDER_DDAPI_UNSUPPORT_DEV_TYPE;
        }
    }
    
    if (eErrorCode == NO_ERROR)
    {
        pFirstCh = strrchr(iter->deviceName,'-') + 1;
        pLastCh = strrchr(pFirstCh,':');
        strncpy(strTemp, pFirstCh, (pLastCh - pFirstCh));
        strTemp[pLastCh - pFirstCh] = '\0';
        sprintf(controllerDevName, "expander-%s", strTemp);

        switch(dataElement)
        {
            case EXP_ERRORELEMENT_DISPARITY_ERR:
                sprintf(cmd, "cat /sys/class/sas_expander/%s/device/%s/sas_phy/%s/running_disparity_error_count", controllerDevName, iter->deviceName, iter->deviceName);
                break;
            case EXP_ERRORELEMENT_INVALID_DWORD_ERR:
                sprintf(cmd, "cat /sys/class/sas_expander/%s/device/%s/sas_phy/%s/invalid_dword_count", controllerDevName, iter->deviceName, iter->deviceName);
                break;
            case EXP_ERRORELEMENT_LOSS_DWORD_SYNC:
                sprintf(cmd, "cat /sys/class/sas_expander/%s/device/%s/sas_phy/%s/loss_of_dword_sync_count", controllerDevName, iter->deviceName, iter->deviceName);
                break;
            case EXP_ERRORELEMENT_PHY_RESET_PRBM:
                sprintf(cmd, "cat /sys/class/sas_expander/%s/device/%s/sas_phy/%s/phy_reset_problem_count", controllerDevName, iter->deviceName, iter->deviceName);
                break;
            default:
                break;
        }

        if((pFile = popen(cmd, "r")) == NULL)
        {
            return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_GET_ERROR_DATA_BY_ID | ERROR_EXPANDER_DDAPI_POPEN_FAIL), NULL, NULL);
        }

        fgets(strTemp, sizeof(strTemp), pFile);
        strTemp[strlen(strTemp) - 1] = '\0';
        
        for (INT32 index = 0; strTemp[index]; index++)
        {
            strTemp[index] = toupper(strTemp[index]);
        }
        strReturnData->assign(strTemp);
        pclose(pFile);
    }

    if (eErrorCode != NO_ERROR)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_GET_ERROR_DATA_BY_ID | eErrorCode), NULL, NULL);
    }
    
    return NO_ERROR_MSG;
}

ErrorMsg * ExpanderDDAPI :: clearPHYError()
{
    FILE    *pFile = NULL;
    CHAR    strTemp[20];
    CHAR    cmd[100];
    UINT32  pageID;
    
    strcpy(cmd, "EXP_Enclosure_Data.sh | grep \"^Port ID\" | awk '{printf $3 \"\\n\"}'");
    if((pFile = popen(cmd, "r")) == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_CLEAR_PHY_ERROR | ERROR_EXPANDER_DDAPI_POPEN_FAIL));
    }
    
    if((fgets(strTemp, sizeof(strTemp), pFile)) != NULL)
    {
        strTemp[strlen(strTemp) - 1] = '\0';
    }
    else
    {
        pclose(pFile);
        return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_CLEAR_PHY_ERROR | ERROR_EXPANDER_DDAPI_SMP_CMD_FAIL), cmd);
    }

    if (strcmp(strTemp, "1") == 0)
    {
        pageID = 0x80;
    }
    else if (strcmp(strTemp, "2") == 0)
    {
        pageID = 0x81;
    }
    else
    {
        pclose(pFile);
        return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_CLEAR_PHY_ERROR | ERROR_EXPANDER_DDAPI_SMP_CMD_FAIL), cmd);
    }
    pclose(pFile);
    
    sprintf(cmd, "exp_ses.sh 0 0x%02X w 1 1", pageID);
    if((pFile = popen(cmd, "r")) == NULL)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_CLEAR_PHY_ERROR | ERROR_EXPANDER_DDAPI_POPEN_FAIL));
    }
    
    if((fgets(strTemp, sizeof(strTemp), pFile)) != NULL)
    {
        strTemp[strlen(strTemp) - 1] = '\0';
    }
    else
    {
        pclose(pFile);
        return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_CLEAR_PHY_ERROR | ERROR_EXPANDER_DDAPI_SES_CMD_FAIL), cmd);
    }
    
    if (strcmp(strTemp, "RESULT: PASSED") != 0)
    {
	    pclose(pFile);
        return returnErrorMsg((ERRORCODE)(ERROR_EXPANDER_DDAPI_CLEAR_PHY_ERROR | ERROR_EXPANDER_DDAPI_SES_CMD_FAIL), cmd);
    }
    
	pclose(pFile);
        
        
    return NO_ERROR_MSG;

}


bool ExpanderDDAPI :: compareStrNum(const EXP_DEV_CONFIG &first, const EXP_DEV_CONFIG &second)
{
    CHAR    *pch;
    UINT32  firstNum, secondNum;
    
    if (first.devType != second.devType)
        return TRUE;

    //Compare previous number
    pch = (CHAR*)first.wellKnownName;
    while((*pch < 0x30) || (*pch > 0x39))
    {
        pch++;
    }
    firstNum = atoi(pch);
    

    pch = (CHAR*)second.wellKnownName;
    while((*pch < 0x30) || (*pch > 0x39))
    {
        pch++;
    }
    secondNum = atoi(pch);

    if (firstNum < secondNum)
        return TRUE;
    else if (firstNum > secondNum)
        return FALSE;

    //Compare last number
    if ((pch = strchr((CHAR*)first.wellKnownName, '-')) == NULL)
    {
        return TRUE;
    }
    while((*pch < 0x30) || (*pch > 0x39))
    {
        pch++;
    }
    firstNum = atoi(pch);
    
    if ((pch = strchr((CHAR*)second.wellKnownName, '-')) == NULL)
    {
        return TRUE;
    }
    while((*pch < 0x30) || (*pch > 0x39))
    {
        pch++;
    }
    secondNum = atoi(pch);
    if (firstNum < secondNum)
        return TRUE;
    else
        return FALSE;
}

string ExpanderDDAPI::convertStringToUpperCase(string strInputString)
{
    string strOutputString = "";
    for(string::iterator iter = strInputString.begin(); iter != strInputString.end(); iter++)
    {
        strOutputString += toupper(*iter);
    }
    
    return strOutputString;
}