/*
 * GPIOGeneric.cpp
 *
 *  Created on: Dec 2, 2015
 *      Author: ianfang
 */
#include "GPIO_FPGADriver.hpp"



GPIO_FPGADriver :: GPIO_FPGADriver (STRING  ClassName): GPIOGeneric(ClassName)
{


    //PCIDDAPI      *m_PCIDriver;
    //MemoryDDAPI   *m_MemoryDriver;

    m_BaseAddressRegister       = 0;
    m_ConfigurationTableIndex   = 0;

    //New a PCI driver
    m_PCIDriver = new PCIDDAPI("GPIO_FPGA_PCI");

    m_errorCode = m_PCIDriver->getErrorMsg();
    if (m_errorCode != NO_ERROR_MSG)
    {
        delete m_PCIDriver;
        addErrorMsg(ERROR_GPIO_DDAPI_GPIO_FPGA_DRIVER_CONSTRUCTOR, NULL, m_errorCode);
        return;
    }

    //New a memory driver
    m_MemoryDriver = new MemoryDDAPI("GPIO_MEM");

    m_errorCode = m_MemoryDriver->getErrorMsg();
    if (m_errorCode != NO_ERROR_MSG)
    {
        delete m_MemoryDriver;
        delete m_PCIDriver;
        addErrorMsg(ERROR_GPIO_DDAPI_GPIO_FPGA_DRIVER_CONSTRUCTOR, NULL, m_errorCode);
        return;

    }

    //Get the FPGA bar address
    m_PCIDriver->getBarByBDF(0, 0x72, 0x00, 0x00, 0, &m_BaseAddressRegister);
    //printf("Bar[0] -- 0x%x\n", m_BaseAddressRegister);

}


GPIO_FPGADriver :: ~GPIO_FPGADriver ()
{
    if(m_PCIDriver != NULL)
        delete m_PCIDriver;

    if(m_MemoryDriver != NULL)
        delete m_MemoryDriver;
}


ErrorMsg* GPIO_FPGADriver :: readINIFile(CHAR *Line)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    char   *pch;
    char   *p;
    COLUMN_NUMBER eCurrentType;
    UINT32  counter = 0;
    UINT32  offset = 0, msb = 0, lsb = 0;
    BOOL    Status = TRUE, WriteProtect = TRUE;
    string DevName, tempStatus, tempWriteProtect;
    FPGA_DRIVER_DATA_TYPE DataType;
    GPIO_RW_MODE    RWMode;
    DevName.clear();
    tempStatus.clear();
    tempWriteProtect.clear();

    pch = strtok(Line, " \t");

    while (pch != NULL)
    {
        counter++;
        //printf("!%s\n", pch);
        eCurrentType = COLUMN_NUMBER(counter);
        switch(eCurrentType)
        {

            case FIRST:
                DevName = pch;
                break;
            case SECOND:
                //DataType = pch;
                if(strcmp(pch, "GPIO_FPGA_DRIVER_DATA_TYPE_BOOLEAN") == 0)
                {
                    DataType = GPIO_FPGA_DRIVER_DATA_TYPE_BOOLEAN;
                }
                if(strcmp(pch, "GPIO_FPGA_DRIVER_DATA_TYPE_UINT8") == 0)
                {
                    DataType = GPIO_FPGA_DRIVER_DATA_TYPE_UINT8;
                }
                if(strcmp(pch, "GPIO_FPGA_DRIVER_DATA_TYPE_UINT16") == 0)
                {
                    DataType = GPIO_FPGA_DRIVER_DATA_TYPE_UINT16;
                }
                if(strcmp(pch, "GPIO_FPGA_DRIVER_DATA_TYPE_UINT32") == 0)
                {
                    DataType = GPIO_FPGA_DRIVER_DATA_TYPE_UINT32;
                }

                break;
            case THIRD:
                if(strcmp(pch, "GPIO_READ_ONLY") == 0)
                {
                    RWMode = GPIO_READ_ONLY;
                }
                if(strcmp(pch, "GPIO_WRITE_ONLY") == 0)
                {
                    RWMode = GPIO_WRITE_ONLY;
                }
                if(strcmp(pch, "GPIO_WRITE_CLEAR") == 0)
                {
                    RWMode = GPIO_WRITE_CLEAR;
                }
                if(strcmp(pch, "GPIO_READ_WRITE") == 0)
                {
                    RWMode = GPIO_READ_WRITE;
                }


                break;
            case FOURTH:
                offset = strtoul(pch, &p, 16);
                if(*p != 0)
                {
                    //printf("frequency:not a number\n");
                    //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
                    //return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_INITIAL_I2C_BUS | ERROR_I2C_DDAPI_PARAMETER_NOT_NUMBER), "frequency is not number", NULL);
                }
                break;
            case FIFTH:
                msb =  strtoul(pch, &p, 10);
                if(*p != 0)
                {
                    //printf("frequency:not a number\n");
                    //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
                    //return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_INITIAL_I2C_BUS | ERROR_I2C_DDAPI_PARAMETER_NOT_NUMBER), "frequency is not number", NULL);
                }
                break;
            case SIXTH:
                lsb =  strtoul(pch, &p, 10);
                if(*p != 0)
                {
                    //printf("frequency:not a number\n");
                    //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
                    //return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_INITIAL_I2C_BUS | ERROR_I2C_DDAPI_PARAMETER_NOT_NUMBER), "frequency is not number", NULL);
                }
                break;
            case SEVENTH:
                //tempStatus = pch;
                if(strcmp(pch, "ACTIVE_HI") == 0)
                    Status = TRUE;
                else
                    Status = FALSE;
                break;
            case EIGHTH:
                //tempWriteProtect = pch;
                if(strcmp(pch, "TRUE") == 0)
                    Status = TRUE;
                else
                    Status = FALSE;
                break;
            default:
                break;
        }
        pch = strtok (NULL, " \t");
    }
    AddEntry(DevName.c_str(), DataType, RWMode, offset, msb, lsb, Status, WriteProtect);
    //printf("%s %s %s %x %d %d %b %b\n", DevName.c_str(), DataType, RWMode, offset, msb, lsb, tempStatus.c_str(), tempWriteProtect.c_str());

    return NO_ERROR_MSG;
}





ErrorMsg* GPIO_FPGADriver :: AddEntry(STRING dataElementName,
                                     FPGA_DRIVER_DATA_TYPE dataElementType,
                                     GPIO_RW_MODE gpioRWMode,
                                     UINT32 offset,
                                     UINT8 msbLocation,
                                     UINT8 lsbLocation,
                                     BOOL bitState,
                                     BOOL disableWriteProtect)
{
    //printf("%s\n", __PRETTY_FUNCTION__);

    // Search the table for the first open entry in the Configuration table
    m_errorCode = NO_ERROR_MSG;

    if (m_ConfigurationTableIndex < MAX_FPGA_GPIO_CONFIG_ENTRIES)
    {
        strncpy(m_gpioConfiguration[m_ConfigurationTableIndex].dataElementName, dataElementName, FPGA_DATA_ELEMENT_NAME_LENGTH + 10);

        // debug message

        m_gpioConfiguration[m_ConfigurationTableIndex].dataElementType    = dataElementType;
        m_gpioConfiguration[m_ConfigurationTableIndex].gpioRWMode         = gpioRWMode;
        m_gpioConfiguration[m_ConfigurationTableIndex].offset             = offset;
        m_gpioConfiguration[m_ConfigurationTableIndex].msbLocation        = msbLocation;
        m_gpioConfiguration[m_ConfigurationTableIndex].lsbLocation        = lsbLocation;
        m_gpioConfiguration[m_ConfigurationTableIndex].bitState           = bitState;
        m_gpioConfiguration[m_ConfigurationTableIndex].disableWriteProtect= disableWriteProtect;

        m_ConfigurationTableIndex++;
    }
    else
    {
        //m_errorCode = ERROR_GPIO_DRIVER_TOO_MANY_ENTRIES;
        ;;
    }



    return NO_ERROR_MSG;
}





ErrorMsg* GPIO_FPGADriver :: getGPIOList(list<string> *pGPIOList)
{
    //printf("%s\n", __PRETTY_FUNCTION__);

    //ErrorMsg*   ErrorMsg = NO_ERROR_MSG;
    UINT32 index;

    //m_errorCode = ERROR_GPIO_DRIVER_UNKNOWN_NAME_REQUEST;

    for (index = 0; index < MAX_FPGA_GPIO_CONFIG_ENTRIES; index++)
    {
        if (m_gpioConfiguration[index].dataElementName[0] != (CHAR)NULL)
        {
            if (m_gpioConfiguration[index].dataElementType == GPIO_FPGA_DRIVER_DATA_TYPE_BOOLEAN)
            {
                //printf("!!%d : %s\n", index, m_gpioConfiguration[index].dataElementName);
                pGPIOList->push_back(m_gpioConfiguration[index].dataElementName);
            }
        }
    }

    if(pGPIOList->empty())
    {
        pGPIOList->push_back("Did Not Find Any GPIO PIN");
        //addErrorMsg(ERROR_GPIO_DDAPI_FPGA_GET_GPIO_LIST, NULL);
    }

    return NO_ERROR_MSG;
}





ErrorMsg*   GPIO_FPGADriver :: readBit(string gpioName, BOOL*   readValue)
{
    //printf("%s\n", __PRETTY_FUNCTION__);

    //ErrorMsg*   ErrorMsg = NO_ERROR_MSG;

    UINT32 Index = 0;

    UINT32      offset;
    UINT32      bitMask;
    UINT8       value;
    PHYSICAL_ADDRESS_T address;


    // display a debug message
//    sprintf(m_tempString, "ReadBit GPIO BIT: \"%s\"\n", gpioName);
//    displayDebugString(DEBUG_LEVEL_6, m_tempString);

    // locate the table index to the requested GPIO
    if ((Index = GetGPIOIndexByName(gpioName)) == GPIO_NOT_FOUND)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_DDAPI_GPIO_FPGA_READ_BIT | ERROR_GPIO_DDAPI_GET_GPIO_INDEX_FAIL),
                              NULL,
                              NULL);
    }

    if (m_gpioConfiguration[Index].gpioRWMode == GPIO_WRITE_ONLY)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_DDAPI_GPIO_FPGA_READ_BIT | ERROR_GPIO_DDAPI_GPIO_READ_ONLY),
                              NULL,
                              NULL);
    }

    if (m_gpioConfiguration[Index].gpioRWMode == GPIO_NOT_SUPPORT)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_DDAPI_GPIO_FPGA_READ_BIT | ERROR_GPIO_DDAPI_GPIO_NOT_SUPPORT),
                              NULL,
                              NULL);
    }

    if (m_gpioConfiguration[Index].dataElementType != GPIO_FPGA_DRIVER_DATA_TYPE_BOOLEAN)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_DDAPI_GPIO_FPGA_READ_BIT | ERROR_GPIO_DDAPI_GPIO_UNSUPPORT_TYPE),
                              NULL,
                              NULL);
    }

    // get the GPIO offset and bit number and bit len
    offset = m_gpioConfiguration[Index].offset;

    // read the GPIO port

    address = m_BaseAddressRegister + offset;



    value = m_MemoryDriver->read8(address);


    // create the requested port bit
    bitMask = 0x01 << m_gpioConfiguration[Index].lsbLocation;

    // display debug if requested
//        sprintf(m_tempString, "ReadBit read 0x%02x   bitMask 0x%02x\n", value, bitMask);
//        displayDebugString(DEBUG_LEVEL_4, m_tempString);

    // isolate the desired value
    value &= bitMask;

    // return information to the caller
    *readValue = FALSE;
    if (value != 0)
    {
         *readValue = TRUE;
    }

    // check for this bit as active HI or LO
    if (m_gpioConfiguration[Index].bitState == ACTIVE_LO)
    {
        // this bit is active LO so toggle the request
        if (*readValue == TRUE)
        {
            *readValue = FALSE;
        }
        else
        {
            *readValue = TRUE;
        }
    }

    return NO_ERROR_MSG;
}





ErrorMsg* GPIO_FPGADriver::setBit(string gpioName, BOOL Status)
{
    //printf("%s\n", __PRETTY_FUNCTION__);

    UINT32  Index;
    UINT32  offset;
    UINT32  bitMask;


    UINT8   value;

    //BOOL    disableWriteProtect = TRUE;


    PHYSICAL_ADDRESS_T address;

    //UINT32      retry;

    //retry = 5;

    // locate the table Index to the requested GPIO
    if ((Index = GetGPIOIndexByName(gpioName)) == GPIO_NOT_FOUND)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_DDAPI_GPIO_FPGA_SET_BIT | ERROR_GPIO_DDAPI_GET_GPIO_INDEX_FAIL),
                              NULL,
                              NULL);
    }

    if(m_gpioConfiguration[Index].gpioRWMode == GPIO_NOT_SUPPORT)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_DDAPI_GPIO_FPGA_SET_BIT | ERROR_GPIO_DDAPI_GPIO_NOT_SUPPORT),
                              NULL,
                              NULL);
    }
	if(m_gpioConfiguration[Index].gpioRWMode == GPIO_READ_ONLY)
	{
		return returnErrorMsg(ERRORCODE(ERROR_GPIO_DDAPI_GPIO_FPGA_SET_BIT | ERROR_GPIO_DDAPI_GPIO_READ_ONLY),
							  NULL,
							  NULL);
	}

	if (m_gpioConfiguration[Index].dataElementType != GPIO_FPGA_DRIVER_DATA_TYPE_BOOLEAN)
	{
		return returnErrorMsg(ERRORCODE(ERROR_GPIO_DDAPI_GPIO_FPGA_READ_BIT | ERROR_GPIO_DDAPI_GPIO_UNSUPPORT_TYPE),
							  NULL,
							  NULL);
	}

	offset = m_gpioConfiguration[Index].offset;
	//disableWriteProtect = m_gpioConfiguration[Index].disableWriteProtect;

	// read the GPIO port
	//m_errorCode = Read8(offset, &value);
	address = m_BaseAddressRegister + offset;
	value = m_MemoryDriver->read8(address);

	if (m_errorCode == NO_ERROR_MSG)
	{
		// create the requested bit mask
		bitMask = 0x01 << m_gpioConfiguration[Index].lsbLocation;

		// display debug if requested
//            sprintf(m_tempString, "UpdateBit read 0x%02x   bitMask 0x%02x   ", value, bitMask);
//            displayDebugString(DEBUG_LEVEL_4, m_tempString);

		// check for this bit as active HI or LO
		if (m_gpioConfiguration[Index].bitState == ACTIVE_LO)
		{
			// this bit is active LO so toggle the request
			if (Status == TRUE)
				Status = FALSE;
			else
				Status = TRUE;
		}

		// update the read value
		if (Status == TRUE)
		{
			value |= bitMask;
		}
		else
		{
			value &= (~bitMask);
		}
/*
		if (offset == 0x56)
		{
			Index = 0;

			if ((strncmp(m_midplaneName, "Camden", strlen("Camden")) == 0) ||
				(strncmp(m_midplaneName, "Ebbets", strlen("Ebbets")) == 0) ||
				(strncmp(m_midplaneName, "Wembley", strlen("Wembley")) == 0))
			{

				while(Index < retry)
				{
					if ((system("rmmod psu_delta")) == NO_ERROR)
						break;
					Index++;
				}

				if (m_errorCode == NO_ERROR)
				{
					// display debug if requested
//                        sprintf(m_tempString, "UpdateBit Wrote 0x%02x\n", value);
//                        displayDebugString(DEBUG_LEVEL_4, m_tempString, FALSE);
					// write the new output to the GPIO port
					//m_errorCode = Write(offset, value, disableWriteProtect);

					address = m_BaseAddressRegister + offset;
					m_errorCode = m_MemoryDriver->write(address, value);
				}

				system("insmod /lib/modules/psu-delta.ko");
			}
			else
			{
				while(Index < retry)
				{
					if ((m_errorCode = system("rmmod psu-delta-psmi")) == NO_ERROR)
						break;
					Index++;
				}

				if (m_errorCode == NO_ERROR)
				{
					// display debug if requested
//                        sprintf(m_tempString, "UpdateBit Wrote 0x%02x\n", value);
//                        displayDebugString(DEBUG_LEVEL_4, m_tempString, FALSE);
					// write the new output to the GPIO port
					//m_errorCode = Write(offset, value, disableWriteProtect);

					address = m_BaseAddressRegister + offset;
					m_errorCode = m_MemoryDriver->write(address, value);
				}

				system("insmod /lib/modules/psu-delta-psmi.ko");
			}
		}
		else
		{*/
			// display debug if requested
//                sprintf(m_tempString, "UpdateBit Wrote 0x%02x\n", value);
//                displayDebugString(DEBUG_LEVEL_4, m_tempString, FALSE);
			// write the new output to the GPIO port

			//m_errorCode = Write(offset, value, disableWriteProtect);

			address = m_BaseAddressRegister + offset;
			m_errorCode = m_MemoryDriver->write(address, value);
	/*
		}
	*/
	}


//    return m_errorCode;

    return m_errorCode;
}









ErrorMsg*   GPIO_FPGADriver::isMyGPIO(string   gpioName)
{
    //printf("%s\n", __PRETTY_FUNCTION__);

    ErrorMsg*   ErrorMsg = NO_ERROR_MSG;


    if (GetGPIOIndexByName(gpioName) == GPIO_NOT_FOUND)
    {
        return returnErrorMsg(ERROR_GPIO_DDAPI_GPIO_FPGA_IS_MY_GPIO, NULL, NULL);
    }


    return ErrorMsg;
}





UINT32 GPIO_FPGADriver::GetGPIOIndexByName(string gpioName)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    INT32   index;

    CHAR tempName[(GPIO_DEVICE_NAME_LENGTH + 1)];

    // we need upper-case names, but don't change the callers input string
    strcpy(tempName, gpioName.c_str());
    ConvertStringToUpperCase(tempName);

    index = 0;
    while (m_gpioConfiguration[index].dataElementName[0] != (CHAR)NULL)
    {
    	//printf("m_gpioConfiguration[index].dataElementName=%s\n", m_gpioConfiguration[index].dataElementName);
    	//printf("tempName=%s\n", tempName);

        if (strcmp(m_gpioConfiguration[index].dataElementName, tempName) == 0)
            return index;

        index++;
    }

    return GPIO_NOT_FOUND;
}









