/*
 * GPIOGeneric.cpp
 *
 *  Created on: Dec 2, 2015
 *      Author: ianfang
 */
#include "GPIO_PCHDriver.hpp"



GPIO_PCHDriver :: GPIO_PCHDriver (STRING  ClassName): GPIOGeneric(ClassName)
{
	//printf("!~!~!~!~!~!~\n");

    //PCIDDAPI      *m_PCIDriver;
    //MemoryDDAPI   *m_MemoryDriver;
	PCIClassCode PCH_CLASS_ID = ISA_BRIDGE;
	vector<PCIInformantion> PCH_information;
	PFA PCH_DFB;
    m_BaseAddressRegister       = 0;
    m_ConfigurationTableIndex   = 0;

    //New a PCI driver
    m_PCIDriver = new PCIDDAPI("GPIO_PCH_PCI");

    m_errorCode = m_PCIDriver->getErrorMsg();
    if (m_errorCode != NO_ERROR_MSG)
    {
        delete m_PCIDriver;
        addErrorMsg(ERROR_GPIO_PCH_DDAPI_DRIVER_CONSTRUCTOR, NULL, m_errorCode);
        return;
    }

    m_PCIDriver->getPciInfoByClass(PCH_CLASS_ID, &PCH_information);
    std::vector<PCIInformantion> tmp = PCH_information;
	PCH_DFB.u16Domain=PCH_information[0].u16Domain;
	PCH_DFB.u16Bus=PCH_information[0].u16Bus;
	PCH_DFB.u16Device=PCH_information[0].u16Device;
	PCH_DFB.u8Function=PCH_information[0].u8Function;


	m_BaseAddressRegister = m_PCIDriver->ConfigReadDword(PCH_DFB, 0x48);
	//printf("Bass Address : %x\n", m_BaseAddressRegister);
	//m_BaseAddressRegister = m_BaseAddressRegister & 0xffc0; //for ICH, access bit15-bit6,reference related document
	m_BaseAddressRegister = m_BaseAddressRegister & 0xff80;  //for PCH , access bit15-bit7,reference related document
	//printf("Bass Address : %x\n", m_BaseAddressRegister);


    //New a memory driver

    m_MemoryDriver = new MemoryDDAPI("GPIO_MEM");

    m_errorCode = m_MemoryDriver->getErrorMsg();
    if (m_errorCode != NO_ERROR_MSG)
    {
        delete m_MemoryDriver;
        delete m_PCIDriver;
        addErrorMsg(ERROR_GPIO_PCH_DDAPI_DRIVER_CONSTRUCTOR, NULL, m_errorCode);
        return;

    }

    //Get the PCH bar address
    //m_PCIDriver->getBarByBDF(0, 0x72, 0x00, 0x00, 0, &m_BaseAddressRegister);
    //printf("Bar[0] -- 0x%x\n", m_BaseAddressRegister);

}


GPIO_PCHDriver :: ~GPIO_PCHDriver ()
{
    if(m_PCIDriver != NULL)
        delete m_PCIDriver;

    //if(m_MemoryDriver != NULL)
    //    delete m_MemoryDriver;
}


ErrorMsg* GPIO_PCHDriver :: readINIFile(CHAR *Line)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    char   *pch;
    char   *p;
    COLUMN_NUMBER eCurrentType;
    UINT32  counter = 0;
    UINT32  offset = 0;//, msb = 0, lsb = 0;
    BOOL    Status = TRUE, WriteProtect = TRUE;
    string DevName, tempStatus, tempWriteProtect;
    PCH_DRIVER_DATA_TYPE DataType;
    GPIO_RW_MODE    RWMode;
    DevName.clear();
    tempStatus.clear();
    tempWriteProtect.clear();



    pch = strtok(Line, " \t");
    //printf("pch=%s\n", pch);
    while (pch != NULL)
    {
        counter++;
        //printf("%s\n", pch);
        eCurrentType = COLUMN_NUMBER(counter);
        switch(eCurrentType)
        {

            case FIRST:
                DevName = pch;
                break;
            case SECOND:
                //DataType = pch;
                if(strcmp(pch, "GPIO_PCH_DRIVER_DATA_TYPE_BOOLEAN") == 0)
                {
                    DataType = GPIO_PCH_DRIVER_DATA_TYPE_BOOLEAN;
                }
                if(strcmp(pch, "GPIO_PCH_DRIVER_DATA_TYPE_UINT8") == 0)
                {
                    DataType = GPIO_PCH_DRIVER_DATA_TYPE_UINT8;
                }
                if(strcmp(pch, "GPIO_PCH_DRIVER_DATA_TYPE_UINT16") == 0)
                {
                    DataType = GPIO_PCH_DRIVER_DATA_TYPE_UINT16;
                }
                if(strcmp(pch, "GPIO_PCH_DRIVER_DATA_TYPE_UINT32") == 0)
                {
                    DataType = GPIO_PCH_DRIVER_DATA_TYPE_UINT32;
                }

                break;
            case THIRD:
                if(strcmp(pch, "GPIO_READ_ONLY") == 0)
                {
                    RWMode = GPIO_READ_ONLY;
                }
                if(strcmp(pch, "GPIO_WRITE_ONLY") == 0)
                {
                    RWMode = GPIO_WRITE_ONLY;
                }
                if(strcmp(pch, "GPIO_WRITE_CLEAR") == 0)
                {
                    RWMode = GPIO_WRITE_CLEAR;
                }
                if(strcmp(pch, "GPIO_READ_WRITE") == 0)
                {
                    RWMode = GPIO_READ_WRITE;
                }


                break;
            case FOURTH:
                offset = strtoul(pch, &p, 10);
                if(*p != 0)
                {
                    //printf("frequency:not a number\n");
                    //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
                    //return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_INITIAL_I2C_BUS | ERROR_I2C_DDAPI_PARAMETER_NOT_NUMBER), "frequency is not number", NULL);
                }
                break;

            case FIFTH:
                //tempStatus = pch;
                if(strcmp(pch, "ACTIVE_HI") == 0)
                    Status = TRUE;
                else
                    Status = FALSE;
                break;
            case SIXTH:
                //tempWriteProtect = pch;
                if(strcmp(pch, "TRUE") == 0)
                    Status = TRUE;
                else
                    Status = FALSE;
                break;
            default:
                break;
        }
        pch = strtok (NULL, " \t");
    }
    AddEntry(DevName.c_str(), DataType, RWMode, offset, Status, WriteProtect);
    //printf("%s %s %s %x %d %d %b %b\n", DevName.c_str(), DataType, RWMode, offset, msb, lsb, tempStatus.c_str(), tempWriteProtect.c_str());

    return NO_ERROR_MSG;
}





ErrorMsg* GPIO_PCHDriver :: AddEntry(STRING dataElementName,
                                     PCH_DRIVER_DATA_TYPE dataElementType,
                                     GPIO_RW_MODE gpioRWMode,
                                     UINT32 offset,
                                     BOOL bitState,
                                     BOOL disableWriteProtect)
{
    //printf("%s\n", __PRETTY_FUNCTION__);

    // Search the table for the first open entry in the Configuration table
    m_errorCode = NO_ERROR_MSG;

    if (m_ConfigurationTableIndex < MAX_PCH_GPIO_CONFIG_ENTRIES)
    {
        strncpy(m_gpioConfiguration[m_ConfigurationTableIndex].dataElementName, dataElementName, PCH_DATA_ELEMENT_NAME_LENGTH + 10);

        // debug message
        //printf ("dataElementType=%s\n", dataElementName);
        m_gpioConfiguration[m_ConfigurationTableIndex].dataElementType    = dataElementType;
        m_gpioConfiguration[m_ConfigurationTableIndex].gpioRWMode         = gpioRWMode;
        m_gpioConfiguration[m_ConfigurationTableIndex].offset             = offset;
        m_gpioConfiguration[m_ConfigurationTableIndex].bitState           = bitState;
        m_gpioConfiguration[m_ConfigurationTableIndex].disableWriteProtect= disableWriteProtect;

        m_ConfigurationTableIndex++;
    }

    return NO_ERROR_MSG;
}

ErrorMsg* GPIO_PCHDriver :: getGPIOList(list<string> *pGPIOList)
{
    //printf("%s\n", __PRETTY_FUNCTION__);

    //ErrorMsg*   ErrorMsg = NO_ERROR_MSG;
    UINT32 index;
    string tempString;
    UINT16	WellKnownNameLength = 0;
    UINT16	HyphenLength = 0;
    tempString.clear();
    char temp[256];

    //m_errorCode = ERROR_GPIO_DRIVER_UNKNOWN_NAME_REQUEST;

    for (index = 0; index < MAX_PCH_GPIO_CONFIG_ENTRIES; index++)
    {
        if (m_gpioConfiguration[index].dataElementName[0] != (CHAR)NULL)
        {
            if (m_gpioConfiguration[index].dataElementType == GPIO_PCH_DRIVER_DATA_TYPE_BOOLEAN)
            {
            	//printf("!!%d : %s\n", index, m_gpioConfiguration[index].dataElementName);
            	//temp = m_gpioConfiguration[index].dataElementName;
                //pGPIOList->push_back(m_gpioConfiguration[index].dataElementName);
            	tempString=m_gpioConfiguration[index].dataElementName;
            	WellKnownNameLength = tempString.length();
            	//
            	HyphenLength = GPIO_LIST_LENGTH - GPIO_SPACE_LENGTH*2 - WellKnownNameLength - 2 - 2;

            	tempString.append("  <");
            	tempString.append(HyphenLength,'-');
            	tempString.append(">  ");

            	//sprintf(temp, "%s  <------->  %d", m_gpioConfiguration[index].dataElementName,m_gpioConfiguration[index].offset);
            	sprintf(temp, "%s%d", tempString.c_str(),m_gpioConfiguration[index].offset);
            	pGPIOList->push_back(temp);
            }
        }
    }

    if(pGPIOList->empty())
    {
        //pGPIOList->push_back("Did Not Find Any GPIO PIN");
        return returnErrorMsg(ERROR_GPIO_PCH_DDAPI_GET_GPIO_LIST, "Did Not Find Any PCH GPIO PIN", NULL);
        //addErrorMsg(ERROR_GPIO_DDAPI_PCH_GET_GPIO_LIST, NULL);
    }

    return NO_ERROR_MSG;
}





ErrorMsg*   GPIO_PCHDriver :: readBit(string gpioName, BOOL*  readValue)
{
    //printf("%s\n", __PRETTY_FUNCTION__);

    //ErrorMsg*   ErrorMsg = NO_ERROR_MSG;

    UINT32 Index = 0;
    UINT32 temp_1 = 0;

    UINT32      offset;
    //UINT32      bitMask;
    UINT32       value;
    PHYSICAL_ADDRESS_T address;




    // display a debug message
//    sprintf(m_tempString, "ReadBit GPIO BIT: \"%s\"\n", gpioName);
//    displayDebugString(DEBUG_LEVEL_6, m_tempString);

    // locate the table index to the requested GPIO

    if ((Index = GetGPIOIndexByName(gpioName)) == PCH_NOT_FOUND)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_READ_BIT | ERROR_GPIO_DDAPI_GET_GPIO_INDEX_FAIL),
                              NULL,
                              NULL);
    }

    if (m_gpioConfiguration[Index].gpioRWMode == GPIO_WRITE_ONLY)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_READ_BIT | ERROR_GPIO_DDAPI_GPIO_READ_ONLY),
                              NULL,
                              NULL);
    }

    if (m_gpioConfiguration[Index].gpioRWMode == GPIO_NOT_SUPPORT)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_READ_BIT | ERROR_GPIO_DDAPI_GPIO_NOT_SUPPORT),
                              NULL,
                              NULL);
    }

    if (m_gpioConfiguration[Index].dataElementType != GPIO_PCH_DRIVER_DATA_TYPE_BOOLEAN)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_READ_BIT | ERROR_GPIO_DDAPI_GPIO_UNSUPPORT_TYPE),
                              NULL,
                              NULL);
    }

    // get the GPIO offset and bit number and bit len
    offset = m_gpioConfiguration[Index].offset;

    if (offset <= 31)
	{
		//printf("offset=%d\n", offset);
		//foxcli_read_port(base_offsetess + 0xc, &temp, 4);
		address = m_BaseAddressRegister + 0xc;
		//printf("address=%x\n", address);
		readIOPort(address, &value, 4);
		//value = inl(address);
		//printf("Value=%x\n", value);
		temp_1=value;
		temp_1=temp_1 & (1 << offset);
		temp_1=temp_1 >> offset;
		//printf("%x ", temp_1);
		}

	else if (offset <= 63 && offset >=32)
	{
		//printf("offset=%d\n", offset);
		//foxcli_read_port(base_offsetess + 0x38, &temp, 4);
		address = m_BaseAddressRegister + 0x38;
		readIOPort(address, &value, 4);
		//value = inl(address);
		temp_1 = value;
		temp_1 = temp_1 & (1 << (offset - 32));
		temp_1 = temp_1 >> (offset - 32);
		//printf("%x ", temp_1);
		}

	else if (offset <= 95 && offset >=64)
	{
		//printf("offset=%d\n", offset);
		//foxcli_read_port(base_offsetess + 0x48, &temp, 4);
		address = m_BaseAddressRegister + 0x48;
		readIOPort(address, &value, 4);
		//value = inl(address);
		temp_1 = value;
		temp_1 = temp_1 & (1 << (offset - 64));
		temp_1 = temp_1 >> (offset - 64);
		//printf("%x ", temp_1);
		}
	else
	{
		//printf("[%s:%d] Other error : offset is bigger than 95!\n", __FILE__, __LINE__);
		return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_READ_BIT | ERROR_GPIO_DDAPI_GPIO_UNSUPPORT_TYPE),
							  NULL,
							  NULL);
	}

    if(temp_1 == 0)
    {
    	*readValue = FALSE;
    }
    else
    {
    	*readValue = TRUE;
    }
    return NO_ERROR_MSG;
}

ErrorMsg* GPIO_PCHDriver::writeBit(string gpioName, BOOL Status)
{
    //printf("%s\n", __PRETTY_FUNCTION__);

    UINT32  Index;
    UINT32  offset;
    UINT32 temp = 0;
    UINT32 temp_1 = 0;

    //ErrorMsg*   ErrorMsg = NO_ERROR_MSG;
    //UINT32  bitMask;


    UINT8   value;
    //BOOL    disableWriteProtect = TRUE;
    PHYSICAL_ADDRESS_T address;

    if(Status)
    {
    	value = 1;
    }
    else
    {
    	value = 0;
    }

    // locate the table Index to the requested GPIO
    if ((Index = GetGPIOIndexByName(gpioName)) == PCH_NOT_FOUND)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_SET_BIT | ERROR_GPIO_DDAPI_GET_GPIO_INDEX_FAIL),
                              NULL,
                              NULL);
    }
    if(m_gpioConfiguration[Index].gpioRWMode == GPIO_NOT_SUPPORT)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_SET_BIT | ERROR_GPIO_DDAPI_GPIO_NOT_SUPPORT),
                              NULL,
                              NULL);
    }

    if(m_gpioConfiguration[Index].gpioRWMode == GPIO_READ_ONLY)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_DDAPI_GPIO_FPGA_SET_BIT | ERROR_GPIO_DDAPI_GPIO_READ_ONLY),
                              NULL,
                              NULL);
    }

    if (m_gpioConfiguration[Index].dataElementType != GPIO_PCH_DRIVER_DATA_TYPE_BOOLEAN)
    {
        return returnErrorMsg(ERRORCODE(ERROR_GPIO_DDAPI_GPIO_FPGA_READ_BIT | ERROR_GPIO_DDAPI_GPIO_UNSUPPORT_TYPE),
                              NULL,
                              NULL);
    }

    offset = m_gpioConfiguration[Index].offset;

    if (offset <= 31)
    {
    	address = m_BaseAddressRegister + 0xc;
		if (value == 0)
		{
			readIOPort(address, &temp, 4);     //GPIO_LVL_REG 1
			//printf("bef temp=%x\n", temp);
			temp = temp & ((1 << offset) ^ 0xFFFFFFFF);
			//printf("aft temp=%x\n", temp);
			writeIOPort(address, temp, 4);
			readIOPort(address, &temp_1, 4);
			temp_1 = temp_1 & (1 << offset);
			temp_1 = temp_1 >> offset;
			if (temp_1 != 0)
			{
				return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_SET_BIT | ERROR_GPIO_DDAPI_GPIO_SET_FAIL),
						                              NULL,
						                              NULL);
			}
			else
			{
				return NO_ERROR_MSG;
			}
		}
		if (value == 1)
		{
			readIOPort(address, &temp, 4);     //GPIO_LVL_REG 1
			temp = temp | (1 << offset);
			writeIOPort(address, temp, 4);
			readIOPort(address, &temp_1, 4);
			temp_1 = temp_1 & (1 << offset);
			temp_1 = temp_1 >> offset;
			if (temp_1 == 0)
			{
		        return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_SET_BIT | ERROR_GPIO_DDAPI_GPIO_SET_FAIL),
		                              NULL,
		                              NULL);
			}
			else
			{
				return NO_ERROR_MSG;
			}
		}
	}
    else if (offset <= 63 && offset >= 32)
    {
    	address = m_BaseAddressRegister + 0x38;
		if (value == 0)
		{

			readIOPort(address, &temp, 4);    //GPIO_LVL_REG 2
			temp = temp & ((1 << (offset - 32)) ^ 0xFFFFFFFF);
			writeIOPort(address, temp, 4);
			readIOPort(address, &temp_1, 4);
			temp_1 = temp_1 & (1 << (offset - 32));
			temp_1 = temp_1 >> (offset - 32);
			if (temp_1 != 0)
			{
				return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_SET_BIT | ERROR_GPIO_DDAPI_GPIO_SET_FAIL),
						                              NULL,
						                              NULL);
			}
			else
			{
				return NO_ERROR_MSG;
			}
		}
		if (value == 1)
		{
			readIOPort(address, &temp, 4);    //GPIO_LVL_REG 2
			temp = temp | (1 << (offset - 32));
			writeIOPort(address, temp, 4);
			readIOPort(address, &temp_1, 4);
			temp_1 = temp_1 & (1 << (offset - 32));
			temp_1 = temp_1 >> (offset - 32);
			if (temp_1 == 0)
			{
				return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_SET_BIT | ERROR_GPIO_DDAPI_GPIO_SET_FAIL),
										                              NULL,
										                              NULL);
			}
			else
			{
				return NO_ERROR_MSG;
			}
		}
	}
    else if (offset >= 64 && offset <= 95)
    {
    	address = m_BaseAddressRegister + 0x48;
		if (value == 0)
		{
			readIOPort(address, &temp, 4);    //GPIO_LVL_REG 3
			temp = temp & ((1 << (offset - 64)) ^ 0xFFFFFFFF);
			writeIOPort(address, temp, 4);
			readIOPort(address, &temp_1, 4);
			temp_1 = temp_1 & (1 << (offset - 64));
			temp_1 = temp_1 >> (offset - 64);
			if (temp_1 != 0)
			{
				return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_SET_BIT | ERROR_GPIO_DDAPI_GPIO_SET_FAIL),
										                              NULL,
										                              NULL);
			}
			else
			{
				return NO_ERROR_MSG;
			}
		}
		else if (value == 1)
		{
			readIOPort(address, &temp, 4);    //GPIO_LVL_REG 3
			temp = temp | (1 << (offset - 64));
			writeIOPort(address, temp, 4);
			readIOPort(address, &temp_1, 4);
			temp_1 = temp_1 & (1 << (offset - 64));
			temp_1 = temp_1 >> (offset - 64);
			if (temp_1 == 0)
			{
				return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_SET_BIT | ERROR_GPIO_DDAPI_GPIO_SET_FAIL),
														                              NULL,
														                              NULL);
			}
			else
			{
				return NO_ERROR_MSG;
			}
		}
		else
		{
			return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_SET_BIT | ERROR_GPIO_DDAPI_GPIO_NOT_SUPPORT),
																	                              NULL,
																	                              NULL);
		}
	}
    else
    {
    	return returnErrorMsg(ERRORCODE(ERROR_GPIO_PCH_DDAPI_SET_BIT | ERROR_GPIO_DDAPI_GPIO_NOT_SUPPORT),
    															                              NULL,
    															                              NULL);
	}
	return NO_ERROR_MSG;
}

INT16  GPIO_PCHDriver::readIOPort(UINT32 Port, UINT32 *ReturnData, UINT32 ByteSize)
{
    int err = 0, privilege = 0;
    unsigned int numperm = 1;

    if (ReturnData == NULL) {
        fprintf (stderr,
                 "[%s +%d]: Port address %d error, or other parameter error !\n",
                 __FILE__, __LINE__, Port);
        return -1;
    }
    if (Port <= BASE_PORT) {
        err = ioperm ((unsigned long) Port, numperm, IO_ENABLE);
        privilege = LOW_IO_PRIVILEGE;
    } else {
        err = iopl (HIGH_IO_ENABLE);
        privilege = HIGH_IO_PRIVILEGE;
    }

    if (err < 0) {
        fprintf (stderr, "[%s +%d]: Can't set the io port permission for reading !\n",
                 __FILE__, __LINE__);
        return -1;
    }

    switch (ByteSize) {
        case ReadWrite_BYTE:
            *ReturnData = inb (Port);
            break;
        case ReadWrite_WORD:
            *ReturnData = inw (Port);
            break;
        case ReadWrite_DWORD:
            *ReturnData = inl (Port);
            break;
        default:
            fprintf (stderr,
                     "[%s +%d]: Not define such size(%c) for write the register!\n",
					 __FILE__, __LINE__, ByteSize);
					             return -1;
					     }

	 if (privilege == LOW_IO_PRIVILEGE) {
		 ioperm ((unsigned int) Port, numperm, IO_DISABLE);
	 } else if (privilege == HIGH_IO_PRIVILEGE) {
		 iopl (IO_DISABLE);
	 }
	 delay (RW_DELAY);

	 //return err;


	return 0;
}
INT16   GPIO_PCHDriver::writeIOPort(UINT32 Port, UINT32 DataInfo, UINT32 ByteSize)
{
	int err = 0, privilege = 0;
	unsigned int numperm = 1;

	if (Port <= BASE_PORT)
	{
		err = ioperm((unsigned long) Port, numperm, IO_ENABLE);
		privilege = LOW_IO_PRIVILEGE;
	} else
	{
		err = iopl(HIGH_IO_ENABLE);
		privilege = HIGH_IO_PRIVILEGE;
	}

	if (err < 0)
	{
		fprintf(stderr,
				"[%s +%d]: Can't set the io port permission for reading !\n",
				__FILE__, __LINE__);
		return -1;
	}

	switch (ByteSize)
	{
		case ReadWrite_BYTE:
			outb(DataInfo, Port);
			break;
		case ReadWrite_WORD:
			outw(DataInfo, Port);
			break;
		case ReadWrite_DWORD:
			outl(DataInfo, Port);
			break;
		default:
			fprintf(stderr,
					"[%s +%d]: Not define such size(%c) for write the register!\n",
					__FILE__, __LINE__, ByteSize);
			return -1;
	}

	if (privilege == LOW_IO_PRIVILEGE)
	{
		ioperm((unsigned int) Port, numperm, IO_DISABLE);
	}
	else if (privilege == HIGH_IO_PRIVILEGE)
	{
		iopl(IO_DISABLE);
	}
	delay(RW_DELAY);

	return err;
}
void GPIO_PCHDriver::delay(int us)
{
    struct timeval tv;

    tv.tv_sec = 0;
    tv.tv_usec = us;
    (void) select (1, NULL, NULL, NULL, &tv);
}

ErrorMsg*   GPIO_PCHDriver::isMyGPIO(string   gpioName)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    if (GetGPIOIndexByName(gpioName) == PCH_NOT_FOUND)
    {
        return returnErrorMsg(ERROR_GPIO_PCH_DDAPI_IS_MY_GPIO, NULL, NULL);
    }
    return NO_ERROR_MSG;
}

UINT32 GPIO_PCHDriver::GetGPIOIndexByName(string gpioName)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    INT32   index;

    CHAR tempName[(GPIO_DEVICE_NAME_LENGTH + 1)];

    // we need upper-case names, but don't change the callers input string
    strcpy(tempName, gpioName.c_str());
    ConvertStringToUpperCase(tempName);

    index = 0;
    while (m_gpioConfiguration[index].dataElementName[0] != (CHAR)NULL)
    {
    	//printf("m_gpioConfiguration[index].dataElementName=%s\n", m_gpioConfiguration[index].dataElementName);
    	//printf("tempName=%s\n", tempName);

        if (strcmp(m_gpioConfiguration[index].dataElementName, tempName) == 0)
            return index;

        index++;
    }

    return PCH_NOT_FOUND;
}









