/// @file       MemoryRange.hpp
/// @brief      MemoryRange class
/// @details    Provide class for a memory range
/// @author     KenHu
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#ifndef MEMORY_RANGE_HPP
#define MEMORY_RANGE_HPP

#include <string>
#include "common.hpp"

/// @class      MemoryRange MemoryRange.hpp "MemoryRange.hpp"
/// @brief      Provide information for a memory range
/// @param      m_start is a physical address for start address of a memory range
/// @param      m_end is a physical address for end address of a memory range
class MemoryRange
{
    public:
                            MemoryRange() 
                            {
                                m_startAddress = 0;
                                m_endAddress = 0;
                            };
        virtual             ~MemoryRange() {};

        /// @fn         MemoryRange::getSize()
        /// @brief      Get size of the memory range
        /// @exception  Nothing
        /// @return     A PHYSICAL_ADDRESS_T for memory size
        PHYSICAL_ADDRESS_T  getSize()
                            {
                                if (m_endAddress > m_startAddress)
                                    return m_endAddress - m_startAddress + 1;
                                else
                                    return (PHYSICAL_ADDRESS_T)0;
                            }
        
        /// @fn         MemoryRange::isInterleave()
        /// @brief      Check if this range is interleaving
        /// @exception  Nothing
        /// @return     A BOOL value, TRUE:interleaving FALSE:normal
        BOOL                isInterleave()
                            {
                                list< list<INT32> >::iterator itList;

                                if (m_controllerList.size() > 1)
                                    return TRUE;

                                itList = m_channelList.begin();
                                while (itList != m_channelList.end())
                                {
                                    if (itList->size() > 1)
                                        return TRUE;
                                    else
                                        itList++;
                                }

                                itList = m_rankList.begin();
                                while (itList != m_rankList.end())
                                {
                                    if (itList->size() > 1)
                                        return TRUE;
                                    else
                                        itList++;
                                }

                                return FALSE;
                            }

        string              m_name;
        PHYSICAL_ADDRESS_T  m_startAddress;
        PHYSICAL_ADDRESS_T  m_endAddress;
        list<INT32>         m_controllerList;
        list< list<INT32> > m_channelList;
        list< list<INT32> > m_rankList;
    protected:
    private:
};

#endif
