/// @file       NVMeDDAPI.hpp
/// @brief      NVMeDDAPI class
/// @details    The header file of NVMe DDAPI
/// @version    1.0.0
/// @date       2014.01.06
/// @author     Jerry Lee
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.
#ifndef NVMeDDAPI_H
#define NVMeDDAPI_H

#include <fcntl.h>
#include <sys/stat.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <stdarg.h>
#include <unistd.h>
#include <strings.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <limits.h>
#include <dirent.h>
#include <vector>
#include <linux/stddef.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <dirent.h>
#include <scsi/sg.h>

#include "BaseClassDDAPI.hpp"
#include "NVMeIOCTL.hpp"
#include "PCIDDAPI.hpp"
using std::vector;

#define NSSRC_VALUE		0x4E564D65

// NVMe information definitions
typedef struct NVMe_DEVICE_INFO
{
    string wellKnowName;
    string nativeName;
    string chardeviceNodeName;
    string blockdeviceNodeName;
    string domindBusDeviceFunction;
    UINT32 barAddr;
	float  pciSpeed;
	UINT32 pciWidth;
    struct nvme_bar *barInfo;
    struct nvme_error_log_page errLog;
    struct nvme_firmware_log_page fwLog;
    struct nvme_id_ctrl idCtrl;
    struct nvme_id_ns idNs;
    struct nvme_smart_log smartLog;
} NVMeInfo;

typedef struct nvmeDriveMap
{
    string rootBDF;
    unsigned int numOfHardDrive;
}NVMeDriveMap;

class NVMeDDAPI: public BaseClassDDAPI
{
    public:
        NVMeDDAPI(STRING className);
        virtual ~NVMeDDAPI(void);

        ErrorMsg* doIOCTLCommand(UINT8 controllerID, NVMeIOCTL::IOCTL_OP_CODE ioctlCommand, void* returnedValue);
        ErrorMsg* getControllerList(vector<NVMe_DEVICE_INFO>* returnedList);
        ErrorMsg* downloadFirmware(UINT8 controllerID, void* buffer, UINT32 size);
		ErrorMsg* resetSubSystem(UINT8 controllerID);


    protected:
	
    private:
        static vector<NVMe_DEVICE_INFO> nvmeList;

        ErrorMsg* addNVMeController(void);
        ErrorMsg* setDeviceNodeByController(UINT8 controllerID);
        static bool sortNVMEcontroller(const NVMe_DEVICE_INFO &left, const NVMe_DEVICE_INFO &right);
        string getNVMeAttachedRootPort(string NVMeBDF);
};


#endif // NVMeDDAPI_H

