#ifndef PCI_CLASS_CODE_HPP
#define PCI_CLASS_CODE_HPP

#include "common.hpp"

#define MAX_TEXT_LENGTH     64
#define ENCODE(BaseClassCode,SubClass) ( (BaseClassCode << 8) | (0xFF & SubClass) )


/*
class PCIDeviceInformation
{
    public:
        UINT16  u16PCIIndex;

        //PCI/PCIE base information
        UINT16  u16Domain;
        UINT16  u16Bus;
        UINT16  u16Device;
        UINT8   u8Function;
        UINT32  u32ClassID; //BaseClass + SubClass
        UINT32  u32VendorID;
        UINT32  u32DeviceID;
        UINT32  u32Revision;
        CHAR    szDeviceName[256];
        
        //Capability information
        UINT8   u8CurrentLinkSpeed;
        UINT8   u8CurrentLinkWidth;
        UINT8   u8MaxLinkSpeed;
        UINT8   u8MaxLinkWidth;
};
*/

/*
class PCICommonClass
{
    public:
        PCIDeviceInformation    *m_Device;
    private:
};

*/

enum PCIBaseClass
{
    DEVICE_NO_CLASS_CODE = 0x00,
    MASS_STORAGE,
    NETWORK_CONTROLLER,
    DISPLAY_CONTROLLER,
    MULTIMEDIA,
    MEMORY_CONTROLLER,
    BRIDGE,
    SIMPLE_COMMUNICATION_CONTROLLER,
    BASE_SYSTEM_PERIPHERAL,
    INPUT,
    DOCKING,
    PROCESSOR,
    SERIAL_BUS_CONTROLLER,
    WIRELESS_CONTROLLER,
    INTELLIGENT_IO_CONTROLLER,
    SATELLITE_COMMUNICATION_CONTROLLER,
    ENCRYPTION_DECRYPTION_CONTROLLER,
    DASPC, //Data acquisition and signal processing controllers
    PROCESSING_ACCELERATOR,
    NON_ESSENTIAL_INSTRUMENTATION,
};

typedef enum
{
    DEVICE_NO_CLASS = 0,

    //Base class = 00h 
    NVGA = ENCODE(DEVICE_NO_CLASS_CODE,0x0),
    VGA = ENCODE(DEVICE_NO_CLASS_CODE,0x1), //VGA device

    //Base class = 01h, mass storage controllers
    PCI_SCSI = ENCODE(MASS_STORAGE,0x0),
    IDE,
    FD, 
    IPI, 
    RAID, 
    ATA,
    PCI_SATA,
    PCI_SAS,
    PCI_NVME, //SSS, 
    UFS, 
    OTHER_01 = ENCODE(MASS_STORAGE, 0x80),

    //Base class = 02h, network controllers
    PCI_ETH = ENCODE(NETWORK_CONTROLLER,0x0),
    TR, 
    FDDI,
    ATM,
    ISDN,
    WF, 
    PICMG,
    PCI_IB, 
    OTHER_02 = ENCODE(NETWORK_CONTROLLER,0x80),

    //Base class = 03h, display controllers
    VGAC = ENCODE(DISPLAY_CONTROLLER, 0x0), //VGA controller
    XGA,
    C3D,
    OTHER_03,
    
    //Base class = 04h, multimedia devices
    VIDEO = ENCODE(MULTIMEDIA,0x0),
    AUDIO,
    CT, 
    MMD, 
    OTHER_04 = ENCODE(MULTIMEDIA,0x80),

    //Base class = 05h, memory controllers
    RAM = ENCODE(MEMORY_CONTROLLER,0x0),
    FLASH_CLASS,
    OTHER_05 = ENCODE(MEMORY_CONTROLLER,0x80),

    //Base class = 06h, bridge devices
    HOST_BRIDGE = ENCODE(BRIDGE,0x0),
    ISA_BRIDGE,
    EISA_BRIDGE,
    MCA_BRIDGE,
    P2P_BRIDGE, 
    PCMCIA_BRIDGE,
    NUBUS_BRIDGE,
    CARD_BUS_BRIDGE,
    RACEWAY_BRIDGE,
    ST_P2P_BRIDGE, 
    I2P_HOST_BRIDGE,
    A2P_HOST_BRIDGE, 
    OTHER_06 = ENCODE(BRIDGE,0x80),

    //Base class = 07h, simple communications controllers
    
    //Base class = 08h, generic system peripherals
    
    PICC = ENCODE(BASE_SYSTEM_PERIPHERAL, 0x0),
    DMA,
    Timer,
    RTC,
    PCI_HB, //Generic PCI Hot-Plug controller
    SD_HOST,
    IOMMU,
    RCE, //Root Complex Event controller
    OTHER_08 = ENCODE(BASE_SYSTEM_PERIPHERAL, 0x80),
    

    //Base class = 09h, input devices
    //Base class = 0Ah, docking stations

    //Base class = 0Bh, processors
    PROCESSOR_386 = ENCODE(PROCESSOR,0x0),
    PROCESSOR_486,
    PROCESSOR_PENTIUM,
    PROCESSOR_ALPHA = ENCODE(PROCESSOR,0x10),
    PROCESSOR_PPC = ENCODE(PROCESSOR,0x20),
    PROCESSOR_MIPS = ENCODE(PROCESSOR,0x30),
    PROCESSOR_CP = ENCODE(PROCESSOR,0x40), 
    OTHER_0B = ENCODE(PROCESSOR,0x80),

    //Base class = 0Ch, serial bus controllers
    IEEE_1394 = ENCODE(SERIAL_BUS_CONTROLLER,0x0), //2 types
    ACCESS_BUS, 
    SSA,
    PCI_USB, 
    FC, 
    SMBUS,
    IBD, 
    IPMI, 
    SERCOS, 
    CANBUS,
    OTHER_0C = ENCODE(SERIAL_BUS_CONTROLLER,0x80),

    //Base class = 0Dh, wireless controllers
    IRDA = ENCODE(WIRELESS_CONTROLLER,0x0),
    Code_0D01, 
    RF = ENCODE(WIRELESS_CONTROLLER,0x10),
    BLUETOOTH,
    BROADBAND,
    ETH_802_11A_5G = ENCODE(WIRELESS_CONTROLLER,0x20), 
    ETH_802_11B_2P4G, 
    OTHER_0D = ENCODE(WIRELESS_CONTROLLER,0x80),


    //Base class = 0xf , satellite communication controllers
    //Base class = 0x10, encryption and decryption controllers
    //Base class = 0x11, data acquisition and signal processing controllers
    //Base class = 0x12, processing accelerators
    //Base class = 0x13, Non-Essential Instrumentation Function

}PCIClassCode;


typedef struct {
    UINT32      code;
    CHAR        szShortName[MAX_TEXT_LENGTH];
    CHAR        szDescription[MAX_TEXT_LENGTH];
}ClassCodeDetailedDescription;


#endif
