#ifndef PCICommonFunction_H
#define PCICommonFunction_H 

#include <list>
#include <fcntl.h>
#include <stdio.h>
#include <sys/unistd.h>
#include <iostream>

#include "pci.h"

using namespace std;

#define MAX_STRING_LENGTH   128

#define DIR_NAME    "/sys/bus/pci/devices"



struct id_bucket {
    struct id_bucket *next;
    unsigned int full;
};

struct device {
    struct device *next;
    struct pci_dev *dev;
    unsigned int config_cached, config_bufsize;
    unsigned char *config;             /* Cached configuration space data */
    unsigned char *present;            /* Maps which configuration bytes are present */
};

struct bridge
{
    struct bridge *chain;                 /* Single-linked list of bridges */
    struct bridge *next, *child;          /* Tree of bridges */
    struct bus *first_bus;                /* List of buses connected to this bridge */
    unsigned int domain;
    unsigned int primary, secondary, subordinate; /* Bus numbers */
    struct device *br_dev;
};

struct bus
{
    unsigned int domain;
    unsigned int number;
    struct bus *sibling;
    struct device *first_dev, **last_dev;
    unsigned int downstream_device;
};

class CapInfo
{
    public:
        CapInfo(unsigned int in_u32CapID, unsigned int in_u32CapOffset, unsigned int in_u32NextCapOffset):
        CapID(in_u32CapID), CapOffset(in_u32CapOffset), NextCapOffset(in_u32NextCapOffset)
        {}

        unsigned int GetCapID()
        {
            return CapID;
        };

        unsigned int GetNextCapOffset()
        {
            return NextCapOffset;
        };

        unsigned int GetCapOffset()
        {
            return CapOffset;
        };

    private:
        unsigned int CapID;
        unsigned int CapOffset;
        unsigned int NextCapOffset;
};


class PCICommonFunction 
{
    public:
        PCICommonFunction();
        ~PCICommonFunction();

        unsigned char   ConfigReadByte(unsigned short in_u16Domain,
                                       unsigned short in_u16Bus,
                                       unsigned short in_u16Device,
                                       unsigned char  in_u8Function,
                                       unsigned int   in_u32Offset);

        unsigned short  ConfigReadWord(unsigned short in_u16Domain,
                                       unsigned short in_u16Bus,
                                       unsigned short in_u16Device,
                                       unsigned char  in_u8Function,
                                       unsigned int   in_u32Offset);

        unsigned int    ConfigReadDword(unsigned short in_u16Domain,
                                        unsigned short in_u16Bus,
                                        unsigned short in_u16Device,
                                        unsigned char  in_u8Function,
                                        unsigned int   in_u32Offset);

        int ConfigWriteByte(unsigned short  in_u16Domain,
                            unsigned short  in_u16Bus,
                            unsigned short  in_u16Device,
                            unsigned char   in_u8Function,
                            unsigned int    in_u32Offset,
                            unsigned char   in_u8WriteData);
        
        int ConfigWriteWord(unsigned short  in_u16Domain,
                            unsigned short  in_u16Bus,
                            unsigned short  in_u16Device,
                            unsigned char   in_u8Function,
                            unsigned int    in_u32Offset,
                            unsigned short  in_u16WriteData);

        int ConfigWriteDword(unsigned short  in_u16Domain,
                             unsigned short  in_u16Bus,
                             unsigned short  in_u16Device,
                             unsigned char   in_u8Function,
                             unsigned int    in_u32Offset,
                             unsigned int    in_u32WriteData);

        void    GetCapList(unsigned short in_u16Domain,
                           unsigned short in_u16Bus,
                           unsigned short in_u16Device,
                           unsigned char  in_u8Function,
                           list<CapInfo> *pCapInfo);

        struct pci_access* pci_alloc(void);
        void pci_cleanup(struct pci_access *a);
        void pci_free_dev(struct pci_dev *d);
        void pci_free_params(struct pci_access *acc);
        void pci_set_name_list_path(struct pci_access *a, char *name, int to_be_freed);
        void pci_free_caps(struct pci_dev *d);
        void pci_mfree(void *x);
        void pci_id_cache_flush(struct pci_access *a);
        void pci_id_hash_free(struct pci_access *a);
        void pci_free_name_list(struct pci_access *a);

        void sysfs_config(struct pci_access *a);
        void sysfs_init(struct pci_access *a);
        void sysfs_scan(struct pci_access *a);

        void sysfs_cleanup_dev(struct pci_dev *d);
        void sysfs_flush_cache(struct pci_access *a);
        void sysfs_cleanup(struct pci_access *a);

        struct device* scan_device(struct pci_dev *p);

        void release_device_struct(struct   device *d);
        void release_pci_tree(struct bridge*    host_bridge);

        void release_bus_struct(struct bus  *bus_to_be_released);
        void release_bridge_struct(struct bridge  *b);
        
        void*   xmalloc(unsigned int howmuch);
        
        struct bus* find_bus(struct bridge *b, unsigned int domain, unsigned int n);
        struct bus* new_bus(struct bridge *b, unsigned int domain, unsigned int n);
        void insert_dev(struct device *d, struct bridge *b);
        

        static int compare_them(const void *A, const void *B);
        void sort_them(struct device* out_first_dev);

        //struct bridge grow_tree();

    private:
        unsigned char   m_u8CapPtrValue;
        //struct bridge   m_host_bridge;
        struct device  *m_first_dev;





};

















#endif 
