#include <iostream>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
//#include <string.h>

#include "PCIDevice.hpp"


PCIDevice::PCIDevice(PCIInformantion in_pciInfo)
{
    //cout << "PCIDevice constructor" << endl; //check 
    
    unsigned int    offset;
    unsigned int    temp;

    m_szDeviceName.assign(in_pciInfo.szDeviceName);
    
    m_u16Domain         = in_pciInfo.u16Domain;
    m_u16Bus            = in_pciInfo.u16Bus;
    m_u16Device         = in_pciInfo.u16Device;
    m_u8Function        = in_pciInfo.u8Function;
    m_u16VendorID       = in_pciInfo.u16VendorID;
    m_u16DeviceID       = in_pciInfo.u16DeviceID;

    //checkPresenceOfDevice();

    //m_bIsPCIe           = false;
    //m_bLinkStatus       = false;
    m_u32CurrentWidth   = 0;
    m_fCurrentSpeed     = 0xff;
    m_u32WidthCap       = 0;
    m_fSpeedCap         = 0xff;
    m_u32ClassCode      = 0x0;
    m_u8RevisionID      = 0x0;
    m_u8HeaderType      = 0x0;
    m_u8CapPtrValue     = 0x0;


    //get class code 
    offset = 0x8;
    temp = ConfigReadDword(offset);
    m_u32ClassCode = (temp & 0xffffff00) >> 8;

    //get revision ID 
    offset = 0x8;
    m_u8RevisionID = ConfigReadByte(offset);

    //get header type 
    offset = 0xe;
    m_u8HeaderType = ConfigReadByte(offset);

    //get capabilities pointer 
    offset = 0x34;
    m_u8CapPtrValue = ConfigReadByte(offset);

    ////get capabilities list
    GetCapList();
    
    //GetLinkInfo();
}


PCIDevice:: ~PCIDevice()
{
    if(m_pPciLibrary != NULL)
    {
        delete m_pPciLibrary;
        m_pPciLibrary = NULL;
    }

    //cout << "PCIDevice distructor" << endl; //check 
    //if(m_dev != NULL)
    //{
    //    free(m_dev);
    //}
}

string
PCIDevice::GetDeviceName()
{
    return m_szDeviceName;
}

unsigned short
PCIDevice::GetDomain()
{
    return m_u16Domain;
}

unsigned short
PCIDevice::GetBus()
{
    return m_u16Bus;
}

unsigned short
PCIDevice::GetDevice()
{
    return m_u16Device;
}

unsigned char 
PCIDevice::GetFunction()
{
    return m_u8Function;
}

unsigned short 
PCIDevice::GetVendorID()
{
    m_u16VendorID = ConfigReadWord(0x0);
    return m_u16VendorID;
}

unsigned short 
PCIDevice::GetDeviceID()
{
    m_u16DeviceID = ConfigReadWord(0x2);
    return m_u16DeviceID;
}

unsigned char 
PCIDevice::GetRevisionID()
{
    m_u8RevisionID = ConfigReadByte(0x8);
    return m_u8RevisionID;
}

unsigned int 
PCIDevice::GetClassCode()
{
    unsigned int temp;

    temp = ConfigReadDword(0x8);
    m_u32ClassCode = (temp & 0xffffff00) >> 8;
    return m_u32ClassCode;    
}

unsigned char 
PCIDevice::GetHeaderType()
{
    m_u8HeaderType = ConfigReadByte(0xe);
    return m_u8HeaderType;
}

unsigned char 
PCIDevice::GetCapPtrValue()
{
    m_u8CapPtrValue = ConfigReadByte(0x34);
    return m_u8CapPtrValue;
}

float
PCIDevice::GetCurrentLinkSpeed()
{
    GetLinkInfo();
    return m_fCurrentSpeed;
}

unsigned short
PCIDevice::GetCurrentLinkWidth()
{
    GetLinkInfo();
    return m_u32CurrentWidth;
}

float
PCIDevice::GetMaxLinkSpeed()
{
    GetLinkInfo();
    return m_fSpeedCap;
}

unsigned short
PCIDevice::GetMaxLinkWidth()
{
    GetLinkInfo();
    return m_u32WidthCap;
}

UINT8
PCIDevice::getPortTypeRawData()
{
    int     ret;
    UINT32  u32PCIeCapOffset;
    UINT16  u16Temp;
    UINT8   u8PortType;

    ret = GetCapabilityOffset(PCIE_CAPABILITY_ID, &u32PCIeCapOffset);
    if(ret == 0)
    {
        u16Temp = ConfigReadWord(u32PCIeCapOffset + PCIE_CAPABILITIES_REG_OFFSET);
        u8PortType = (u16Temp & 0xf0) >> 4;
    }
    else
    {
        u8PortType = 0xff;
    }

    return u8PortType;
}


string
PCIDevice::getPortTypeString()
{
    int     ret;
    UINT32  u32PCIeCapOffset;
    UINT16  u16Temp;
    UINT8   u8PortType;

    ret = GetCapabilityOffset(PCIE_CAPABILITY_ID, &u32PCIeCapOffset);
    if(ret == 0)
    {
        u16Temp = ConfigReadWord(u32PCIeCapOffset + PCIE_CAPABILITIES_REG_OFFSET);
        u8PortType = (u16Temp & 0xf0) >> 4;
    }
    else
    {
        u8PortType = 0xff;
    }

    switch(u8PortType)
    {
        case 0x0: //0000b
            return "PCIe Endpoint";
        case 0x1: //0001b
            return "Legancy PCIe Endpoint";
        case 0x4: //0100b
            return "PCIe Root Complex Root Port";
        case 0x5://0101b
            return "PCIe switch Upstream port";
        case 0x6: //0110b
            return "PCIe switch Downstream port";
        case 0x7: //0111b
            return "PCIe to PCI/PCI-X Bridge";
        case 0x8: //1000b
            return "PCI/PCI-X to PCIe Bridge";
        case 0x9: //1001b
            return "Root Complex Integrated Endpoint";
        case 0xA: //1010b
            return "Root Complex Event Collector";
        default:
            return "Port Type undefined";
    }
}



ERRORCODE 
PCIDevice::getErrorCode()
{
    return m_eErrorCode;
}


/*
ERRORCODE
PCIDevice::checkPresenceOfDevice(UINT16 in_u16Domain,
                                 UINT16 in_u16Bus,
                                 UINT16 in_u16Device,
                                 UINT8  in_u8Function)
{
    int     fd;
    char    namebuf[MAX_STRING_LENGTH];

    snprintf(namebuf, MAX_STRING_LENGTH, "/sys/bus/pci/devices/%04x:%02x:%02x.%d/config",
             in_u16Domain, in_u16Bus, in_u16Device, in_u8Function);

    fd = open(namebuf, O_RDONLY);
    if(fd == -1)
    {
        m_eErrorCode = ERROR_NO_DEVICE_FOUND;
    }
    else
    {
        m_eErrorCode = NO_ERROR;
    }
}
*/

unsigned char   
PCIDevice::ConfigReadByte(unsigned int in_u32Offset)
{
    unsigned char u8Temp;

    //m_pPciLibrary = new PCILibrary();
    m_pPciLibrary = new PCICommonFunction();

    u8Temp = m_pPciLibrary->ConfigReadByte(m_u16Domain, m_u16Bus, m_u16Device, m_u8Function, in_u32Offset);
    
    delete m_pPciLibrary;
    m_pPciLibrary = NULL;

    return u8Temp;
}


unsigned short   
PCIDevice::ConfigReadWord(unsigned int in_u32Offset)
{
    unsigned short u16Temp;

    //m_pPciLibrary = new PCILibrary();
    m_pPciLibrary = new PCICommonFunction();
    
    u16Temp = m_pPciLibrary->ConfigReadWord(m_u16Domain, m_u16Bus, m_u16Device, m_u8Function, in_u32Offset);
    
    delete m_pPciLibrary;
    m_pPciLibrary = NULL;

    return u16Temp;
}


unsigned int    
PCIDevice::ConfigReadDword(unsigned int in_u32Offset)
{
    unsigned int u32Temp;

    //m_pPciLibrary = new PCILibrary();
    m_pPciLibrary = new PCICommonFunction();
    
    u32Temp = m_pPciLibrary->ConfigReadDword(m_u16Domain, m_u16Bus, m_u16Device, m_u8Function, in_u32Offset);

    delete m_pPciLibrary;
    m_pPciLibrary = NULL;

    return u32Temp;

}

int 
PCIDevice::ConfigWriteByte(unsigned int     in_u32Offset,
                           unsigned char    in_u8WriteData)
{
    int ret;

    m_pPciLibrary = new PCICommonFunction();

    ret = m_pPciLibrary->ConfigWriteByte(m_u16Domain, m_u16Bus, m_u16Device, m_u8Function, 
                                         in_u32Offset, in_u8WriteData);

    delete m_pPciLibrary;
    m_pPciLibrary = NULL;

    return ret;   
}


int 
PCIDevice::ConfigWriteWord(unsigned int     in_u32Offset,
                           unsigned short   in_u16WriteData)
{
    int ret;

    //m_pPciLibrary = new PCICommonFunction();

    ret = m_pciLibrary.ConfigWriteWord(m_u16Domain, m_u16Bus, m_u16Device, m_u8Function,
                                       in_u32Offset, in_u16WriteData);

    //ret = m_pPciLibrary->ConfigWriteWord(m_u16Domain, m_u16Bus, m_u16Device, m_u8Function, 
    //                                     in_u32Offset, in_u16WriteData);

    //delete m_pPciLibrary;
    //m_pPciLibrary = NULL;

    return ret;   
}


int 
PCIDevice::ConfigWriteDword(unsigned int    in_u32Offset,
                            unsigned int    in_u32WriteData)
{
    int ret;

    m_pPciLibrary = new PCICommonFunction();

    ret = m_pPciLibrary->ConfigWriteDword(m_u16Domain, m_u16Bus, m_u16Device, m_u8Function, 
                                          in_u32Offset, in_u32WriteData);

    delete m_pPciLibrary;
    m_pPciLibrary = NULL;

    return ret;   
}


void
PCIDevice::GetBARList(vector<unsigned int> *pBarList)
{
    unsigned int     index;
    unsigned int     u32tempBar;
    unsigned int     u32BarAmount;
    unsigned int     u32BarOffset;
    unsigned char    u8HeaderType;

    u8HeaderType = ConfigReadByte(0xe);

    if((u8HeaderType & 0x1) == 0x1)
    {
        u32BarAmount = 2;
    }
    else 
    {
        u32BarAmount = 6;
    }

    u32BarOffset = PCI_BAR_START_OFFSET; //BAR starts from 0x10 
    for(index = 0; index < u32BarAmount; index++)
    {
        u32tempBar = ConfigReadDword(u32BarOffset);
        //u32tempBar = u32tempBar & 0xfffffff0;
        pBarList->push_back(u32tempBar);
        u32BarOffset = u32BarOffset + 0x4;
    }
}


/*
unsigned long  
PCIDevice::GetMMIO()
{
    unsigned char   u8MemoryMapType;
    unsigned long   u32BAR0, u32BAR1;
    unsigned long   u64mmioAddress;

    printf("sizeof unsigned long long: %ld\n", sizeof(unsigned long long));
    printf("sizeof unsigned long: %ld\n", sizeof(unsigned long));
    
    u32BAR0 = ConfigReadDword(0x10);
    u8MemoryMapType = (u32BAR0 & 0x6) >> 1;
    printf("u8MemoryMapType: 0x%x\n", u8MemoryMapType);//check 

    u64mmioAddress = u32BAR0 & 0xfffffff0;

    if(u8MemoryMapType == 0x2) 
    {
        u32BAR1 = ConfigReadDword(0x14);
        u64mmioAddress = (u32BAR1 << 32) | u64mmioAddress;
    }

    return u64mmioAddress;
}


void 
PCIDevice::GetMMIO(list<unsigned long> *pMMIOList)
{


}
*/


void 
PCIDevice::GetCapList()
{
    UINT32  u32temp;
    UINT32  u32tempCapID;
    UINT32  u32tempCapOffset;
    UINT32  u32tempNextCapOffset;
    UINT8   u8CapPtrValue;

    u8CapPtrValue = ConfigReadByte(0x34);

    if(u8CapPtrValue != 0x0)
    {
        u32tempCapOffset = (UINT32)u8CapPtrValue;
        u32temp = ConfigReadDword(u32tempCapOffset);
        u32tempCapID = u32temp & 0xff;
        u32tempNextCapOffset = (u32temp & 0xff00) >> 8;
        //printf("tempNextCapOffset: 0x%x\n", tempNextCapOffset);//check 
        
        while(u32tempNextCapOffset != 0x0)
        {
            CapInfo tempCapInfo(u32tempCapID, u32tempCapOffset, u32tempNextCapOffset);
            u32tempCapOffset = u32tempNextCapOffset;
            m_listCap.push_back(tempCapInfo);
            u32temp = ConfigReadDword(u32tempNextCapOffset);
            u32tempCapID = u32temp & 0xff;
            u32tempNextCapOffset = (u32temp & 0xff00) >> 8;
        }

        CapInfo tempCapInfo(u32tempCapID, u32tempCapOffset, u32tempNextCapOffset);
        m_listCap.push_back(tempCapInfo);
    }

    getExtendedCapList();
    
    //check
    /*
    list<CapInfo>::iterator it;

    for(it = m_listCap.begin(); it != m_listCap.end(); ++it)
    {
        printf("CapID: 0x%02x  CapOffset: 0x%02x NextCapOffset:0x%02x\n",
                (unsigned short)it->GetCapID(), (unsigned short)it->GetCapOffset(), (unsigned short)it->GetNextCapOffset());
    }
    */
}


void 
PCIDevice::getExtendedCapList()
{
    UINT32    u32temp;
    UINT32    u32tempCapID;
    UINT32    u32tempCapOffset;
    UINT32    u32tempNextCapOffset;

    u32tempCapOffset = 0x100;
    u32temp = ConfigReadDword(u32tempCapOffset);
    u32tempCapID = u32temp & 0xffff;
    //printf("ext tempCapID: 0x%x\n", u32tempCapID);//check 
    u32tempNextCapOffset = (u32temp & 0xfff00000) >> 20;
    //printf("ext tempNextCapOffset: 0x%x\n", u32tempNextCapOffset);//check 
    
    while(u32tempNextCapOffset != 0x0)
    {
        CapInfo tempCapInfo(u32tempCapID, u32tempCapOffset, u32tempNextCapOffset);
        u32tempCapOffset = u32tempNextCapOffset;
        m_listCap.push_back(tempCapInfo);
        u32temp = ConfigReadDword(u32tempNextCapOffset);
        u32tempCapID = u32temp & 0xffff;
        u32tempNextCapOffset = (u32temp & 0xfff00000) >> 20;
    }

    CapInfo tempCapInfo(u32tempCapID, u32tempCapOffset, u32tempNextCapOffset);
    m_listCap.push_back(tempCapInfo);
}



bool
PCIDevice::IsPCIe()
{
    list<CapInfo>::iterator it;

    for(it = m_listCap.begin(); it != m_listCap.end(); ++it)
    {
        if(it->GetCapID() == 0x10 && it->GetNextCapOffset() < 0x100)
        {
            return true;
        }
    }
    
    return false;
}

bool
PCIDevice::LinkStatus()
{
    UINT16                      u16Temp;
    list<CapInfo>::iterator     it;


    for(it = m_listCap.begin(); it != m_listCap.end(); ++it)
    {
        if(it->GetCapID() == 0x10 && it->GetNextCapOffset() < 0x100)
        {
            u16Temp = ConfigReadWord(it->GetCapOffset() + LINK_CAPABILITY_REG_OFFSET);
            if(u16Temp & 0x100000) //Data Link Layer Link Active Reporting Capable bit
            {
                u16Temp = ConfigReadWord(it->GetCapOffset() + LINK_STATUS_REG_OFFSET);
                u16Temp = (u16Temp & 0x2000) >> 13;

                if(u16Temp == 0x1)
                {
                    return true;
                }
            }
            else
            {
                GetLinkInfo();
                if(m_u32CurrentWidth != 0)
                {
                    return true;
                }
            }
        }
    }

    return false;
}

unsigned char 
PCIDevice::getPortNumber()
{
    int             ret;
    unsigned int    u32Temp;
    unsigned int    u32PCIeCapOffset;

    ret = GetCapabilityOffset(PCIE_CAPABILITY_ID, &u32PCIeCapOffset);
    if(ret == 0)
    {
        u32Temp = ConfigReadDword(u32PCIeCapOffset + LINK_CAPABILITY_REG_OFFSET);
        return ((u32Temp & 0xff000000) >> 24);
    }
    return 0xff;
}


void 
PCIDevice::GetLinkInfo()
{
    //printf("PCIDevice::GetLinkInfo\n");//check 

    list<CapInfo>::iterator it;
    unsigned short          u16Temp;
    unsigned int            u32PCIeCapOffset;
    int                     ret; //need to change!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    ret = GetCapabilityOffset(PCIE_CAPABILITY_ID, &u32PCIeCapOffset);
    if(ret == 0)
    {
        //get current link speed
        //printf("PCIDevice::GetLinkInfo, u32PCIeCapOffset: 0x%x\n", u32PCIeCapOffset);//check 
        m_fCurrentSpeed = GetLinkSpeed(u32PCIeCapOffset + LINK_STATUS_REG_OFFSET);
        //printf("current speed: %.1f\n", m_fCurrentSpeed);//check 
        //get max link speed 
        m_fSpeedCap = GetLinkSpeed(u32PCIeCapOffset + LINK_CAPABILITY_REG_OFFSET);
        //printf("PCIDevice::GetLinkInfo, m_fSpeedCap: %.1f\n", m_fSpeedCap);//check 
        //get current link width
        u16Temp = ConfigReadWord(u32PCIeCapOffset + LINK_STATUS_REG_OFFSET);
        m_u32CurrentWidth = (u16Temp & 0x3f0) >> 4;
        //printf("PCIDevice::GetLinkInfo, m_u32CurrentWidth: 0x%x\n", m_u32CurrentWidth);//check 

        //get max link width
        u16Temp = ConfigReadWord(u32PCIeCapOffset + LINK_CAPABILITY_REG_OFFSET);
        m_u32WidthCap = (u16Temp & 0x3f0) >> 4;
        //printf("PCIDevice::GetLinkInfo, m_u32WidthCap: 0x%x\n", m_u32WidthCap);//check

    }
}


int 
PCIDevice::getExtendedCapabilityOffset(unsigned int   in_u32CapID,
                                       unsigned int   *CapOffset)
{
    list<CapInfo>::iterator it;

    for(it = m_listCap.begin(); it != m_listCap.end(); ++it)
    {
        if(it->GetCapID() == in_u32CapID && it->GetCapOffset() >= 0x100)
        {
            *CapOffset = it->GetCapOffset();
            return 0;
        }
    }

    *CapOffset = 0xffffffff;

    return 1;
}


int //need to change to ERRORCODE in the future 
PCIDevice::GetCapabilityOffset(unsigned int   in_u32CapID,
                               unsigned int   *CapOffset)
{
    list<CapInfo>::iterator it;

    //GetCapList();

    for(it = m_listCap.begin(); it != m_listCap.end(); ++it)
    {
        //printf("PCIDevice::GetCapabilityOffset, CapID: 0x%x\n", it->GetCapID());//check 
        //printf("PCIDevice::GetCapabilityOffset, CapIDOffset: 0x%x\n", it->GetCapOffset());//check 
        if(it->GetCapID() == in_u32CapID && it->GetCapOffset() < 0x100)
        {
            *CapOffset = it->GetCapOffset();
            return 0; //need to change!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!            
        }
    }

    *CapOffset = 0xffffffff;

    return 1; //need to change!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
}



float
PCIDevice::GetLinkSpeed(unsigned int in_u32LinkStatusRegAddr)
{
    unsigned short  u16Temp;

    u16Temp = ConfigReadWord(in_u32LinkStatusRegAddr);
    u16Temp = u16Temp & 0xf;
    switch(u16Temp)
    {
        case 0x1:
            return 2.5;
        case 0x2:
            return 5.0;
        case 0x3:
            return 8.0;
        default:
            return 0.0;
    }
}

PCIE_TYPE
PCIDevice::GetPCIeType()
{
   unsigned short   u16Temp;
   unsigned int     u32CapOffset;
   int              ret; //need to change !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   u16Temp = 0xffff;

   ret = GetCapabilityOffset(PCIE_CAPABILITY_ID, &u32CapOffset);
   if(ret == 0)
   {
       u16Temp = ConfigReadWord(u32CapOffset + PCIE_CAPABILITIES_REG_OFFSET);
       u16Temp = (u16Temp & 0xf0) >> 4;
   }
   
   return (PCIE_TYPE)u16Temp;
}

/*
void 
PCIDevice::GetCapList()
{
    m_pPciLibrary = new PCILibrary();

    printf("%04x:%02x:%02x.%x\n", m_u16Domain, m_u16Bus, m_u16Device, m_u8Function);//check 
    m_pPciLibrary->GetCapList(m_u16Domain, m_u16Bus, m_u16Device, m_u8Function, &m_listCap);

    delete m_pPciLibrary;
    m_pPciLibrary = NULL;    
}
*/

bool
PCIDevice::checkRetrainCap()
{
    int     ret;
    UINT8   u8PortType;
    UINT16  u16Temp;
    UINT32  u32CapOffset;
    
    ret = GetCapabilityOffset(PCIE_CAPABILITY_ID, &u32CapOffset);
    if(ret != 0)
    {
        return false;
    }
    
    u16Temp = ConfigReadWord(u32CapOffset + PCIE_CAPABILITIES_REG_OFFSET);
    u8PortType = (u16Temp & 0xf0) >> 4;
    switch(u8PortType)
    {
        //retrain unsupported
        case PORT_TYPE_PCIE_ENDPOINT: //0000b
        case PORT_TYPE_LEGACY_PCIE_ENDPOINT: //0001b
        case PORT_TYPE_PCIE_SWITCH_UPSTREAM_PORT://0101b
        case PORT_TYPE_PCIE_TO_PCI_PCIX_BRIDGE: //0111b
            return false;  

        //retrain supported 
        case PORT_TYPE_PCIE_ROOT_COMPLEX_ROOT_PORT: //0100b 
        case PORT_TYPE_PCIE_SWITCH_DOWNSTREAM_PORT: //0110b
        case PORT_TYPE_PCI_PCIX_TO_PCIE_BRIDGE: //1000b
        case PORT_TYPE_ROOT_COMPLEX_INTEGRATED_ENDPOINT: //1001b
        case PORT_TYPE_ROOT_COMPLEX_EVENT_COLLECTOR: //1010b
            return true;

        default:
            return false;
    }
    
}



/*
PCIDevice::getError(PCI_ERROR_TYPE   in_pciError)
{
    switch(in_pciError)
    {
        //PCI-compatible 
        case DETECTED_PARITY_ERROR:
        case SIGNALED_SYSTEM_ERROR:
        case RECEIVED_MASTER_ABORT:
        case RECEIVED_TARGET_ABORT:
        case SIGNALED_TARGET_ABORT:
        case MASTER_DATA_PARITY_ERROR:
        case RECEIVED_SYSTEM_ERROR:

        //PCIE
        case UNSUPPORTED_REQUEST_DETECTED:
        case FATAL_ERROR_DETECTED:
        case NON_FATAL_ERROR_DETECTED:
        case CORRECTABLE_ERROR_DETECTED:

        //AER
        case UNCORRECTABLE_ERROR_STATUS:
        case CORRECTABLE_ERROR_STATUS:

        default:
    }
}
*/

