/// @file       SPDDDAPI.hpp
/// @brief      SPDDDAPI class
/// @details    Provide low layer utility APIs for DIMM's SPD
/// @author     KenHu
/// @copyright  Copyright 2015 Hon Hai Technology Group. All Rights Reserved.

#ifndef SPD_DDAPI_HPP
#define SPD_DDAPI_HPP

#include <string>
#include <list>
#include "BaseClassDDAPI.hpp"
#include "I2CDAPI.hpp"

#define SPD_OFFSET_DEVICE_TYPE          2
#define SPD_OFFSET_MODULE_TYPE          3
#define SPD_OFFSET_SDRAM_DENSITY_BANKS  4

// DDR3 SPD offset definition
#define SPD_DDR3_OFFSET_MODULE_ORGANIZATION     7
#define SPD_DDR3_OFFSET_MODULE_MEM_BUS_WIDTH    8
#define SPD_DDR3_OFFSET_MINI_CYCLE_TIME         12
#define SPD_DDR3_OFFSET_MODULE_MANUFACTURE_ID   148
#define SPD_DDR3_OFFSET_MODULE_SERIAL_NUMBER    122
#define SPD_DDR3_OFFSET_MODULE_PART_NUMBER      128
#define SPD_DDR3_OFFSET_DRAM_MANUFACTURE_ID     148

// DDR4 SPD offset definition
#define SPD_DDR4_OFFSET_SDRAM_PACKAGE_TYPE      6
#define SPD_DDR4_OFFSET_MODULE_ORGANIZATION     12
#define SPD_DDR4_OFFSET_MODULE_MEM_BUS_WIDTH    13
#define SPD_DDR4_OFFSET_MINI_CYCLE_TIME         18
#define SPD_DDR4_OFFSET_MODULE_MANUFACTURE_ID   320
#define SPD_DDR4_OFFSET_MODULE_SERIAL_NUMBER    325
#define SPD_DDR4_OFFSET_MODULE_PART_NUMBER      329
#define SPD_DDR4_OFFSET_DRAM_MANUFACTURE_ID     350

// SPD byte length definition
#define SPD_LENGTH_MODULE_MANUFACTURE_ID        2
#define SPD_LENGTH_MODULE_SERIAL_NUMBER         4
#define SPD_LENGTH_DRAM_MANUFACTURE_ID          2
// DDR3 SPD byte length definition
#define SPD_DDR3_LENGTH_MODULE_PART_NUMBER      18
// DDR4 SPD byte length definition
#define SPD_DDR4_LENGTH_MODULE_PART_NUMBER      20


typedef enum
{
    DDR_TYPE_UNKNOWN,
    DDR3 = 0x0B,
    DDR4 = 0x0C,
} DDRType;

typedef enum
{
    MEMORY_MODUEL_TYPE_UNKNOWN,
    RDIMM = 0x1,
    UDIMM,
    SO_DIMM,
    LRDIMM,
    MINI_RDIMM,
    MINI_UDIMM,
    SO_RDIMM_72B = 0x8,
    SO_UDIMM_72B,
    SO_DIMM_16B = 0xC,
    SO_DIMM_32B = 0xD,
} MemoryModuleType;

/// @class      SPDDDAPI SPDDDAPI.hpp "SPDDDAPI.hpp"
/// @brief      Provide low layer utility APIs for DIMM's SPD
/// @param      m_pI2CDAPI is a pointer to I2C DAPI
class SPDDDAPI : public BaseClassDDAPI
{
    public:
                        SPDDDAPI(void);
        virtual         ~SPDDDAPI(void);

        ErrorMsg*       getDeviceList(list<string> *out_pDeviceList,
                                      BOOL          in_bDetectedOnly = TRUE);
        ErrorMsg*       getDeviceType(string    in_strDIMM,
                                      DDRType   *out_epType);
        ErrorMsg*       getDRAMVendor(string in_strDIMM,
                                      string *out_strpName);
        ErrorMsg*       getModuleType(string in_strDIMM,
                                      string *out_strpType);
        ErrorMsg*       getModuleVendor(string in_strDIMM,
                                        string *out_strpName);
        ErrorMsg*       getPartNumber(string in_strDIMM,
                                      string *out_strpNumber);
        ErrorMsg*       getRankCount(string in_strDIMM,
                                     UINT32 *out_u32pCount);
        ErrorMsg*       getSerialNumber(string in_strDIMM,
                                        string *out_strpNumber);
        ErrorMsg*       getSize(string in_strDIMM,
                                UINT64 *out_u64pSize);
        ErrorMsg*       getSpeed(string in_strDIMM,
                                 UINT32 *out_u32pSpeed);

    protected:

    private:
        I2CDAPI*        m_pI2CDAPI;
};
#endif

