#ifndef STORAGEDDAPI_OPERATION_HPP
#define STORAGEDDAPI_OPERATION_HPP

#include <iostream>
#include <sstream>
#include <string>
#include <stdio.h>
#include <stdlib.h>

#include "BaseClassDDAPI.hpp"
#include "StorageInfo.hpp"

#include "SASDDAPI.hpp"
#include "NVMeDDAPI.hpp"
#include "USBDDAPI.hpp"

#include "SCSIFactory.hpp"

using namespace std;

/// @emun       IOCTL operations
///             List all supported IOCTL operations
typedef enum
{
     eStorageGenerateList = 0,
     eStorageListDrive,
     eStorageReadPerformanceTest,
     eStorageWritePerformanceTest,
     eStorageReadWriteTest,

     eStorageUnknownIOCTLOperation,
} StorageIOCTLOperation;

/// @emun       data elements
///             List all supported Storage data elements
typedef enum
{
     eStorageDriveDataAll,
     eStorageDriveDeviceNodeName,
     eStorageDriveSGDeviceNodeName,
     eStorageDriveVendorName,
     eStorageDriveSerialNumber,
     eStorageDriveModelName,
     eStorageDriveFirmwareVersion,
     eStorageDriveLogicalBlockSize,
     eStorageDrivePhysicalBlockSize,
     eStorageDriveSASAddress,
     eStorageDriveCapacity,
     eStorageDrivePortNumber,
     eStorageDriveLinkStatus,
     eStorageDriveLinkSpeed,
     eStorageDriveTemperature,
     eStorageDriveRotationRate,
     eStorageDriveFormFactor,
     eStorageDriveProtocolType,
     eStorageDriveUSBVersion,

     eStorageDriveUnknownDataElement,
} StorageDataElement;

/// @class      StorageDDAPIOperation StorageDDAPIOperation.hpp "StorageDDAPIOperation.hpp"
/// @brief      Provide low layer diagnostic and utility APIs for Storage module
/// @param      *m_pPCIDDAPI is a pointer to access PCI DDAPI
class StorageDDAPIOperation: public BaseClassDDAPI
{
public:
     StorageDDAPIOperation(STRING className);
     virtual ~StorageDDAPIOperation(void);
     
     ErrorMsg *IOCTL(UINT32 in_u32DefinedIndex, StorageIOCTLOperation in_eOperation);

     ErrorMsg *generateStorageList(void);
     ErrorMsg *listDrive(void);

     ErrorMsg *getIndexByName(string in_strDefinedName, UINT32 *out_u32pDefinedIndex);
     ErrorMsg *getNameByIndex(UINT32 in_u32DefinedIndex, string *out_strDefinedName);
     ErrorMsg *getElementDataByIndex(UINT32 in_u32DefinedIndex, StorageDataElement in_eStorageDataElement, string *out_strpReturnData);
     ErrorMsg *getElementDataByName(string in_strDefinedName, StorageDataElement in_eStorageDataElement, string *out_strpReturnData);
     ErrorMsg *getControllerTypeByIndex(UINT32 in_u32DefinedIndex, UINT32 *out_u32pControllerType);
     ErrorMsg *getSMARTByIndex(UINT32 in_u32DefinedIndex, string *out_strpReturnData);

protected:

private:
     list <StorageDriveStruct> m_storageDriveList;
     SASDDAPI                 *m_pSASDDAPI;
     NVMeDDAPI                *m_pNVMeDDAPI;
     USBDDAPI                 *m_pUSBDDAPI;
     SCSIFactory              *m_pSCSIFactory;

     string convertStringToUpperCase(string in_strInputString);
};
#endif

