#include "Temperature_I2C.hpp"



Temperature_I2C :: Temperature_I2C(STRING  ClassName) : TemperatureGeneric(ClassName)
{
	m_I2CDdapi = new I2CDDAPI("TEMP_I2C");
	if(m_I2CDdapi != NULL)
	{

	}

}



Temperature_I2C :: ~Temperature_I2C()
{
	if( m_I2CDdapi != NULL )
		delete m_I2CDdapi;

}

ErrorMsg*  Temperature_I2C :: getTemperatureNameList(list<string> *pTemperatureList)
{
    //printf("%s\n", __PRETTY_FUNCTION__);;
	ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    I2CDevice *temp_i2c_device[DEVICE_NUMBER];

    pErrorMsg = m_I2CDdapi->DisplayI2cDevicesListByType(temp_i2c_device, TEMPERATURE);

    if(pErrorMsg == NO_ERROR_MSG)
    {
		for (int i=0; temp_i2c_device[i]; i++)
		{
			string tempString;
			tempString.clear();
			pTemperatureList->push_back(temp_i2c_device[i]->get_Name());
		}

		//Delete the non-NULL point
		for (int i=0; i<DEVICE_NUMBER; i++)
		{
		   if (temp_i2c_device[i] != NULL)
		   {
			  delete temp_i2c_device[i];
		   }
		}
    }
    else
    {
    	return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_GET_TEMPERATURE_NAME_LIST, NULL, pErrorMsg);
    }

    return NO_ERROR_MSG;
}

ErrorMsg*  Temperature_I2C :: getDataByName(STRING WellKnownName,
										    TEMPERATURE_DATA_ELEMENT TemperatureDataElement,
                                            list<string> *returnDataInfo)
{

    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;

	switch(TemperatureDataElement)
	{
		case TEMPERATURE_DATA_ALL_INFO:
			//TemperatureDataElement=TEMPERATURE_DATA_ALL_INFO;
			break;
		case TEMPERATURE_DATA_CURRENT_TEMPERATURE:
			pErrorMsg = getCurrentTemperatureList(WellKnownName, returnDataInfo);
			if(pErrorMsg != NO_ERROR_MSG)
			{
				return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_GET_DATA_BY_NAME, NULL, pErrorMsg);
			}
			break;

		case TEMPERATURE_DATA_THRESHOLD_HIGH0:
			pErrorMsg = getThresholdHigh0List(WellKnownName, returnDataInfo);
			if(pErrorMsg != NO_ERROR_MSG)
			{
				return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_GET_DATA_BY_NAME, NULL, pErrorMsg);
			}
			break;

		case TEMPERATURE_DATA_THRESHOLD_LOW0:
			pErrorMsg = getThresholdLow0List(WellKnownName, returnDataInfo);
			if(pErrorMsg != NO_ERROR_MSG)
			{
				return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_GET_DATA_BY_NAME, NULL, pErrorMsg);
			}
			break;

		case TEMPERATURE_DATA_THRESHOLD_HIGH1:
			pErrorMsg = getThresholdHigh1List(WellKnownName, returnDataInfo);
			if(pErrorMsg != NO_ERROR_MSG)
			{
				return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_GET_DATA_BY_NAME, NULL, pErrorMsg);
			}
			break;

		case TEMPERATURE_DATA_THRESHOLD_LOW1:
			pErrorMsg = getThresholdLow1List(WellKnownName, returnDataInfo);
			if(pErrorMsg != NO_ERROR_MSG)
			{
				return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_GET_DATA_BY_NAME, NULL, pErrorMsg);
			}
			break;
		default:
			returnDataInfo->push_back("Not Support");
			break;
	}

    return NO_ERROR_MSG;
}

ErrorMsg*  Temperature_I2C :: setDataByName(STRING WellKnownName,
											TEMPERATURE_DATA_ELEMENT TemperatureDataElement,
											string	SetValue)
{
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;

	switch(TemperatureDataElement)
	{
		case TEMPERATURE_DATA_ALL_INFO:
			//TemperatureDataElement=TEMPERATURE_DATA_ALL_INFO;
			break;
		case TEMPERATURE_DATA_CURRENT_TEMPERATURE:
			break;
		case TEMPERATURE_DATA_THRESHOLD_HIGH0:
			pErrorMsg = setThresholdHigh0List(WellKnownName, SetValue);
			if(pErrorMsg != NO_ERROR_MSG)
			{
				return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_SET_DATA_BY_NAME, NULL, pErrorMsg);
			}
			break;
		case TEMPERATURE_DATA_THRESHOLD_LOW0:
			pErrorMsg = setThresholdLow0List(WellKnownName, SetValue);
			if(pErrorMsg != NO_ERROR_MSG)
			{
				return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_SET_DATA_BY_NAME, NULL, pErrorMsg);
			}
			break;

		case TEMPERATURE_DATA_THRESHOLD_HIGH1:
			pErrorMsg = setThresholdHigh1List(WellKnownName, SetValue);
			if(pErrorMsg != NO_ERROR_MSG)
			{
				return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_SET_DATA_BY_NAME, NULL, pErrorMsg);
			}
			break;

		case TEMPERATURE_DATA_THRESHOLD_LOW1:
			pErrorMsg = setThresholdLow1List(WellKnownName, SetValue);
			if(pErrorMsg != NO_ERROR_MSG)
			{
				return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_SET_DATA_BY_NAME, NULL, pErrorMsg);
			}
			break;

		default:
			break;
	}

    return NO_ERROR_MSG;
}



ErrorMsg* Temperature_I2C :: getCurrentTemperatureList(STRING WellKnownName, list<string> *pTemperatureList)
{
    //printf("%s\n", __PRETTY_FUNCTION__);;

	//ERRORCODE errorCode = NO_ERROR;
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

    //printf("WellKnownName=%s\n", WellKnownName);

	pErrorMsg = m_I2CDdapi->getTemperatureDataByID(WellKnownName, CURRENT_TEMPERATURE, &tempString);
    //pErrorMsg = m_I2CDdapi->getTemperatureDataByID("Tmpe01", CURRENT_TEMPERATURE, &tempString);

	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
		return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_GET_CURRENT_TEMPERATURE, NULL, pErrorMsg);
    }
    else
    {
    	pTemperatureList->push_back(tempString);
    }
    return NO_ERROR_MSG;
}


ErrorMsg*   Temperature_I2C :: getThresholdHigh0List(STRING WellKnownName, list<string> *pTemperatureList)
{
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

    //printf("WellKnownName=%s\n", WellKnownName);

	pErrorMsg = m_I2CDdapi->getTemperatureDataByID(WellKnownName, THRESHOLD_HIGH, &tempString);
    //pErrorMsg = m_I2CDdapi->getTemperatureDataByID("Tmpe01", CURRENT_TEMPERATURE, &tempString);

	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_GET_THRESHOLD_HIGH0_LIST, NULL, pErrorMsg);
    }
    else
    {
    	pTemperatureList->push_back(tempString);
    }


    return NO_ERROR_MSG;
}

ErrorMsg*   Temperature_I2C :: getThresholdLow0List(STRING WellKnownName, list<string> *pTemperatureList)
{
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

    //printf("WellKnownName=%s\n", WellKnownName);

	pErrorMsg = m_I2CDdapi->getTemperatureDataByID(WellKnownName, THRESHOLD_LOW, &tempString);
  	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_GET_THRESHOLD_LOW0_LIST, NULL, pErrorMsg);
    }
    else
    {
    	pTemperatureList->push_back(tempString);
    }
    return NO_ERROR_MSG;
}

ErrorMsg*   Temperature_I2C :: getThresholdHigh1List(STRING WellKnownName, list<string> *pTemperatureList)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
	/*
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

	pErrorMsg = m_I2CDdapi->getTemperatureDataByID(WellKnownName, THRESHOLD_LOW, &tempString);
  	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_GET_THRESHOLD_HIGH1_LIST, NULL, pErrorMsg);
    }
    else
    {
    	pTemperatureList->push_back(tempString);
    }
	*/

	pTemperatureList->push_back("Not Support");

	return NO_ERROR_MSG;
}

ErrorMsg*   Temperature_I2C :: getThresholdLow1List(STRING WellKnownName, list<string> *pTemperatureList)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
	/*
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

	pErrorMsg = m_I2CDdapi->getTemperatureDataByID(WellKnownName, THRESHOLD_LOW, &tempString);
  	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_GET_THRESHOLD_LOW1_LIST, NULL, pErrorMsg);
    }
    else
    {
    	pTemperatureList->push_back(tempString);
    }
	*/

	pTemperatureList->push_back("Not Support");

	return NO_ERROR_MSG;
}




ErrorMsg*   Temperature_I2C :: setThresholdHigh0List(STRING WellKnownName, string SetValue)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

    //printf("WellKnownName=%s\n", WellKnownName);

	pErrorMsg = m_I2CDdapi->setTemperatureDataByID(WellKnownName, THRESHOLD_HIGH, SetValue);
    //pErrorMsg = m_I2CDdapi->getTemperatureDataByID("Tmpe01", CURRENT_TEMPERATURE, &tempString);

	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_SET_THRESHOLD_HIGH0_LIST, NULL, pErrorMsg);
    }
	return NO_ERROR_MSG;
}
ErrorMsg*   Temperature_I2C :: setThresholdLow0List(STRING WellKnownName, string SetValue)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

    //printf("WellKnownName=%s\n", WellKnownName);

	pErrorMsg = m_I2CDdapi->setTemperatureDataByID(WellKnownName, THRESHOLD_LOW, SetValue);
    //pErrorMsg = m_I2CDdapi->getTemperatureDataByID("Tmpe01", CURRENT_TEMPERATURE, &tempString);

	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_SET_THRESHOLD_LOW0_LIST, NULL, pErrorMsg);
    }
	return NO_ERROR_MSG;
}
ErrorMsg*   Temperature_I2C :: setThresholdHigh1List(STRING WellKnownName, string SetValue)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
	/*
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

    //printf("WellKnownName=%s\n", WellKnownName);

	pErrorMsg = m_I2CDdapi->setTemperatureDataByID(WellKnownName, THRESHOLD_LOW, SetValue);
    //pErrorMsg = m_I2CDdapi->getTemperatureDataByID("Tmpe01", CURRENT_TEMPERATURE, &tempString);

	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_SET_THRESHOLD_HIGH1_LIST, NULL, pErrorMsg);
    }
*/


	return NO_ERROR_MSG;
}
ErrorMsg*   Temperature_I2C :: setThresholdLow1List(STRING WellKnownName, string SetValue)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
/*
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

    //printf("WellKnownName=%s\n", WellKnownName);

	pErrorMsg = m_I2CDdapi->setTemperatureDataByID(WellKnownName, THRESHOLD_LOW, SetValue);
    //pErrorMsg = m_I2CDdapi->getTemperatureDataByID("Tmpe01", CURRENT_TEMPERATURE, &tempString);

	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_TEMPERATURE_I2C_DDAPI_SET_THRESHOLD_LOW1_LIST, NULL, pErrorMsg);
    }
*/
	return NO_ERROR_MSG;
}


