#include "UARTDDAPI.hpp"

UART_PORT_TYPE  UartPortTypeTable[] = 
{
    {PORT_UNKNOWN,    "UNKNOWN"},
    {PORT_8250,       "8250"},
    {PORT_16450,      "16450"},
    {PORT_16550,      "16550"},
    {PORT_16550A,     "16550A"},
    {PORT_CIRRUS,     "CIRRUS"},
    {PORT_16650,      "16650"},
    {PORT_16650V2,    "16650V2"},
    {PORT_16750,      "16750"},
    {PORT_STARTECH,   "STARTECH"},
    {PORT_16C950,     "16C950"},
    {PORT_16654,      "16654"},
    {PORT_16850,      "16850"},
    {PORT_UNKNOWN,    "UNKNOWN"},
    {-1,              "UNKNOWN"},
};

UARTDDAPI::UARTDDAPI(STRING className) : BaseClassDDAPI(className)
{

}

UARTDDAPI::~UARTDDAPI()
{

}

ErrorMsg*
UARTDDAPI::getDevice(STRING          in_uartName,
                     serial_struct  *out_pSerialInfo)
{
    int             fd;
    int             ret;
    struct termios  opts;
    ERRORCODE       eErrorCode = NO_ERROR;

    //printf("device name: %s\n", in_uartName);//check 
    fd = open(in_uartName, O_RDWR);
    if(fd < 0)
    {
        //cout << "open device fail" << endl;//check
        return returnErrorMsg((ERRORCODE)(ERROR_UART_DDAPI_GET_DEVICE | ERROR_OPEN_FILE), NULL, NULL);
    }

    ret = tcgetattr(fd, &opts);
    if(ret < 0)
    {
        //printf("tcgetattr fail\n");//check 
        close(fd);
        return returnErrorMsg((ERRORCODE)(ERROR_UART_DDAPI_GET_DEVICE | ERROR_NO_DEVICE_FOUND), NULL, NULL);
    }

    //get serial info 
    ret = ioctl(fd, TIOCGSERIAL, out_pSerialInfo);
    if(ret < 0)
    {
        //cout << "fail to get serial info" << endl;//check 
        eErrorCode = ERROR_UART_DDAPI_IOCTL_GET_SERIAL_INFO;
    }

    close(fd);
    
    if(eErrorCode != NO_ERROR)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_UART_DDAPI_GET_DEVICE | eErrorCode), NULL, NULL);
    }
    return NO_ERROR_MSG;
}

string 
UARTDDAPI::getUartPortType(int  in_s32Type)
{
    UINT32 u32Index;

    for(u32Index = 0; UartPortTypeTable[u32Index].ID != -1; u32Index++)
    {
        if(UartPortTypeTable[u32Index].ID == in_s32Type)
        {
            return UartPortTypeTable[u32Index].strUartPortType;
        }
    }
    return "UNKNOWN";
}

ErrorMsg*
UARTDDAPI::scanUarts(vector<SERIAL_INFO>*   serialList)
{
    FILE*           pFile;
    char            line[64];
    char            uartName[16];
    char            uartNodeName[16];
    char*           pTemp;
    UINT32          u32Index;
    serial_struct   serialSrtuct;
    SERIAL_INFO     tempSerialInfo;
    ErrorMsg*       pErrorMsg;
    char            systemCmd[128];
    
   
    //scan available UARTs from /sys/class/tty and store them to a file 
    sprintf(systemCmd, "ls /sys/class/tty/*/device/driver |grep /sys > %s", UART_LIST_FILE);
    system(systemCmd);

    //get available UART list from file  
    pFile = fopen(UART_LIST_FILE, "r");
    if(!pFile)
    {
        printf("open uartList file fail !!!\n"); //check 
        return returnErrorMsg((ERRORCODE)(ERROR_UART_DDAPI_SCAN_UARTS | ERROR_OPEN_FILE), NULL, NULL);
    }

    u32Index = 0;
    while(fgets(line, sizeof(line), pFile) != NULL)
    {
        pTemp = strtok(line, "/");
        pTemp = strtok(NULL, "/");
        pTemp = strtok(NULL, "/");
        pTemp = strtok(NULL, "/");
        //printf("find uart name - %s: ", pTemp);//check 
        sprintf(uartNodeName, "/dev/%s", pTemp);
        pErrorMsg = getDevice(uartNodeName, &serialSrtuct);    
        if(pErrorMsg == NO_ERROR_MSG)
        {
            //printf("open device successfully\n"); //check 
            sprintf(uartName, "uart%d", u32Index);
            u32Index++;
            tempSerialInfo.strUartName.assign(uartName);
            tempSerialInfo.strUartNodeName.assign(uartNodeName);
            tempSerialInfo.serialInfo = serialSrtuct;
            tempSerialInfo.strType = getUartPortType(serialSrtuct.type);
            serialList->push_back(tempSerialInfo);
        }
    }

    fclose(pFile);
    return NO_ERROR_MSG; 
}


ErrorMsg*
UARTDDAPI::searchUart(STRING            in_uartLogicalName,
                      serial_struct     *out_pSerialInfo)
{
    vector<SERIAL_INFO>             uartList;
    vector<SERIAL_INFO>::iterator   it;


    m_pErrorMsg = scanUarts(&uartList);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_UART_DDAPI_SEARCH_DEVICE, NULL, m_pErrorMsg);
    }

    for(it = uartList.begin(); it != uartList.end(); ++it)
    {
        if(it->strUartName.compare(in_uartLogicalName) == 0)
        {
            *out_pSerialInfo = it->serialInfo;
            return NO_ERROR_MSG;
        }
    }

    return returnErrorMsg((ERRORCODE)(ERROR_UART_DDAPI_SEARCH_DEVICE | ERROR_NO_DEVICE_FOUND), NULL, NULL);
}



ErrorMsg*
UARTDDAPI::readRegister(UINT32  in_u32Offset,
                        UINT8*  out_u8Data)
{
    int ret;
    
    ret = ioperm(m_u32Port + in_u32Offset, 1, 1);
    if(ret < 0)
    {
        return returnErrorMsg(ERROR_UART_DDAPI_READ_REGISTER, NULL, NULL);
    }

    *out_u8Data = inb(m_u32Port + in_u32Offset);

    return NO_ERROR_MSG;
}

ErrorMsg*
UARTDDAPI::writeRegister(UINT32  in_u32Offset,
                         UINT8   in_u8WriteData)
{
    int ret;

    ret = ioperm(m_u32Port + in_u32Offset, 1, 1);
    if(ret < 0)
    {
        return returnErrorMsg(ERROR_UART_DDAPI_WRITE_REGISTER, NULL, NULL);
    }

    outb(in_u8WriteData, m_u32Port + in_u32Offset);

    return NO_ERROR_MSG;
}


ErrorMsg*
UARTDDAPI::testScratchPadRun(UINT8  in_u8WritePattern)
{
    UINT8       u8DataCheck;
    ErrorMsg*   pErrorMsg;

    //write data 
    pErrorMsg = writeRegister(UART_SCR_OFFSET, in_u8WritePattern);
    if(pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_UART_DDAPI_SCRATCHPAD_TEST, NULL, pErrorMsg);
    }

    //check data 
    readRegister(UART_SCR_OFFSET, &u8DataCheck);
    if(u8DataCheck != in_u8WritePattern)
    {
        return returnErrorMsg(ERROR_UART_DDAPI_SCRATCHPAD_TEST, "compare data fail", NULL);
    }
    //printf("UARTDDAPI::testScratchPadRun, data check: 0x%x\n", u8DataCheck);//check 

    return NO_ERROR_MSG;    
}



ErrorMsg*
UARTDDAPI::testScratchPad(STRING   in_uartLogicalName)
{
    UINT8           u8DataSave;
    UINT8           patternArray[4] = {0xAA, 0x55, 0xFF, 0x0};
    UINT32          u32Index;
    serial_struct   uartInfo;
    
    m_pErrorMsg = searchUart(in_uartLogicalName, &uartInfo);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_UART_DDAPI_SCRATCHPAD_TEST, NULL, m_pErrorMsg);
    }

    if(uartInfo.port != 0x2E8 && uartInfo.port != 0x3E8 && 
       uartInfo.port != 0x2F8 && uartInfo.port != 0x3F8)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_UART_DDAPI_SCRATCHPAD_TEST | ERROR_UART_DDAPI_FUNCTION_UNSUPPORTED), NULL, NULL);
    }

    m_u32Port = uartInfo.port;

    m_pErrorMsg = readRegister(UART_SCR_OFFSET, &u8DataSave);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_UART_DDAPI_SCRATCHPAD_TEST, NULL, m_pErrorMsg);
    }
    //printf("UARTDDAPI::testScratchPad, u8DataSave: 0x%x\n", u8DataSave); //check 

    //start to write pattern to SCR and compare data 
    for(u32Index = 0; u32Index < 4; u32Index++)
    {
        m_pErrorMsg = testScratchPadRun(patternArray[u32Index]);
        if(m_pErrorMsg != NO_ERROR_MSG)
        {
            //restore SCR 
            writeRegister(UART_SCR_OFFSET, u8DataSave);
            return m_pErrorMsg;
        }
    }

    //restore SCR 
    writeRegister(UART_SCR_OFFSET, u8DataSave);

    /*
    UINT8   u8Check;
    readRegister(uartInfo.port, UART_SCR_OFFSET, &u8Check);
    printf("UARTDDAPI::testScratchPad, data check (restore): 0x%x\n", u8Check); //check
    */

    return NO_ERROR_MSG;
}


ErrorMsg*
UARTDDAPI::backupRegistersForLoopbackTest()
{
    ErrorMsg*  pErrorMsg;

    //backup LCR
    if((pErrorMsg = readRegister(UART_LCR_OFFSET, &m_u8BackupLCR)) != NO_ERROR_MSG)
    {
        return pErrorMsg;
    }
    //backup MCR 
    if((pErrorMsg = readRegister(UART_MCR_OFFSET, &m_u8BackupMCR)) != NO_ERROR_MSG)
    {
        return pErrorMsg;
    }
    //backup IER 
    if((pErrorMsg = readRegister(UART_IER_OFFSET, &m_u8BackupIER)) != NO_ERROR_MSG)
    {
        return pErrorMsg;
    }

    //check
    //printf("m_u8BackupLCR: 0x%x\n", m_u8BackupLCR);
    //printf("m_u8BackupMCR: 0x%x\n", m_u8BackupMCR);
    //printf("m_u8BackupIER: 0x%x\n", m_u8BackupIER);
    
    return NO_ERROR_MSG;
}


ErrorMsg*
UARTDDAPI::restoreRegisters()
{
    ErrorMsg*   pErrorMsg;

    //restore MCR
    if((pErrorMsg = writeRegister(UART_LCR_OFFSET, m_u8BackupMCR)) != NO_ERROR_MSG)
    {
        return pErrorMsg;
    }

    //restore LCR
    if((pErrorMsg = writeRegister(UART_LCR_OFFSET, m_u8BackupLCR)) != NO_ERROR_MSG)
    {
        return pErrorMsg;
    }

    //restore IER
    pErrorMsg = writeRegister(UART_LSR_OFFSET, m_u8BackupIER);


#if 0
    //check 
    UINT8   u8Check;

    readRegister(UART_LCR_OFFSET, &u8Check);
    printf("LCR (restore): 0x%x\n", u8Check);
    
    readRegister(UART_MCR_OFFSET, &u8Check);
    printf("MCR (restore): 0x%x\n", u8Check);
    
    readRegister(UART_IER_OFFSET, &u8Check);
    printf("IER (restore): 0x%x\n", u8Check);
#endif 
    return pErrorMsg;
}


void
UARTDDAPI::defineLoopbackPatterns(bool       in_userDefinedPattern,
                                  UINT8      in_u8Pattern)
{
    UINT32      u32Index;

    //setup the transmitted pattern buffer
    if(in_userDefinedPattern)
    {
        for(u32Index = 0; u32Index < m_u32BufferSize; u32Index++)
        {
            m_patternBuffer[u32Index] = in_u8Pattern; 
        }
    }
    else 
    {
        //cout << "UARTDDAPI::defineLoopbackPatterns, use default pattern" << endl; //check 
        for(u32Index = 0; u32Index < m_u32BufferSize; u32Index++)
        {
            m_patternBuffer[u32Index] = (u32Index & 0xFF);
        }
    }
}


ErrorMsg*
UARTDDAPI::setupForInternalLoopbackTest()
{
    UINT8       u8Temp;
    ErrorMsg*   pErrorMsg;
    
    //set internal loopback mode: connect TxD pin to RxD pin
    pErrorMsg = writeRegister(UART_MCR_OFFSET, UART_MCR_LOOP);
    if(pErrorMsg != NO_ERROR_MSG)
    {
        return pErrorMsg;
    }

    //set the LCR to 8bit word length(11), No parity(000), 1 stop bit(0)
    if((pErrorMsg = writeRegister(UART_LCR_OFFSET, UART_LCR_WLEN_8)) != NO_ERROR_MSG)
    {
        return pErrorMsg;
    }

    //read LSR register to clear any uart status errors
    if((pErrorMsg = readRegister(UART_LSR_OFFSET, &u8Temp)) != NO_ERROR_MSG)
    {
        return pErrorMsg;
    }

    return NO_ERROR_MSG;
}



ErrorMsg*
UARTDDAPI::doInternalLoopbackTest(STRING     in_uartLogicalName, 
                                  UINT32     in_u32BufferSize,
                                  bool       in_userDefinedPattern,
                                  UINT8      in_u8Pattern)
{
    //cout << "UARTDDAPI::doInternalLoopbackTest - polling" << endl; //check 

    serial_struct   uartInfo;

    if((m_pErrorMsg = searchUart(in_uartLogicalName, &uartInfo))!= NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_UART_DDAPI_LOOPBACK, NULL, m_pErrorMsg);
    }

    if(uartInfo.port != 0x2E8 && uartInfo.port != 0x3E8 && 
       uartInfo.port != 0x2F8 && uartInfo.port != 0x3F8)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_UART_DDAPI_LOOPBACK | ERROR_UART_DDAPI_FUNCTION_UNSUPPORTED), NULL, NULL);
    }

    m_u32Port = uartInfo.port;
    m_u32BufferSize = in_u32BufferSize;

    //backup registers
    if((m_pErrorMsg = backupRegistersForLoopbackTest()) != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_UART_DDAPI_LOOPBACK, NULL, m_pErrorMsg);
    }

    //initialization for internal loopback test 
    if((m_pErrorMsg = setupForInternalLoopbackTest()) != NO_ERROR_MSG)
    {
        restoreRegisters();
        return returnErrorMsg(ERROR_UART_DDAPI_LOOPBACK, NULL, m_pErrorMsg);
    }

    //define patterns 
    m_patternBuffer = (UINT8*)malloc(sizeof(UINT8)*m_u32BufferSize);
    m_receivedBuffer = (UINT8*)malloc(sizeof(UINT8)*m_u32BufferSize);
    defineLoopbackPatterns(in_userDefinedPattern, in_u8Pattern);

    //start to test  
    m_pErrorMsg = doInternalLoopbckTestRun();

    restoreRegisters();
    free(m_patternBuffer);
    free(m_receivedBuffer);
    
    return m_pErrorMsg;
}


ErrorMsg*
UARTDDAPI::waitTHREmpty()
{
    UINT8       u8Data;
    UINT32      u32Index;
    ErrorMsg*   pErrorMsg;

    //check THRE bit, wait 35ms for transmiter to be available
    for(u32Index = 0; u32Index < 3500; u32Index++)
    {
        pErrorMsg = readRegister(UART_LSR_OFFSET, &u8Data);
        if(pErrorMsg != NO_ERROR_MSG)
        {
            return  returnErrorMsg(ERROR_UART_DDAPI_LOOPBACK, NULL, pErrorMsg);
        }
        if((u8Data & UART_LSR_THRE) == UART_LSR_THRE)
        {
            return NO_ERROR_MSG;
        }
        usleep(10);
    }

    return  returnErrorMsg((ERRORCODE)(ERROR_UART_DDAPI_LOOPBACK | ERROR_TIME_OUT), NULL, NULL);
}


ErrorMsg*
UARTDDAPI::waitDataTerminalReady()
{
    UINT32      u32Index;
    UINT8       u8Data;
    ErrorMsg*   pErrorMsg;

    //while(1)
    //wait 35ms 
    for(u32Index = 0; u32Index < 3500; u32Index++)
    {
        pErrorMsg = readRegister(UART_LSR_OFFSET, &u8Data);
        if(pErrorMsg != NO_ERROR_MSG)
        {
            return  returnErrorMsg(ERROR_UART_DDAPI_LOOPBACK, NULL, pErrorMsg);
        }
        if(u8Data & UART_LSR_DR)
        {
            return NO_ERROR_MSG;
        }
        usleep(10);
    }

    return  returnErrorMsg((ERRORCODE)(ERROR_UART_DDAPI_LOOPBACK | ERROR_TIME_OUT), NULL, NULL);
}


ErrorMsg*
UARTDDAPI::doInternalLoopbckTestRun()
{
    UINT32      u32Index;
    ErrorMsg*   pErrorMsg;

    for(u32Index = 0; u32Index < m_u32BufferSize; u32Index++)
    {
        // check THRE bit: wait 35ms for transmiter to be available
        if((pErrorMsg = waitTHREmpty()) != NO_ERROR_MSG)
        {
            return pErrorMsg;
        }
       
        //write data 
        writeRegister(UART_THR_OFFSET, m_patternBuffer[u32Index]);

        //check DR bit
        if((pErrorMsg = waitDataTerminalReady()) != NO_ERROR_MSG)
        {
            printf("check DR bit fail !!!!!!!!!!!!!!!!!!!!!!\n");//check 
            return pErrorMsg;
        }
        
        //read data back
        readRegister(UART_RBR_OFFSET, &m_receivedBuffer[u32Index]);
    }

    //compare data 
    for(u32Index = 0; u32Index < m_u32BufferSize; u32Index++)
    {
        if(m_patternBuffer[u32Index] != m_receivedBuffer[u32Index])
        {
            return returnErrorMsg(ERROR_UART_DDAPI_LOOPBACK, "compare data fail", NULL);
        }
    }
    
    return NO_ERROR_MSG;
}


