#ifndef UARTDDAPI_H
#define UARTDDAPI_H

#include <fcntl.h>
#include <unistd.h>//for close() to be used in c++
#include <sys/io.h>

#include <string>
#include <vector>
#include <termios.h>
#include <linux/serial.h>
#include <sys/ioctl.h>
#include <iostream>

#include "BaseClassDDAPI.hpp"
using namespace std;

#define UART_UNUSED     0xff

#define UART_LIST_FILE  "uartList"

#define UART_THR_OFFSET 0x0
#define UART_RBR_OFFSET 0x0
#define UART_IER_OFFSET 0x1
#define UART_LCR_OFFSET 0x3
#define UART_MCR_OFFSET 0x4
#define UART_LSR_OFFSET 0x5 
#define UART_SCR_OFFSET 0x7

#define UART_MCR_LOOP   0x10
#define UART_LCR_WLEN_8 0x03
#define UART_LSR_THRE   0x20
#define UART_LSR_DR     0x01 //data ready

typedef struct _SERIAL_INFO
{
    string          strUartName;
    string          strUartNodeName;
    string          strType;
    serial_struct   serialInfo;
}SERIAL_INFO;


typedef struct _UART_PORT_TYPE
{
    int     ID;
    string  strUartPortType; 
}UART_PORT_TYPE;


typedef struct _LOOPBACK_REGS
{
    UINT8   u8LCR;
    UINT8   u8MCR;
    UINT8   u8IER;
}LOOPBACK_REGS;


class UARTDDAPI : public BaseClassDDAPI
{
    public:
        UARTDDAPI(STRING);
        ~UARTDDAPI();

        ErrorMsg*   scanUarts(vector<SERIAL_INFO>*   serialList);
        
        ErrorMsg*   getDevice(STRING          in_uartName,
                              serial_struct  *out_pSerialInfo);

        ErrorMsg*   searchUart(STRING            in_uartLogicalName,
                               serial_struct     *out_pSerialInfo);

        ErrorMsg*   readRegister(UINT32  in_u32Offset,
                                 UINT8*  out_u8Data);

        ErrorMsg*   writeRegister(UINT32  in_u32Offset,
                                  UINT8   in_u8WriteData);

        ErrorMsg*   testScratchPad(STRING   in_uartLogicalName);

        ErrorMsg*   doInternalLoopbackTest(STRING     in_uartLogicalName, 
                                           UINT32     in_u32BufferSize,
                                           bool       in_userDefinedPattern,
                                           UINT8      in_u8Pattern);

    private:
        ErrorMsg*       m_pErrorMsg;
        LOOPBACK_REGS   m_loopbackRegs;

        UINT8   m_u8BackupLCR;
        UINT8   m_u8BackupMCR;
        UINT8   m_u8BackupIER;
        UINT8*  m_patternBuffer;
        UINT8*  m_receivedBuffer;
        UINT32  m_u32BufferSize;
        UINT32  m_u32Port;

        string  getUartPortType(int  in_s32Type);

        ErrorMsg*   testScratchPadRun(UINT8  in_u8WritePattern);

        ErrorMsg*   restoreRegisters();
        ErrorMsg*   backupRegistersForLoopbackTest();
        ErrorMsg*   setupForInternalLoopbackTest();
        ErrorMsg*   doInternalLoopbckTestRun();
        ErrorMsg*   waitTHREmpty();
        ErrorMsg*   waitDataTerminalReady();

        void    defineLoopbackPatterns(bool       in_userDefinedPattern,
                                       UINT8      in_u8Pattern);

};



#endif
